# 
# build-by-patching.sh - get and apply the next patch                      
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents
#                          of the University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
          printf "get and apply a patch to a source tree\\n"
          printf "usage: build-by-patching [options] revision\\n"
          printf "\\n"
          printf " -V --version                  print version info\\n"
          printf " -h --help                     display help\\n"
          printf "\\n"
          printf " --silent                      no output\\n"
          printf " --quiet                       only output errors\\n"
          printf " --default-output              default output\\n"
          printf " --report                      slightly verbose output\\n"
          printf " --verbose                     maximal output\\n"
          printf " --debug                       debugging output\\n"
          printf "\\n"
          printf "Read the patch set for REVISION from the attached archive.\\n"
          printf "Apply that patch set to the source directory in the current\\n"
          printf "directory.\\n"
          printf "\\n"
          exit 0
          ;;

      *)
          ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=

while test $# -ne 0 ; do

  case "$1" in 


    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "build-by-patching: debugging output enabled\\n"
                    set -x
                debug_opt=--debug
                ;;

    --)                        shift
                        break
                        ;;

    -*)                        printf "build-by-patching: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                        break
                            ;;

  esac

done


################################################################
# Ordinary Arguments
# 

if test $# -ne 1 ; then
  printf "usage: build-by-patching [options] revision\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

revision="$1"
shift

################################################################
# Sanity Check and Process Defaults
# 

source="`pwd`"

arx valid-package-name -l -e build-by-patching -- "$revision"

category=`arx parse-package-name -b $revision`
branch=`arx parse-package-name $revision`
vsn=`arx parse-package-name -v $revision`
lvl=`arx parse-package-name -l $revision`

################################################################
# Greetings
# 
# This is an internal command.
# 

if test $verbosity -ge $report ; then
  ARCH__OUTLINE_DEPTH="$ARCH__OUTLINE_DEPTH*"
  arx heading "build-by-patching\\n"
  printf "arguments: %s\\n" "$command_line" | fold -w 60 | arx body-indent 
  if test $verbosity -ge $verbose ; then
    arx heading --sub "build-by-patching start time: %s\\n" "`date`"
  fi
fi

if test $verbosity -ge $verbose ; then
  arx heading --sub "patching for revision %s\\n" $branch--$vsn--$lvl
fi


################################################################
# Obtain the patch
# 

wftp-home
wftp-cd $category/$branch/$branch--$vsn/$lvl

here="`pwd`"

bail()
{
  rm -rf "$here/,,patch.$branch--$vsn--$lvl"
  if test $verbosity -ge $verbose ; then
    arx heading --sub "build-by-patching finish time: %s\\n" "`date`"
  fi
  exit 1
}

trap "printf \"build-by-patching: interrupted -- cleaning up\\n\" 1>&2 ; bail" INT

rm -rf ,,patch.$branch--$vsn--$lvl
mkdir ,,patch.$branch--$vsn--$lvl
cd ,,patch.$branch--$vsn--$lvl

if test $verbosity -ge $verbose ; then
  arx heading --sub "start retrieving patch from archive: %s\\n" "`date`"
fi

if ! wftp-get $branch--$vsn--$lvl.patches.tar.gz > $branch--$vsn--$lvl.patches.tar.gz ; then
  rm -f $branch--$vsn--$lvl.patches.tar.gz
  cd .. 
  rmdir ,,patch.$branch--$vsn--$lvl
  printf "\\n" 1>&2
  printf "build-by-patching: unable to retrieve patch\\n" 1>&2
  printf "  patch: $branch--%s--%s.patches.tar.gz\\n" "$vsn" "$lvl" 1>&2
  printf "\\n" 1>&2
  bail
fi

if test $verbosity -ge $verbose ; then
  arx heading --sub "finish retrieving patch from archive: %s\\n" "`date`"
fi

$ARCH_TAR -m -zxf $branch--$vsn--$lvl.patches.tar.gz 

if ! arx nested arx dopatch $debug_opt --quiet $verbose_opt --delete-removed $branch--$vsn--$lvl.patches "$source" ; then
  printf "\\n" 1>&2
  printf "build-by-patching: INTERNAL ERROR! patch did not apply cleanly!\n" 1>&2
  printf "\\n" 1>&2
  bail
fi


cd ..
rm -rf ,,patch.$branch--$vsn--$lvl

if test $verbosity -ge $verbose ; then
  arx heading --sub "build-by-patching finish time: %s\\n" "`date`"
fi
