# build-revision.sh - build a revision from an archive and cache
# 
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents
#                          of the University of California
# Minor bug fix 2003 Chris Gray
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#       --version | -V
#       --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "*build a revision from a connected database and local cache\\n"
                printf "usage: build-revision [options] revision\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " --cache dir                   specify where cached revisions are\\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf "See \"arx get --help\".\\n"
                printf "\\n"
                exit 0
                ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=
cache=

silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=
__restart=

while test $# -ne 0 ; do

  case "$1" in 

    ----restart) shift
                     __restart=----restart
                 ;;

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "build-revision: debugging output enabled\\n"
                    set -x
                debug_opt=--debug
                ;;

    --cache)               shift
                        if test $# -eq 0 ; then
                          printf "build-revision: --cache requires an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        cache="$1"
                        shift
                        ;;

    --)                        shift
                            break
                        ;;

    -*)                 printf "build-revision: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                  break
                        ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 1 ; then
  printf "usage: build-revision [options] revision\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

revision="$1"
arx valid-package-name -e build-revision -l -- "$revision"

archive="`arx parse-package-name --arch "$revision"`"
spec="`arx parse-package-name --non-arch "$revision"`"
category="`arx parse-package-name --basename \"$revision\"`"
branch="`arx parse-package-name --package \"$revision\"`"

################################################################
# Greetings
# 

if test $verbosity -ge $report; then
    if test -z "$__restart" -o $verbosity -ge $verbose ; then
        arx heading "build-revision\\n"
        printf "arguments: %s\\n" "$command_line" | fold -w 60 | arx body-indent 
        if test $verbosity -ge $verbose; then
          arx heading --sub "build-revision start time: %s\\n" "`date`"
        fi
    fi
fi

################################################################
# Ensure an Archive Connection
# 

if test "$WITHARCHIVE" != "$archive" ; then

  cd "$parent"

  if test $verbosity -ge $verbose; then
    arx heading --sub "restarting with connection to archive\\n"
  fi

  if test -n "$dirname"; then
      echo
      echo "dirname is" $dirname "but it should be empty."
      echo
      echo "aborting."
      exit 1
  else
      exec arx with-archive -A "$archive" \
          arx build-revision --cache "$cache" \
          $verbose_opt $debug_opt \
          ----restart \
          "$archive/$spec"
  fi

fi

################################################################
# What Revision is Intended?
# 

revision="`arx indicated-revision -e build-revision \"$archive/$spec\"`"
version="`arx parse-package-name --package-version \"$revision\"`"
lvl="`arx parse-package-name --patch-level \"$revision\"`"


################################################################
# Build It
# 
# 
# We start at the goal revision and search patch levels backwards
# until we find:
# 
#         1. A revision in the local cache
#        2. A revision with a cached source tree in the archive
#        3. A fresh-start base revision
# 
# Then we move forward through the missing patch levels, 
# applying patches, until the final goal revision is reached.
# 

top()
{
  if test $# = 0 ; then
    printf "stack underflow\\n" 1>&2
    exit 1
  else
    printf "%s\\n" "$1" 
  fi
}


pop()
{
  if test $# = 0 ; then
    printf "stack underflow\\n" 1>&2
    exit 1
  else
    shift
    printf "%s\\n" "$*" 
  fi
}

goal="$revision"
goal_stack=

while true ; do

  if test $verbosity -ge $report; then
    arx heading --sub "revision goal: %s\\n" "$goal"
    arx heading --sub "goal stack: %s\\n" "$goal_stack"
  fi

  if    arx nested arx copy-from-library \
                                           --silent \
                                	   -- "$goal" \
     || arx nested arx copy-from-cache \
                                         --silent \
                                	-- "$cache" "$goal" \
     || arx nested arx copy-from-archive-cache --silent \
                                	         "$goal" \
     || arx nested arx copy-from-fresh-start-base --silent \
                                		    "$goal" ; then

    break

  else

    prev=`arx previous-patch-level "$goal"`

    prev_archive="`arx parse-package-name --arch \"$prev\"`"
    prev_revision="`arx parse-package-name --non-arch \"$prev\"`"

    goal_stack="$goal $goal_stack"
    goal="$prev_archive/$prev_revision"

    if test "$prev_archive" = "$prev" -o "$prev_archive" = "$archive" ; then

      continue

    else

      arx with-archive -A "$prev_archive"  \
        arx nested arx build-revision $verbose_opt $debug_opt \
                                  	--cache "$cache" \
                                	"$prev_archive/$prev_revision"
      break

    fi

  fi

done

while test "x$goal_stack" != x ; do

  goal=`top $goal_stack`
  goal_stack=`pop $goal_stack`

  arx nested arx build-by-patching $verbose_opt $debug_opt "$goal"
done


if test $verbosity -ge $verbose; then
  arx heading --sub "build-revision finish time: %s\\n" "`date`"
fi
