# 
# commit.sh: archive a pre-patch revision
################################################################
# Copyright (C) 2001, 2002, 2003 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents
#                          of the University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#       --version | -V
#       --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;

      --help|-h)
                printf "archive a revision\\n"
                printf "usage: commit [options] [[archive/]version]\\n"
                printf "usage: commit --files [options] [[archive/]version] -- file ...\\n"
                printf "usage: commit --file-list FILE-LIST [options] [[archive/]version]\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " -L --log-file file            specify the log-file\\n"
                printf " --not-default                 do not make this the default branch\\n"
                printf " --no-lint                     do not tree-lint the project tree\\n"
                printf " --strict                      require the tree to pass a strict tree-lint\\n"
                printf "\\n"
                printf " -d --dir DIR                  cd to DIR first\\n"
                printf "\\n"
                printf " --seal                        commit the version revision\\n"
                printf " --fix                         commit a versionfix- patch level\\n"
                printf " --continuation [archive/]prev-revision  commit a continuation revision\\n"
                printf " --out-of-date-ok              permit commit even if project tree is\\n"
                printf "                                 not up to date\\n"
                printf " --unchanged-ok                commit even if project tree is\\n"
                printf "                                 unmodified\\n"
                printf "\\n"
                printf " --dry-run                     don't actually commit -- just\\n"
                printf "                               prepare the patch set\\n"
                printf "\\n"
                printf " --files                       (note syntax above) restrict commit to\\n"
                printf "                                 selected files from the command line\\n"
                printf " --file-list FILE-LIST         restrict commit to the files named\\n"
                printf "                                 in FILE-LIST (one per line)\\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf "Archive a revision of the project tree containing DIR (or the\\n"
                printf "current directory)\\n"
                printf "\\n"
                printf "Unless commiting a new branch or version, no commit will occur\\n"
                printf "if the only changes to the project tree are the new log entry.\\n"
                printf "To force such a commit to occur anyway,\\n"
                printf "use \"--unchanged-ok\".\\n"
                printf "\\n"
                printf "If --files or --file-list is used, the indicated files must exist in\\n"
                printf "both the project tree and the immediate ancestor revision (though they\\n"
                printf "may have been renamed).  The resulting commit will include file differences\\n"
                printf "only -- not tree rearrangements.\\n"
                printf "\\n"
                printf "If the file specified to --file-list is -, the list is read from standard\\n"
                printf "input.\\n"
                printf "\\n"
                exit 0
                ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=

linting=
makedefault=1
logfile=
continuation=0
continuation_revision=
fix=0
seal=0
out_of_date_ok=0
dry_run=
unchanged_ok=
__restart=

silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=

selected_files=
file_list=

dir=.

while test $# -ne 0 ; do

  case "$1" in 

    -d|--dir)           shift
                        if test $# -eq 0 ; then
                          printf "commit: -d and --dir require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        dir="$1"
                        shift
                        ;;

     --file-list)	shift
     			if test $# -eq 0 ; then
                          printf "commit: --file-list requires an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        file_list="$1"
                        shift
                        selected_files=yes
                        ;;

     --files)		shift
                        selected_files=--files
                        file_list=
                        ;;

     ----restart) shift
                 __restart=----restart
                 ;;

     --no-lint)  shift
                 linting=--no-lint
                 ;;

     --strict)  shift
                 linting=--strict
                 ;;

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)    shift
                arx heading "commit: debugging output enabled\\n"
                set -x
                debug_opt=--debug
                ;;


    -L|--log-file)      shift
                        if test $# -eq 0 ; then
                          printf "commit: -L and --log-file require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        logfile="$1"
                        shift
                        ;;

    --not-default)      shift
                        makedefault=0
                        ;;

    --fix)              shift
                        fix=1
                        seal=0
                        continuation=0
                        ;;

    --seal)             shift
                        fix=0
                        seal=1
                        continuation=0
                        unchanged_ok=--unchanged-ok
                        ;;

    --continuation)     shift
                        fix=0
                        seal=0
                        continuation=1
                        unchanged_ok=--unchanged-ok
                        if test $# -eq 0 ; then
                          printf "commit: --continuation requires an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        continuation_revision="$1"
                        shift
                        ;;

    --out-of-date-ok)   shift
                        out_of_date_ok=1
                        ;;

    --unchanged-ok)     shift
                        unchanged_ok=--unchanged-ok
                        ;;

    --dry-run)          shift
                        dry_run=--dry-run
                        ;;

    --)			break
                        ;;

    -*)                 printf "commit: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                  break
                        ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test -z "$selected_files" -a '(' $# -gt 1 ')' ; then
  printf "usage: commit [options] [[archive]/version]\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

if test '(' $# -gt 0 ')' -a '(' "$1" != -- ')' ; then
  archive_version="$1"
  shift
else
  archive_version=
fi

if test ! -z "$selected_files" ; then

  if test -z "$file_list" ; then

    if test '(' $# -eq 2 ')' -o '(' "$1" = -- ')' ; then
      shift
    else
      printf "usage: commit --files [options] [[archive]/version] -- file ...\\n" 1>&2
      printf "try --help\\n" 1>&2
      exit 1
    fi
  else
    
    if test $# -ne 0 ; then
      printf "usage: commit --file-list FILE [options] [[archive]/version]\\n" 1>&2
      printf "try --help\\n" 1>&2
      exit 1
    fi
  fi

fi


################################################################
# Sanity Check and Process Defaults
# 
  
start_dir="`pwd`"

cd "$dir"
dir="`pwd`"

wdroot="`arx tree-root --accurate`"
cd "$wdroot"

# If doing an initial import, make sure that the archive version is
# not specified.
if test -f "$wdroot/{arch}/++import" ; then
  if ! test -z "$archive_version" ; then
    if test "$archive_version" != `cat $wdroot/{arch}/++import` ; then
      printf "You may not specify an archive version.  It is already\\n" 1>&2
      printf "specified in $wdroot/{arch}/++import\\n" 1>&2
      exit 1
    fi
  fi
  archive_version=`cat $wdroot/{arch}/++import`
fi

# If a branch or a new version is being commit'ed, make sure the
# old-branch is not specified.  Also, set the old_revision and the
# continuation flag.

if test 1 -ne $continuation; then
  if test -f "$wdroot/{arch}/++branch-revision" ; then
    if ! test -z "$continuation_revision" ; then
      if test "$continuation_revision" != `cat $wdroot/{arch}/++branch-revision` ; then
        printf "The continuation revision you specified is not the same as what\\n" 1>&2
        printf "is specified in $wdroot/{arch}/++branch-revision\\n" 1>&2
        exit 1
      fi
    fi
    continuation_revision=`cat $wdroot/{arch}/++branch-revision`
    fix=0
    seal=0
    continuation=1
    unchanged_ok=--unchanged-ok
  fi
fi

if test "x$archive_version" = x ; then
  archive_version=`arx tree-version`
else
  arx valid-package-name -e commit --vsn -- "$archive_version"
fi

archive=`arx parse-package-name --arch $archive_version`
version=`arx parse-package-name --package-version $archive_version`
category=`arx parse-package-name --basename $version`
branch=`arx parse-package-name $version`
vsn=`arx parse-package-name -v $version`

if test "x$logfile" = x ; then
  logfile="$wdroot/++log.$version--$archive"
fi

cd "$start_dir"
logfile_dir="`dirname \"$logfile\"`"
logfile_base="`basename \"$logfile\"`"
cd "$logfile_dir"
logfile="`pwd`/$logfile_base"

# Make sure the wd has a patch log for this version.
# 
arx wd-check-for-patch-log -e commit "$archive/$version" "$wdroot"

# Make sure we have a valid log file for this revision.
# 
arx valid-log-file -e commit -- "$logfile"

tmpdir="$wdroot/,,commit-tmp-dir"



################################################################
# Make Sure that the Remote Site is not a Mirror
# 

mirror="`arx with-archive -A \"$archive\" wftp-get \"=meta-info/mirror\" 2> /dev/null || true`"

if test "$mirror" ; then
  printf "\\n" 1>&2
  printf "commit: cannot commit -- the target archive is a mirror!\\n" 1>&2
  printf "\\n" 1>&2
  exit 1
fi


################################################################
# Greetings
# 

if test "(" -z "$__restart" -a $verbosity -ge $report ")" -o $verbosity -ge $verbose ; then
  arx heading "commit\\n"
  printf "arguments: %s\\n"  "$command_line" | fold -w 60 | arx body-indent
  if test $verbosity -ge $verbose ; then
    arx heading --sub "commit start time: %s\\n" "`date`"
    arx heading --sub "project tree: %s\\n" "$wdroot"
    arx heading --sub "archive: %s\\n" "$archive"
    arx heading --sub "version: %s\\n" "$version"
    if test ! -z "$continuation_revision" ; then
      arx heading --sub "this is a continuation revision (--continuation)\\n"
    elif test $fix -ne 0 ; then
      arx heading --sub "this is versionfix revision (--fix)\\n"
    elif test $seal -ne 0 ; then
      arx heading --sub "this is version revision (--seal)\\n"
    fi
  fi
  if test ! -z "$dry_run" ; then 
    arx heading --sub "this is a dry run (no modifications to the archive)\\n"
  fi
fi

################################################################
# Ensure that We Have an Archive Connection 
# 

if test "$WITHARCHIVE" != "$archive" ; then

  if test $verbosity -ge $verbose ; then
    arx heading --sub "restarting with connection to archive\\n"
  fi

  flags=

  if test $makedefault -eq 0 ; then
    flags="-d $flags"
  fi

  if test $fix -ne 0 ; then
    flags="--fix $flags"
  elif test $seal -ne 0 ; then
    flags="--seal $flags"
  elif test $continuation -ne 0 ; then
    flags="--continuation $continuation_revision $flags"
  fi

  if test $out_of_date_ok -ne 0 ; then
    flags="--out-of-date-ok $flags"
  fi

  cd "$start_dir"

  if test -z "$file_list" ; then

    if test ! -z "$selected_files" ; then
      dashdash="--"
    else
      dashdash=
    fi

    exec arx with-archive -A "$archive" \
        arx commit \
                    -L "$logfile" \
                    $verbose_opt $debug_opt \
                    $unchanged_ok \
                    $linting \
                    $flags \
                    $dry_run \
                    $selected_files \
                    --dir "$dir" \
                    ----restart \
                    "$archive/$version" \
                    $dashdash \
                    "$@"

  else
    exec arx with-archive -A "$archive" \
        arx commit \
                    -L "$logfile" \
                    --file-list "$file_list" \
                    $verbose_opt $debug_opt \
                    $unchanged_ok \
                    $linting \
                    $flags \
                    $dry_run \
                    --dir "$dir" \
                    ----restart \
                    "$archive/$version"
  fi
fi

# Make all of the necessary categories, branches, and versions.  Not
# very transactional.  Ick.

wftp-home
if ! wftp-cd "$category" 2> /dev/null ; then
   arx make-category "$archive/$category"
fi

wftp-home
if ! wftp-cd "$category/$branch" 2> /dev/null ; then
  arx make-branch "$archive/$branch"
fi

wftp-home
if ! wftp-cd "$category/$branch/$version" 2> /dev/null ; then
  arx make-version "$archive/$version"
fi

# If doing an initial import, do that and exit
if test -f "$wdroot/{arch}/++import" ; then
  arx import $verbose_opt $debug_opt -L "$logfile" "$archive_version"
  rm -f "$wdroot/{arch}/++import"
  exit 0
fi


################################################################
# Create a Temp Dir in Which to Build the Patch
# 

bail()
{
  cd "$wdroot"
  rm -rf "$tmpdir"
  exit 1
}

trap "printf \"commit: interrupted -- cleaning up\\n\" 1>&2 ; bail" INT

if test -d "$tmpdir" ; then
  rm -rf "$tmpdir"
fi
mkdir "$tmpdir"


################################################################
# Process Defaults for the Continuation Revision
# 

first()
{
  if test $# = 0 ; then
    printf "first called on empty list\\n" 1>&2
    bail
  else
    printf "%s\\n" "$1" 
  fi
}

second()
{
  if test $# -lt 2 ; then
    printf "first called on short list\\n" 1>&2
    bail
  else
    printf "%s\\n" "$2" 
  fi
}

if test ! -z "$continuation_revision" ; then
  
  continuation_archive="`arx parse-package-name --arch \"$continuation_revision\"`"
  continuation_non_archive="`arx parse-package-name --non-arch \"$continuation_revision\"`"

  indicated_continuation="`arx indicated-revision -e commit \"$continuation_archive/$continuation_non_archive\"`"

  continuation_revision="`arx parse-package-name --non-arch \"$indicated_continuation\"`"

  continuation_category=`arx parse-package-name --basename $continuation_revision`
  continuation_branch=`arx parse-package-name $continuation_revision`
  continuation_vsn=`arx parse-package-name --vsn $continuation_revision`
  continuation_lvl=`arx parse-package-name --patch-level $continuation_revision`

fi



################################################################
# What Revision Comes Next?
# 


preclude_fix()
{
  if test $fix -ne 0 ; then
    printf "commit: too early for versionfix patches\\n" 1>&2
    printf "\\n" 1>&2
    printf "  version-0 patch level doesn't exist yet.\\n" 1>&2
    printf "\\n" 1>&2
    printf "Try \"--seal\" (see \"arx commit --help\").\\n" 1>&2
    printf "\\n" 1>&2
    bail
  fi
}

all_levels="`arx revisions --reverse \"$archive/$version\"`"

if test "x$all_levels" = x ; then

  if test $continuation -ne 0 ; then
    lvl=base-0
  else
    printf "commit: version has no revisions yet\\n" 1>&2
    printf "\\n" 1>&2
    printf "  version: %s\\n" $version 1>&2
    printf "\\n" 1>&2
    printf "Try \"arx import --help\" or \"commit --continuation\" instead.\\n" 1>&2
    printf "\\n" 1>&2
    bail
  fi

else

  latest_lvl=`first $all_levels`

  case "$latest_lvl" in
    base-0)             preclude_fix
                        if test $seal -eq 0 ; then
                          lvl=patch-1
                        else
                          lvl=version-0
                        fi
                        ;;

    patch-*)            preclude_fix
                        if test $seal -eq 0 ; then
                          lvl="patch-$((${latest_lvl#patch-} + 1))"
                        else
                          lvl=version-0
                        fi
                        ;;

    version-0|versionfix-*)

                        lvl="versionfix-$((${latest_lvl##*-} + 1))"
                        if test $fix -eq 0 ; then
                          printf "commit: too late for ordinary patches\\n" 1>&2
                          printf "\\n" 1>&2
                          printf "  version-0 already exists.\\n" 1>&2
                          printf "\\n" 1>&2
                          printf "Try \"--fix\" (see \"arx commit --help\").\\n" 1>&2
                          printf "\\n" 1>&2
                          bail
                        fi
                        ;;

    
  esac

fi


# Conditions
#
#   lvl set to the patch level name for the revision 
#     we should create
# 
#   latest_lvl set to the patch level name of the previous revision 
# 
################################################################
# Is the Working Dir Sufficiently Up to Date
# 
# 
#

if test $out_of_date_ok -eq 0 ; then

  cd "$wdroot"

  missing=

  for rev in $all_levels ; do
    if test ! -e "{arch}/$category/$branch/$version/$archive/patch-log/$rev" ; then
      missing="$rev $missing"
    fi
  done

  if test "x$missing" != x ; then
    printf "commit: project tree out of date\\n" 1>&2
    printf "\\n" 1>&2
    printf "  Working directory is missing these patches:\n" 1>&2
    printf "\\n" 1>&2
    for m in $missing ; do
      printf "\t%s\\n" $m 1>&2
    done
    printf "\\n" 1>&2
    printf "  Unable to complete \"commit\".\\n" 1>&2
    printf "\\n" 1>&2
    printf "Try \"arx update --help\" or \"arx commit --out-of-date-ok\"\\n" 1>&2
    printf "\\n" 1>&2
    bail 
  fi

fi


################################################################
# Lint the Source Tree
# 

if test "$linting" != "--no-lint" ; then
  cd "$wdroot"

  arx nested arx tree-lint --quiet $linting $verbose_opt $debug_opt
fi



################################################################
# Get a Pristine Copy of the Previous Revision to Compare With
# 

cd "$tmpdir"

if test $verbosity -ge $report ; then
  arx heading --sub "finding (or making) previous revision for comparison\\n"
  if test $verbosity -ge $verbose ; then
    arx heading --sub --sub "start time: %s\\n" "`date`"
  fi
fi

if test $continuation -eq 0 ; then

  prev_archive="$archive"
  prev_category="$category"
  prev_branch="$branch"
  prev_vsn="$vsn"
  prev_lvl="$latest_lvl"

else

  prev_archive="$continuation_archive"
  prev_category="$continuation_category"
  prev_branch="$continuation_branch"
  prev_vsn="$continuation_vsn"
  prev_lvl="$continuation_lvl"

fi

if test -e "$wdroot/{arch}/++pristine-trees/locked/$prev_category/$prev_branch/$prev_branch--$prev_vsn/$prev_archive/$prev_branch--$prev_vsn--$prev_lvl" ; then

  prev_tree="$wdroot/{arch}/++pristine-trees/locked/$prev_category/$prev_branch/$prev_branch--$prev_vsn/$prev_archive/$prev_branch--$prev_vsn--$prev_lvl"

# Remove ++cached-index if it is there.
  rm -f "$prev_tree/++cached-index"

elif test -e "$wdroot/{arch}/++pristine-trees/unlocked/$prev_category/$prev_branch/$prev_branch--$prev_vsn/$prev_archive/$prev_branch--$prev_vsn--$prev_lvl" ; then

  prev_tree="$wdroot/{arch}/++pristine-trees/unlocked/$prev_category/$prev_branch/$prev_branch--$prev_vsn/$prev_archive/$prev_branch--$prev_vsn--$prev_lvl"

# Remove ++cached-index if it is there.
  rm -f "$prev_tree/++cached-index"

elif arx library-find --silent $verbose_opt "$prev_archive/$prev_branch--$prev_vsn--$prev_lvl" ; then

  prev_tree="`arx library-find \"$prev_archive/$prev_branch--$prev_vsn--$prev_lvl\"`"

else

  mkdir "$tmpdir/,,prev_tree"
  cd  "$tmpdir/,,prev_tree"

  if ! arx nested \
           arx build-revision \
                               --quiet $verbose_opt $debug_opt \
                               --cache "$wdroot/.." "$prev_archive/$prev_branch--$prev_vsn--$prev_lvl" ; then
    bail
  fi

  cd "$tmpdir"
  arx wd-mv-pristine "$prev_archive" "$prev_branch--$prev_vsn--$prev_lvl" ,,prev_tree "$wdroot"
  prev_tree="$wdroot/{arch}/++pristine-trees/unlocked/$prev_category/$prev_branch/$prev_branch--$prev_vsn/$prev_archive/$prev_branch--$prev_vsn--$prev_lvl"

fi

if test $verbosity -ge $verbose ; then
  arx heading --sub --sub "finish time: %s\\n" "`date`"
fi

# Conditions
#
#   prev_tree points to a pristine tree for the previous revision
# 
################################################################
# Make a Patch Set Between The two Trees
# 

if test $verbosity -ge $report ; then
  arx heading --sub "making patch set\\n" 
fi

cd "$tmpdir"

if test -z "$file_list" -a -z "$selected_files" ; then

  arx nested \
    arx mkpatch --quiet $verbose_opt $debug_opt "$prev_tree" "$wdroot" ,,patch-set

else

  if test -z "$file_list" ; then

    arx nested \
      arx mkpatch-files --file-root "$start_dir" --quiet $verbose_opt $debug_opt "$prev_tree" "$wdroot" ,,patch-set "$@"

  else

    # --file-list case
    # 

    cat "$file_list" \
    | arx nested \
        arx mkpatch-files --file-list -  --file-root "$start_dir" --quiet $verbose_opt $debug_opt "$prev_tree" "$wdroot" ,,patch-set

  fi
fi

  

################################################################
# Modify the Patch Set to Include the Wrong Log File
# 

arx patch-add-log ,,patch-set "$archive/$version--$lvl" "$logfile"


################################################################
# Compute the Changed File Sets
# 
# 

cd "$wdroot"
touch "$tmpdir/,,log-ls"
for v in `arx logs` ; do
  for p in `arx log-ls $v` ; do
    printf "%s--%s\\n" $v $p >> "$tmpdir/,,log-ls"
  done
done
printf "%s/%s\\n" "$archive" "$version--$lvl" >> "$tmpdir/,,log-ls"

cd "$prev_tree"
touch "$tmpdir/,,prev-patches"
for v in `arx logs` ; do
  for p in `arx log-ls $v` ; do
    printf "%s--%s\\n" $v $p  >> "$tmpdir/,,prev-patches"
  done
done

cd "$tmpdir"

rm -f ,,tmp
mv ,,log-ls ,,tmp
sort ,,tmp > ,,log-ls

rm -f ,,tmp
mv ,,prev-patches ,,tmp
sort ,,tmp > ,,prev-patches
rm -f ,,tmp

comm -2 -3 ,,prev-patches ,,log-ls > ,,prev-only-patches
comm -1 -3 ,,prev-patches ,,log-ls > ,,wd-only-patches

cd "$tmpdir/,,patch-set/patches"
find . -type f \
| grep -E -e '\.(link-mod|modified|patch|meta-mod)$' \
| sed -e "s/\.[^.]*$//" \
| sort -u \
> "$tmpdir/,,modified-files"

cd "$tmpdir/,,patch-set/patches"
find . -type f -name "=dir-meta-mod" \
| xargs -n 1 need-args dirname \
| sort \
> "$tmpdir/,,modified-dirs"


################################################################
# Has Anything Really Changed?
# 
# 

cd "$tmpdir"

( printf "%s\\n" "./{arch}" ; \
  printf "%s\\n" "./{arch}/$category" ; \
  printf "%s\\n" "./{arch}/$category/$branch" ; \
  printf "%s\\n" "./{arch}/$category/$branch/$version" ; \
  printf "%s\\n" "./{arch}/$category/$branch/$version/$archive" ; \
  printf "%s\\n" "./{arch}/$category/$branch/$version/$archive/patch-log" ) \
| sed -e '{
            s,[/$*[\\],\\&,g
            s,^,/^,
            s,$,$/d,
          }' \
> ,,ignored-dirs-sed-script

if test  -z "$unchanged_ok" \
         -a -z "`cat ,,prev-only-patches ,,modified-dirs | head -1`" \
         -a  -z "`cat ,,modified-files | head -1`" \
         -a "`cat ,,wd-only-patches`" = "$archive/$version--$lvl" \
         -a "`join -v 2 -o 2.1 -1 2 -2 2 ,,patch-set/orig-files-index ,,patch-set/mod-files-index`" = "./{arch}/$category/$branch/$version/$archive/patch-log/$lvl" \
         -a  -z "`join -v 1 -o 1.1 -1 2 -2 2 ,,patch-set/orig-files-index ,,patch-set/mod-files-index | head -1`" \
         -a  -z "`join -v 2 -o 2.1 -1 2 -2 2 ,,patch-set/orig-dirs-index ,,patch-set/mod-dirs-index | sed -f ,,ignored-dirs-sed-script | head -1`" \
         -a  -z "`join -v 1 -o 1.1 -1 2 -2 2 ,,patch-set/orig-dirs-index ,,patch-set/mod-dirs-index | head -1`" \
   && join -o 1.1,2.1 -1 2 -2 2 ,,patch-set/orig-files-index ,,patch-set/mod-files-index | $ARCH_AWK '($1 != $2) { exit(1); }' \
   && join -o 1.1,2.1 -1 2 -2 2 ,,patch-set/orig-dirs-index ,,patch-set/mod-dirs-index | $ARCH_AWK '($1 != $2) { exit(1); }' \
 ; then

  # nothing changed -- without the --unchanged-ok option
  # 
  if test $verbosity -ge $quiet ; then
    arx heading --sub "project tree unchanged except for log entries\\n"
    printf "no commit will take place (use --unchanged-ok to override)\\n" | arx body-indent --sub
    if test $verbosity -ge $verbose ; then
      arx heading --sub "commit finish time: %s\\n" "`date`"
      arx heading --sub "commit exit status: 0\\n"
    fi
  fi

  cd "$wdroot"
  rm -rf "$tmpdir"
  exit 0
fi

################################################################
# Fix Up the Log File
# 

sort_fqrn_list()
{
  sed -e 's+\(.*\)/\(.*\)--\(.*\)--\([0-9]*\)\.\([0-9]*\)--\(.*\)-\([0-9]*\)+\1,\2,\3,\4,\5,\6,\7+' \
      -e 's+\(.*\)/\(.*\)--\([0-9]*\)\.\([0-9]*\)--\(.*\)-\([0-9]*\)+\1,\2,,\3,\4,\5,\6+' \
  | sort -u -t, -k 1,1 -k2,2 -k3,3 -k4,4n -k5,5n -k6,6 -k7,7n \
  | sed -e 's+^\(.*\),\(.*\),,\(.*\),\(.*\),\(.*\),\(.*\)+\1/\2--\3.\4--\5-\6+' \
        -e 's+^\(.*\),\(.*\),\(.*\),\(.*\),\(.*\),\(.*\),\(.*\)+\1/\2--\3--\4.\5--\6-\7+' 
}


if test $verbosity -ge $report ; then
  arx heading --sub "mangling the log file\\n"
fi

cd "$wdroot"
cp "$logfile" "$tmpdir/,,raw-log"

cd "$tmpdir"

(   printf "Revision: %s\\n" $version--$lvl \
 && printf "Archive: %s\\n" $archive \
 && printf "Creator: %s\\n" "`arx my-id`" \
 && printf "Date: %s\\n" "`date`" \
 && printf "Standard-date: %s\\n" "`date -u \"+%Y-%m-%d %H:%M:%S GMT\"`" \
 && sed -e "/^\$/,\$d" ,,raw-log \
 && (  join -v 2 -o 2.1 -1 2 -2 2 ,,patch-set/orig-files-index ,,patch-set/mod-files-index \
     | sed -e 's,^\./,,' \
     | unfold -w 60 \
     | sed -e "1s/^/New-files: /" -e "1!s/^/  /" ) \
 && (  join -v 2 -o 2.1 -1 2 -2 2 ,,patch-set/orig-dirs-index ,,patch-set/mod-dirs-index \
     | sed -e 's,^\./,,' \
     | unfold -w 60 \
     | sed -e "1s/^/New-directories: /" -e "1!s/^/  /" ) \
 && (   join -v 1 -o 1.1 -1 2 -2 2 ,,patch-set/orig-files-index ,,patch-set/mod-files-index \
      | sed -e 's,^\./,,' \
      | unfold -w 60 \
      | sed -e "1s/^/Removed-files: /" -e "1!s/^/  /" ) \
 && (  join -v 1 -o 1.1 -1 2 -2 2 ,,patch-set/orig-dirs-index ,,patch-set/mod-dirs-index \
     | sed -e 's,^\./,,' \
     | unfold -w 60 \
     | sed -e "1s/^/Removed-directories: /" -e "1!s/^/  /" ) \
 && (  cat ",,patch-set/mod-files-index" \
     | $ARCH_AWK -f "$ARCH_SUBCMD_ROOT/patch-sets/compute-renamed.awk" \
           -v orig_dir_index=",,patch-set/orig-dirs-index" \
           -v mod_dir_index=",,patch-set/mod-dirs-index" \
           -v orig_X_index=",,patch-set/orig-files-index" \
     | cut -s -d ' ' -f 1,2 \
     | sed -e 's,^\./,,' -e 's, ./, ,' \
     | unfold -w 60 \
     | sed -e "1s/^/Renamed-files: /" -e "1!s/^/  /" ) \
 && (  cat ",,patch-set/mod-dirs-index" \
     | $ARCH_AWK -f "$ARCH_SUBCMD_ROOT/patch-sets/compute-renamed.awk" \
           -v orig_dir_index=",,patch-set/orig-dirs-index" \
           -v mod_dir_index=",,patch-set/mod-dirs-index" \
           -v orig_X_index=",,patch-set/orig-dirs-index" \
     | cut -s -d ' ' -f 1,2 \
     | sed -e 's,^\./,,' -e 's, ./, ,' \
     | unfold -w 60 \
     | sed -e "1s/^/Renamed-directories: /" -e "1!s/^/  /" ) \
 && ( cat ,,modified-files | sed -e 's,^\./,,' | unfold -w 60 | sed -e "1s/^/Modified-files: /" -e "1!s/^/  /" ) \
 && ( cat ,,modified-dirs | sed -e 's,^\./,,' | unfold -w 60 | sed -e "1s/^/Modified-directories: /" -e "1!s/^/  /" ) \
 && ( cat ,,wd-only-patches | sort_fqrn_list | unfold -w 60 | sed -e "1s/^/New-patches: /" -e "1!s/^/  /" ) \
 && ( cat ,,prev-only-patches | sort_fqrn_list | unfold -w 60 | sed -e "1s/^/Removed-patches: /" -e "1!s/^/  /" ) )\
> ,,cooked-log

if test $continuation -ne 0 ; then
  printf "Continuation-of: %s\\n" "$continuation_archive/$continuation_branch--$continuation_vsn--$continuation_lvl" >> ,,cooked-log
fi

(   printf "\\n" \
 && sed -e "1,/^\$/d" ,,raw-log )\
>> ,,cooked-log

if test $verbosity -ge $verbose ; then
  sed -e "/^\$/,\$d" ,,cooked-log | arx body-indent --sub
fi

################################################################
# Put the Right Log File in the Patch Set
# 

cd "$tmpdir"
new_file_in_patch=",,patch-set/new-files-archive/{arch}/$category/$branch/$version/$archive/patch-log/$lvl"
rm -f "$new_file_in_patch"
cp ,,cooked-log "$new_file_in_patch"

# Conditions
#
#   $tmpdir/,,patch-set         -- patch set for new revision
#   $tmpdir/,,cooked-log        -- log file for new revision
#   $tmpdir/$version--$lvl      -- pristine tree of new revision
#
################################################################
# Construct the arx Template Dir
# 
#

if test $verbosity -ge $report ; then
  arx heading --sub "assembling the archive directory\\n"
  if test $verbosity -ge $verbose ; then
    arx heading --sub --sub "start time: %s\\n" "`date`"
  fi
fi

tmparch="$tmpdir/,,arch"
tmparch2="$tmpdir/arch"

mkdir "$tmparch"

mv "$tmpdir/,,patch-set" "$tmparch/$version--$lvl.patches"
cd "$tmparch"
$ARCH_TAR -zcf "$version--$lvl.patches.tar.gz" "$version--$lvl.patches"
mv "$version--$lvl.patches" ../,,patch-set

cp "$tmpdir/,,cooked-log" "$tmparch/log"

cd "$tmparch"

arx make-lock revision-lock

if test $continuation -ne 0 ; then
  printf "%s\\n" "$continuation_archive/$continuation_branch--$continuation_vsn--$continuation_lvl" > CONTINUATION
fi

cd "$tmpdir"
mv "$tmparch" "$tmparch2"

if test $verbosity -ge $verbose ; then
  arx heading --sub --sub "finish time: %s\\n" "`date`"
fi

# Conditions
# 
#   $tmparch2                   -- template of revision directory
#   $prev_tree                  -- points to previous revision
# 
# $tmpdir has these files:
#    
#    ,,wd-dirs          Various specialized file lists
#    ,,wd-files
#    ,,log-ls
#    
#    ,,prev-dirs
#    ,,prev-files
#    ,,prev-patches
#    
#    ,,wd-only-dirs     Various file list diffs and intersections
#    ,,prev-only-dirs
#    
#    ,,wd-only-files
#    ,,prev-only-files
#    ,,files-in-both
#    
#    ,,wd-only-patches
#    ,,wd-only-patches
#    
#    ,,modified-files   List of modified files
#    
#    ,,raw-log          Log as written by user
#    
#    ,,cooked-log       Log with headers added
#    
#    ,,patch-set        The pach-set tree
#    
#    arch               A template of the archive directory
# 
#    $version--$lvl     A copy of the project tree
# 
# 
# 
# 
################################################################
# Install the Arch Template Dir or Clean Up the Output Dir
# 
# 

status=0

if test -z "$dry_run" ; then

  if test $verbosity -ge $report ; then
    arx heading --sub "installing the archive directory in the archive\\n"
    if test $verbosity -ge $verbose ; then
      arx heading --sub --sub "start time: %s\\n" "`date`"
    fi
  fi

  # where does this get stored in the archive?
  # 
  destdir="$category/$branch/$version/$lvl"

  wftp-home

  if ! arx putlast -e commit -A "$archive" \
                    lock-revision "$archive/$version--$lvl" \
                    "$tmparch2" "$destdir" \
                    wd-txn "$wdroot" "$archive" "$version--$lvl" "$tmparch2/log" \
  ; then

    if test $verbosity -ge $quiet ; then
      arx heading --sub --sub "ARCHIVE TRANSACTION FAILED\\n"
      if test $verbosity -ge $verbose ; then
	arx heading --sub --sub "finish time: %s\\n" "`date`"
      fi
    fi
    status=1

  else

    if test $verbosity -ge $verbose ; then
      arx heading --sub --sub "finish time: %s\\n" "`date`"
    fi

#     if arx my-notifier > /dev/null 2>&1 ; then
#       notifier="`arx my-notifier`"
#       ( cd "$notifier" ; \
#         arx without-archive as-daemon $ARCH_LOG_OPTION --null-output arx_notify "$archive/$version" )
#     fi

    if test $verbosity -ge $report ; then
      arx heading --sub "updating local cache to include new revision\\n"
    fi

    cd "$tmpdir"
    mkdir ,,new-pristine
    cd ,,new-pristine


    cached_for_this_wd=`arx pristines --unlocked --dir "$wdroot" "$prev_archive/$prev_branch--$prev_vsn--$prev_lvl"`

    if test -z "$cached_for_this_wd" ; then

      if test $verbosity -ge $report ; then
        arx heading --sub --sub "building revision from scratch\\n"
      fi

      arx nested --sub \
        arx build-revision \
                            --quiet $verbose_opt $debug_opt \
                            --cache "$wdroot/.." \
                            "$archive/$prev_branch--$prev_vsn--$prev_lvl"

    else
      
      if test $verbosity -ge $report ; then
        arx heading --sub --sub "recycling cached pristine revision\\n"
      fi

      arx take-from-cache "$prev_archive/$prev_branch--$prev_vsn--$prev_lvl"

    fi

    arx dopatch --delete-removed --quiet $verbose_opt $debug_opt "$tmpdir/,,patch-set" .

    cd "$tmpdir"
    arx wd-mv-pristine $archive $version--$lvl "$tmpdir/,,new-pristine" "$wdroot"

    cd "$wdroot"
    rm -rf "$tmpdir"
    rm -f "$logfile"

  fi

else

  cd "$tmpdir"

  mv ,,patch-set patch-set

  ln -s "$prev_tree" $prev_branch--$prev_vsn--$prev_lvl

  if test $verbosity -ge $verbose ; then
    arx heading --sub "output in %s\\n" "$tmpdir"
  fi

fi

if test $verbosity -ge $verbose ; then
  arx heading --sub "commit finish time: %s\\n" "`date`"
  arx heading --sub "commit exit status: %d\\n" "$status"
fi

if test 0 -eq $status ; then
  rm -f "$wdroot/{arch}/++branch-revision"
fi

exit $status

# tag: Tom Lord Thu Dec 13 16:04:36 2001 (archive-transactions/fast-commit.sh)
#
