# 
# get-patch.sh - retrieve a patch set from an archive
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents
#                          of the University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

################################################################
# special options
# 
# Some options are special:
# 
#       --version | -V
#       --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;



      --help|-h)
                printf "retrieve a patch set from an archive\\n"
                printf "usage: get-patch [options] [archive/]revision [dir]\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf "Obtain the patch set for REVISION and store it in the DIR\\n"
                printf "(or a directory named REVISION.patches).  DIR must not already\\n"
                printf "exist.\\n"
                printf "\\n"
                exit 0
                ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=

while test $# -ne 0 ; do

  case "$1" in 

    --)                        shift
                            break
                        ;;
                        
    -*)                 printf "get-patch: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                  break
                        ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -lt 1 -o $# -gt 2 ; then
  printf "usage: get-patch [options] [archive/]revision [dir]\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

archive_revision="$1"
shift

if test $# -ne 0 ; then
  dir="$1"
  shift
else
  dir=
fi

################################################################
# Sanity Check and Process Defaults
# 

archive="`arx parse-package-name --arch \"$archive_revision\"`"
revision="`arx parse-package-name --non-arch \"$archive_revision\"`"

arx valid-archive-name -e get-patch -- "$archive"
arx valid-package-name -e get-patch --patch-level -- "$revision"

category=`arx parse-package-name --basename $revision`
branch=`arx parse-package-name $revision`
vsn=`arx parse-package-name --vsn $revision`
lvl=`arx parse-package-name --lvl $revision`

patch_set=$category/$branch/$branch--$vsn/$lvl/$branch--$vsn--$lvl.patches.tar.gz

if test -z "$dir" ; then
  dir=$branch--$vsn--$lvl.patches
fi

cd "`dirname \"$dir\"`"

parent="`pwd`"
dir="$parent/`basename \"$dir\"`"

if test -e "$dir" ; then
  printf "get-patch: output directory already exists\\n" 1>&2
  printf "  dir: %s\\n" "$dir" 1>&2
  printf "\\n" 1>&2
  exit 1
fi

################################################################
# Ensure that We Have an Archive Connection 
# 

if test "$WITHARCHIVE" != "$archive" ; then

  exec arx with-archive -A "$archive" \
      arx get-patch "$archive/$revision" "$dir"
fi

################################################################
# Get a Temporary Directory
# 

tmpdir="$parent/,,get-patch.$$"

bail()
{
  cd "$parent"
  rm -rf "$tmpdir"
  exit 1
}


finish()
{
  cd "$parent"
  rm -rf "$tmpdir"
  exit 0
}

trap "printf \"get-patch: interrupted -- cleaning up\\n\" 1>&2  ; bail" INT

rm -rf "$tmpdir"
mkdir "$tmpdir"



################################################################
# Get the Patch
# 

cd "$tmpdir"

wftp-home

if ! wftp-get $patch_set > $branch--$vsn--$lvl.patches.tar.gz ; then
  printf "get-patch: unable to retrieve patch set\\n" 1>&2
  printf "\\n"
  bail
fi

$ARCH_TAR -m -zxf $branch--$vsn--$lvl.patches.tar.gz 

if test -e "$dir" ; then
  printf "get-patch: output directory already exists\\n" 1>&2
  printf "  dir: %s\\n" "$dir" 1>&2
  printf "\\n" 1>&2
  bail
fi

mv $branch--$vsn--$lvl.patches "$dir"

finish




