# 
# import.sh - archive a from scratch base revision
################################################################
# Copyright (C) 2001, 2002, 2003 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the
#                                  University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#       --version | -V
#       --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "archive a from-scratch base revision\\n"
                printf "usage: import [options] [[archive/]version]\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " -L --log-file file            specify the log-file\\n"
                printf " --not-default                 do not make this the default branch\\n"
                printf "\\n"
                printf " -d --dir DIR                  cd to DIR first\\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf " --begin                       prepare a revision for check-in\\n"
                printf "                               but do not modify the archive.\\n"
                printf " --finish\\n"
                printf "\\n"
                printf "Archive a from-scratch base revision of the project tree\\n"
                printf "containing DIR (or the current directory)\\n"
                printf "\\n"
                printf "1) Make the selected version and archive the defaults for this working\\n"
                printf "directory (unless -d (--not-default) is specified).\\n"
                printf "\\n"
                printf "2) In a scratch directory, make a pristine copy of the source tree.\\n"
                printf "Install the log file in the patch log of the pristine source tree.\\n"
                printf "\\n"
                printf "3) In the same scratch directory, create a template for the revision\\n"
                printf "directory.  The revision directory is what will eventually be installed\\n"
                printf "in the archive.\\n"
                printf "\\n"
                printf "4) Lock the revision in the archive.  This step will fail if someone else\\n"
                printf "has already committed the same revision.\\n"
                printf "\\n"
                    printf "5) Copy the template for the revision directory to a temporary directory\\n"
                printf "in the archive.\\n"
                printf "\\n"
                printf "6) Mark the project tree as \"might be in a weird sate.\"  A weird state\\n"
                printf "occurs if the revision is committed in the archive, but the project tree\\n"
                printf "hasn't yet been updated to reflect that fact.\\n"
                printf "\\n"
                printf "7) Commit the change to the archive.  This will fail if someone got there\\n"
                printf "first and broke your lock.\\n"
                printf "\\n"
                printf "8) Mark the project tree as definately weird.\\n"
                printf "\\n"
                printf "9) Modify the project tree to reflect the fact that the commit suceeded.\\n"
                printf "\\n"
                printf "10) Mark the project tree as not weird at all.\\n"
                printf "\\n"
                printf "11) Clean up by removing various temporary files in the project tree.\\n"
                printf "\\n"
                printf "If a project tree winds up in a \"maybe weird\" or \"definately weird\"\\n"
                printf "state, try the command \"arx tree-repair --help\" to learn how to recover.\\n"
                printf "\\n"
                exit 0
                ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=
not_default=
logfile=

begin=1
finish=1


silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=

dir=.

while test $# -ne 0 ; do

  case "$1" in 

    -d|--dir)  shift
               if test $# -eq 0 ; then
                  printf "import: -d and --dir require an argument\\n" 1>&2
                  printf "try --help\\n" 1>&2
                  exit 1
               fi
               dir="$1"
               shift
               ;;

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "import: debugging output enabled\\n"
                    set -x
                debug_opt=--debug
                ;;

    --begin)                shift
                            begin=1
                            finish=0
                        ;;

    --finish)                shift
                        begin=0
                        finish=1
                        ;;

    -L|--log-file)      shift
                        if test $# -eq 0 ; then
                          printf "import: -L and --log-file require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        logfile="$1"
                        shift
                        ;;

    --not-default)      shift
                        not_default=--not-default
                        ;;

    --)                        shift
                            break
                        ;;

    -*)                 printf "import: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                  break
                        ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -gt 1 ; then
  printf "usage: import [options] [[archive/]version]\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

if test $# -ge 1 ; then
  archive_version="$1"
  shift
else
  archive_version=
fi

################################################################
# Sanity Check and Process Defaults
# 
  
cd "$dir"
dir="`pwd`"
wdroot="`arx tree-root --accurate`"
cd "$wdroot"
  
if test "x$archive_version" = x ; then
  archive_version=`arx tree-version`
else
  arx valid-package-name -e import --vsn -- "$archive_version"
fi

archive=`arx parse-package-name --arch $archive_version`
version=`arx parse-package-name --package-version $archive_version`
category=`arx parse-package-name --basename $version`
branch=`arx parse-package-name $version`
patchlvl=base-0

destdir=$category/$branch/$version/$patchlvl
  
tmpdir="$wdroot/,,commit-tmp-dir"
tmptree="$tmpdir/$version--$patchlvl"
tmparch="$tmpdir/,,$patchlvl"
tmparch2="$tmpdir/$patchlvl"

if test "x$logfile" = x ; then
  logfile=++log.$version--$archive
fi


################################################################
# Greetings
# 

if test $verbosity -ge $report; then
  arx heading "import\\n"
  printf "arguments: %s\\n" "$command_line" | fold -w 60 | arx body-indent
  if test $verbosity -ge $verbose; then
    arx heading --sub "import start time: %s\\n" "`date`"
    arx heading --sub "project tree: %s\\n" "$wdroot"
    arx heading --sub "archive: %s\\n" "$archive"
    arx heading --sub "version: %s\\n" "$version"
    arx heading --sub "patch level: %s\\n" "$patchlvl"
  fi
fi

################################################################
# Check Roughly That the Version Can Be Created
# 
# Does the wd have a patch log for this version?
# 

arx wd-check-for-patch-log -e import "$archive/$version" "$wdroot"


################################################################
# This Script Comes in Two Parts
# 
# This script has a `begin' half, and a `finish' half.
# It can either run both in succession, or either one
# by itself.
# 
# The idea here is that if you are being very careful, 
# you can run the `begin' half, review the results, then
# run the `finish' half.
# 
# First, here are some computed values needed by both the
# begin and the finish half:
# 


################################################################
# The --begin Half of the Script:
# 

if test $begin = 1 ; then

  ################################################################
  # Did --begin Already Start to Run in this Working Directory?
  # 

  if test -e "$tmpdir" ; then
    if test -e "$tmpdir/,,,import" ; then
      operation=import
    else
      operation=commit
    fi

    printf "\\n" 1>&2
    printf "import: fatal error\\n" 1>&2
    printf "\\n" 1>&2
    printf "A temporary directory for a %s operation already exists:\\n" "$operation" 1>&2
    printf "\\n" 1>&2
    printf "        %s\\n" "$tmpdir" 1>&2
    printf "\\n" 1>&2

    ################################################################
    # Did --begin Run to Completion?
    # 

    if test $operation != import ; then
      printf "It appears that you have already started some other\\n" 1>&2
      printf "form of commit but that it did not run to completion.\\n" 1>&2
      printf "\\n" 1>&2
      printf "If you started that other commit with --begin, consider\\n" 1>&2
      printf "finishing it with --end.  Otherwise, remove the temporary\\n" 1>&2
      printf "directory and try again.\\n" 1>&2
      printf "\\n" 1>&2
    elif test -e "$tmparch2" ; then
      printf "It appears that you have already run \"import --begin\".\\n" 1>&2
      printf "Consider trying \"arx import --finish\" now.\\n" 1>&2
      printf "\\n" 1>&2
      printf "\\n" 1>&2
    else
      printf "It appears that a previous \"import\" or \"import --begin\" did\\n" 1>&2
      printf "run to completion.\\n" 1>&2
      printf "\\n" 1>&2
      printf "The recommeded recovery is to remove the temporary directory and\\n" 1>&2
      printf "try again.\\n" 1>&2
      printf "\\n" 1>&2
    fi

    exit 1
  fi


  ################################################################
  # Sanity Check and Process Defaults
  # 
  
  arx valid-log-file -e import -- "$logfile" "$wdroot"

  arx nested arx tree-lint --quiet $verbose_opt $debug_opt "$wdroot"
  

  ################################################################
  # Set the Default Version for the Working Dir
  # 

  if test -z "$not_default" ; then

    if test $verbosity -ge $report; then
      arx heading --sub "set default tree version\\n"
      arx heading --sub --sub "archive: %s\\n" "$archive"
      arx heading --sub --sub "version: %s\\n" "$version"
    fi

    cd "$wdroot"
    arx set-tree-version "$archive/$version"

  fi

  ################################################################
  # Make Temp Dir for our Work
  # 
  # Note that we create the temp dir at the root of the working 
  # directory.  Note that the temp dir must not already exist.
  # If it does, we fail, because that might indicate a concurrent
  # import attempt for the same wd.  We trust the user to be
  # smart enough to remove the temp dir if she sees an error message
  # from mkdir.
  # 
  # The temp dir MUST have a temporary name, otherwise, it will
  # appear to be source and will be recursively copied into the
  # (now infinite) pristine copy of the source.
  # 
  
  bail()
  {
    cd "$wdroot"
    rm -rf "$tmpdir"
    exit 1
  }

  trap "printf \"import: interrupted -- cleaning up\\n\" 1>&2 ; bail" INT

  rm -rf "$tmpdir"
  mkdir "$tmpdir"
  touch "$tmpdir/,,,import"

  
  ################################################################
  # Create a Temp Dir for the Branch Revision Template
  # 
  #
  
  mkdir "$tmparch"
  cd "$tmparch"
  arx make-lock revision-lock
  
  ################################################################
  # Build the Pristine Tree
  # 
  
  if test $verbosity -ge $report; then
    arx heading --sub "make a pristine copy of source tree\\n"
    if test $verbosity -ge $verbose; then
      arx heading --sub --sub "start time: %s\\n" "`date`"
    fi
  fi

  cd "$wdroot"
  mkdir "$tmptree"
  
  # Copy the Source
  # 
  arx inventory --source --control --files \
  | copy-file-list -- - . "$tmptree"

  if test $verbosity -ge $verbose; then
    arx heading --sub --sub "finish time: %s\\n" "`date`"
  fi
  
  ################################################################
  # Sanity check 2
  # 
  # Double check that the pristine tree has a patch log for this
  # version.
  # 

  cd "$tmptree"
  arx wd-check-for-patch-log -e import "$archive/$version" "$wdroot"
  
  ################################################################
  # Install the Log File in the Pristine Tree Patch Log and Revision Template
  # 
  # 
  
sort_fqrn_list()
{
  sed -e 's+\(.*\)/\(.*\)--\(.*\)--\([0-9]*\)\.\([0-9]*\)--\(.*\)-\([0-9]*\)+\1,\2,\3,\4,\5,\6,\7+' \
      -e 's+\(.*\)/\(.*\)--\([0-9]*\)\.\([0-9]*\)--\(.*\)-\([0-9]*\)+\1,\2,,\3,\4,\5,\6+' \
  | sort -u -t, -k 1,1 -k2,2 -k3,3 -k4,4n -k5,5n -k6,6 -k7,7n \
  | sed -e 's+^\(.*\),\(.*\),,\(.*\),\(.*\),\(.*\),\(.*\)+\1/\2--\3.\4--\5-\6+' \
        -e 's+^\(.*\),\(.*\),\(.*\),\(.*\),\(.*\),\(.*\),\(.*\)+\1/\2--\3--\4.\5--\6-\7+' 
}

  if test $verbosity -ge $report; then
    arx heading --sub "mangle log entry\\n"
  fi

  cd "$wdroot"
  cp "$logfile" "$tmparch/,,log"
  cd "$tmparch"

  # 
  # start by installing the wrong log file
  # 
  arx copy-to-patch-log "$tmparch/,,log" "$archive/$version" $patchlvl "$tmptree"


  (   printf "Revision: %s\\n" $version--$patchlvl \
   && printf "Archive: %s\\n" $archive \
   && printf "Creator: %s\\n" "`arx my-id`" \
   && printf "Date: %s\\n" "`date`" \
   && printf "Standard-date: %s\\n" "`date -u \"+%Y-%m-%d %H:%M:%S GMT\"`" \
   && sed -e "/^\$/,\$d" ,,log \
   && ( cd "$tmptree" ; arx inventory --source --control --files | grep -v -F "./{arch}" | sed -e "s/\\n/ /" | unfold -w 60 | sed -e "1s/^/New-files: /" -e "1!s/^/  /" ) \
   && ( cd "$tmptree" ; \
        ( arx logs | xargs -n 1 need-args "arx log-ls --full" ) \
        | sort_fqrn_list \
        | unfold -w 60 \
        | sed -e "1s/^/New-patches: /" -e "1!s/^/  /" ) \
   && printf "\\n" \
   && sed -e "1,/^\$/d" ,,log ) \
  > log

  rm ,,log

  if test $verbosity -ge $verbose; then
    sed -e "/^\$/,\$d" log | arx body-indent --sub
  fi

  # 
  # now install the right log file
  # 
  arx copy-to-patch-log "$tmparch/log" "$archive/$version" $patchlvl "$tmptree"

  ################################################################
  # Store the tar.gz File in the Revision Template 
  #
  
  if test $verbosity -ge $report; then
    arx heading --sub "creating tar file of source\\n"
    if test $verbosity -ge $verbose; then
      arx heading --sub --sub "start time: %s\\n" "`date`"
    fi
  fi

  cd "$tmpdir"
  $ARCH_TAR -zcf "$tmparch/$version--$patchlvl.src.tar.gz"  "$version--$patchlvl"
  
  if test $verbosity -ge $verbose; then
    arx heading --sub --sub "tar file details\\n"
    (cd "$tmparch" ; ls -l $version--$patchlvl.src.tar.gz) | arx body-indent --sub --sub
    arx heading --sub --sub "finish time: %s\\n" "`date`"
  fi

  ################################################################
  # Finish --begin
  # 
  # The pristine tree and revision template are now ready.  Record
  # that --begin ran to completion by renaming the archive template:
  # 

  cd "$tmpdir"
  mv "$tmparch" "$tmparch2"

  ################################################################
  # Give Advice
  # 

  if test $finish -eq 0 ; then
    if test -a $verbosity -ge $verbose ; then
      arx heading --sub "import finish time: %s\\n" "`date`"
      arx heading --sub --sub "pristine project tree: %s\\n" "$tmptree"
      arx heading --sub --sub "archive data: %s\\n" "$tmparch2"
    fi
    exit 0
  fi
  
fi

################################################################
# The --finish Half of the Script:
# 

if test $finish -eq 1 ; then

  ################################################################
  # Did --begin Already Start to Run in this Working Directory?
  # 

  if test ! -e "$tmpdir" ; then
    printf "\\n" 1>&2
    printf "import: fatal error\\n" 1>&2
    printf "\\n" 1>&2
    printf "You have not run --begin for this import yet.  This directory\\n" 1>&2
    printf "was not found:\\n" 1>&2
    printf "\\n" 1>&2
    printf "        %s\\n" "$tmpdir" 1>&2
    printf "\\n" 1>&2
    exit 1
   fi


  ################################################################
  # Did --begin Run to Completion?
  # 

  if test ! -e "$tmparch2" ; then
    printf "\\n" 1>&2
    printf "import: fatal error\\n" 1>&2
    printf "\\n" 1>&2
    printf "It appears that you have already run \"import --begin\"\n" 1>&2
    printf "but that it did not run to completion.\\n" 1>&2
    printf "\\n" 1>&2
    printf "The recommended recovery is to remove the temporary directory and\\n" 1>&2
    printf "try again (from --begin).\\n" 1>&2
    printf "\\n" 1>&2
    printf "The temporary directory is:\\n" 1>&2
    printf "\\n" 1>&2
    printf "        %s\\n" "$tmpdir" 1>&2
    printf "\\n" 1>&2
    exit 1
  fi


  ################################################################
  # Attempt to Commit the Revision
  # 

  status=0

  if test $verbosity -ge $report; then
    arx heading --sub "storing revision in archive\\n"
    if test $verbosity -ge $verbose; then
      arx heading --sub --sub "transaction start: %s\\n" "`date`"
    fi
  fi

  if ! arx with-archive -A "$archive"  \
            arx putlast -e import -A "$archive" \
                         lock-revision "$archive/$version--$patchlvl" \
                         "$tmparch2" $destdir \
                         wd-txn "$wdroot" "$archive" $version--$patchlvl "$tmparch2/log" \
  ; then

    if test $verbosity -ge $quiet; then
      arx heading --sub --sub "transaction FAILED: %s\\n" "`date`"
    fi

    status=1

  else

    if test $verbosity -ge $verbose; then
      arx heading --sub --sub "transaction complete: %s\\n" "`date`"
    fi

#     if arx my-notifier > /dev/null 2>&1 ; then
#       notifier="`arx my-notifier`"
#       ( cd "$notifier" ; \
#         arx without-archive as-daemon $ARCH_LOG_OPTION --null-output arx_notify "$archive/$version" )
#     fi

    arx wd-mv-pristine "$archive" "$version--$patchlvl" "$tmptree" "$wdroot"
    cd "$wdroot"
    rm -rf "$tmpdir"
    rm -f "$logfile"
  fi

  if test $verbosity -ge $verbose; then
    if test $status -eq 0 ; then
      arx heading --sub "import complete: %s\\n" "`date`"
    fi
  fi

  exit $status

fi

