# 
# archive-uncache-revision.sh - remove a cached revision
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents
#                          of the University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 
command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#       --version | -V
#       --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "remove a cached revision from an archive\\n"
                printf "usage: archive-uncache-revision [options] [[archive/]revision]\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf "Remove the cached REVISION from its archive\\n."
		printf "If no revision is specified, it uses the output of tree-version.\\n"
                printf "\\n"
                printf "Also see \"arx archive-cache-revision --help\"\\n"
                printf "\\n"
                exit 0
                ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=

__restart=


silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=

while test $# -ne 0 ; do

  case "$1" in 

    ----restart) shift
                    __restart=----restart
                ;;

    --no-lint)  shift
                    no_lint=--no-lint
                ;;

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "archive-uncache-revision: debugging output enabled\\n"
                    set -x
                debug_opt=--debug
                ;;

    -*)                 printf "archive-uncache-revision: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                  break
                        ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -gt 1 ; then
  printf "usage: archive-uncache-revision [options] [[archive/]revision]\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

if test $# -ne 0 ; then
  rvnspec="$1"
  shift
else
  rvnspec=
fi


################################################################
# Sanity Check and Process Defaults
# 

archive_revision=`arx indicated-revision -e archive-uncache-revision "$rvnspec"`

if test -z "$rvnspec" ; then
  rvnspec="`arx tree-version`"
fi

arx valid-package-name --tolerant "$rvnspec"

archive="`arx parse-package-name --arch \"$rvnspec\"`"
revision_spec="`arx parse-package-name --non-arch \"$rvnspec\"`"
category=`arx parse-package-name --basename $revision_spec`
branch=`arx parse-package-name --package $revision_spec`


################################################################
# Greetings
# 

if test "(" -z "$__restart" -a $verbosity -ge $report ")" -o $verbosity -ge $verbose ; then
  arx heading "archive-uncache-revision\\n"
  printf "arguments: %s\\n"  "$command_line" | fold -w 60 | arx body-indent
  if test $verbosity -ge $verbose ; then
    arx heading --sub "archive-uncache-revision start time: %s\\n" "`date`"
    arx heading --sub "archive: %s\\n" "$archive"
    arx heading --sub "revision spec: %s\\n" "$revision_spec"
  fi
fi


################################################################
# Ensure that We Have an Archive Connection 
# 

if test "$WITHARCHIVE" != "$archive" ; then

  if test $verbosity -ge $verbose ; then
    arx heading --sub "restarting with connection to archive\\n"
  fi

  exec arx with-archive -A "$archive"  \
        arx archive-uncache-revision $verbose_opt $debug_opt \
                                      ----restart \
                                      "$archive/$revision_spec"

fi


################################################################
# Which Revision are We Caching?
# 

archive_revision="`arx indicated-revision -e archive-uncache-revision \"$archive/$revision_spec\"`"
revision="`arx parse-package-name --non-arch \"$archive_revision\"`"

if test $verbosity -ge $report ; then
  arx heading --sub "revision: %s\\n" "$revision" 
fi

vsn=`arx parse-package-name --vsn $revision`
lvl=`arx parse-package-name --lvl $revision`


################################################################
# Delete the Cached Revision
# 

wftp-home
wftp-cd $category/$branch/$branch--$vsn/$lvl
wftp-delete $revision.tar.gz

if test $verbosity -ge $verbose ; then
  arx heading --sub "archive-uncache-revision finish time: %s\\n" "`date`"
fi
