# 
# make-branch.sh - create a new branch within an archive
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents
#                          of the University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "create a new branch within an archive\\n"
                printf "usage: make-branch [options] branch\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " -r --readme file              save FILE as the README\\n"
                printf "                               for this category\\n"
                printf "\\n"
                printf "Create BRANCH as a branch in the indicated archive.\\n"
                printf "\\n"
                printf "The category containing the branch must already exist.\\n"
                printf "That category has the name of the package basename of\\n"
                printf "the branch (\"arx parse-package-name -b \$branch\").\\n"
                printf "See \"arx make-category --help\".\\n"
                printf "\\n"
                printf "For information about how the archive to operate on is selected,\\n"
                printf "try \"arx my-default-archive --help\".\\n"
                printf "\\n"
                exit 0
                      ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=
readme=

while test $# -ne 0 ; do

  case "$1" in 

    -r|--readme)        shift
                        if test $# -eq 0 ; then
                          printf "make-branch: -r and --readme require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        readme="$1"
                        shift
                        ;;

    -*)                        printf "make-branch: unrecognized option (%s)\\n" $1 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                        break
                            ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 1 ; then
    printf "usage: make-branch [options] category\\n" 1>&2
    printf "try --help\\n" 1>&2
    exit 1
fi

branch="$1"

################################################################
# Sanity Check 
# 

arx valid-package-name -e make-branch -- "$branch"
idstr="`arx my-id -e make-branch`"

archive="`arx parse-package-name --arch \"$branch\"`"
cat=`arx parse-package-name -b $branch`
branch="`arx parse-package-name \"$branch\"`"

################################################################
# Get a Temp Dir
# 

tmpdir="`arx make-cache-temp-dir new-branch-$$`"


################################################################
# Create the Category Directory (in the Temp Dir)
# 

tmpbranch="$tmpdir/$branch"
mkdir "$tmpbranch"


################################################################
# Make the =README file
# 

cd "$tmpbranch"
printf "Creator: %s\\nDate: %s\\n\\n" "$idstr" "`date`" > =README


if test "x$readme" != x ; then

  if ! cat "$readme" >> =README ; then
     rm -r "$tmpdir"
  fi

fi


################################################################
# Make the Tip of the Version Sequence
# 
cd "$tmpbranch"
arx make-lock version-lock

################################################################
# Try to install it in the archive
# 

cd "$tmpdir"

status=0

if ! arx with-archive -A "$archive" arx putdir -e make-branch -A "$archive"  "$branch" "$cat/$branch" ; then
  printf "make-branch: unable to create category\\n" 1>&2
  status=1
fi


# if arx my-notifier > /dev/null 2>&1 ; then
#   notifier="`arx my-notifier`"
#   ( cd "$notifier" ; \
#     arx without-archive as-daemon $ARCH_LOG_OPTION --null-output arx_notify "$archive/$branch" )
# fi

################################################################
# Remove the Cache Temp Directory
# 
cd ..
rm -rf "$tmpdir"

exit "$status"

