# 
# make-category.sh - create a new category within an archive
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents
#                          of the University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "create a new category within an archive\\n"
                printf "usage: make-category [options] category\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " -r --readme file              save FILE as the README\\n"
                printf "                               for this category\\n"
                printf "\\n"
                printf "Create CATEGORY as a category in the indicated archive.\\n"
                printf "The category must not already exist.\\n"
                printf "\\n"
                printf "For information about how the archive to operate on is selected,\\n"
                printf "try \"arx my-default-archive --help\".\\n"
                printf "\\n"
                exit 0
                      ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=
readme=

while test $# -ne 0 ; do

  case "$1" in 

    -r|--readme)        shift
                        if test $# -eq 0 ; then
                          printf "make-category: -r and --readme require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        readme="$1"
                        shift
                        ;;

    -*)                        printf "make-category: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                        break
                            ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 1 ; then
    printf "usage: make-category [options] category\\n" 1>&2
    printf "try --help\\n" 1>&2
    exit 1
fi

category="$1"

################################################################
# Sanity Check 
# 

arx valid-package-name -e make-category -b -- "$category"
idstr="`arx my-id -e make-category`"

archive="`arx parse-package-name --arch \"$category\"`"
category="`arx parse-package-name --basename \"$category\"`"

################################################################
# Get a Temp Dir
# 

tmpdir="`arx make-cache-temp-dir new-category-$$`"


################################################################
# Create the Category Directory (in the Temp Dir)
# 

tmpcat="$tmpdir/$category"
mkdir "$tmpcat"


################################################################
# Make the =README file
# 

printf "Creator: %s\\nDate: %s\\n\\n" "$idstr" "`date`" > "$tmpcat/=README" 


if test ! -z "$readme" ; then

  cat "$readme" >> "$tmpcat/=README" 

fi




################################################################
# Try to install it in the archive
# 

cd "$tmpdir"

status=0

if ! arx with-archive -A "$archive" arx putdir -e make-category -A "$archive"  "$category" "$category" ; then
  printf "make-category: unable to create category\\n" 1>&2
  status=1
fi

# if arx my-notifier > /dev/null 2>&1 ; then
#   notifier="`arx my-notifier`"
#   ( cd "$notifier" ; \
#     arx without-archive as-daemon $ARCH_LOG_OPTION --null-output arx_notify "$archive/$category" )
# fi


################################################################
# Remove the Cache Temp Directory
# 
cd ..
rm -rf "$tmpdir"

exit "$status"

