# 
# create-branch.sh: make a project tree for a new branch
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and
#                          the Regents of the University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 
command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#       --version | -V
#       --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "create a project tree for a new branch\\n"
                printf "usage: create-branch [options] \\\\\\n"
                printf "                      [archive/]old-revision \\\\\\n"
                printf "                      [archive/]new-version \\\\\\n"
                printf "                      dir\\n"
		printf "       create-branch [options] --in-place\\n"
		printf "                     [archive/]new-version\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
		printf " -L --log-file file            specify the name of the log file\\n"
		printf " --non-interactive             Don't pop up an editor \\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf "Create a new branch from an OLD-REVISION, putting the result in DIR.\\n"
		printf "With --in-place, it converts the current project tree into the new branch.\\n"
                printf "\\n"
                printf "It initializes an empty log entry for commit to the new version.\\n"
		printf "Normally it will try to invoke an editor for the log file.\\n"
		printf "See \"arx make-log --help\" for details on how the editor is chosen.\\n"
		printf "With the --non-interactive option, the name of the log file is printed to\\n"
		printf "standard output.\\n"
                printf "\\n"
                printf "This can be followed by \"arx commit\" to create a new\\n"
                printf "branch in an archive.\\n"
                printf "\\n"
                exit 0
                ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=

silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=
__restart=
logname=
non_interactive=
in_place=

while test $# -ne 0 ; do

  case "$1" in 

    ----restart) shift
                    __restart=----restart
                ;;

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "create-branch: debugging output enabled\\n"
                    set -x
                debug_opt=--debug
                ;;

    -L|--log-file)        shift
                            if test $# -eq 0 ; then
                          printf "create-branch: -L and --log-file require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        logname="$1"
                        shift
                        ;;

    --in-place)         shift
	                in_place=--in-place
			;;

    --non-interactive)  shift
	                non_interactive=--non-interactive
			;;


    --)                        shift
                            break
                        ;;
                        
    -*)                 printf "create-branch: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                  break
                        ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 3 ; then
  if test -n "$in_place" ; then
    if test $# -ne 1 ; then
      printf "usage: create-branch [options] --in-place\\n"
      printf "                     [archive/]new-version\\n"
      exit 1
    fi
  else
    printf "usage: create-branch [options] \\\\\\n" 1>&2
    printf "                      [archive/]old-revision \\\\\\n" 1>&2
    printf "                      [archive/]version \\\\\\n" 1>&2
    printf "                      dir\\n" 1>&2
    printf "try --help\\n" 1>&2
    exit 1
  fi
fi

if test -z "$in_place" ; then
  old_revision="$1"
  shift
else
  old_revision="`arx tree-version`"
fi

new_version="$1"
shift

if test -z "$in_place" ; then
  dir="$1"
  shift
fi


################################################################
# Sanity Check and Process Defaults
# 

here="`pwd`"
if test -z "$in_place" ; then
  cd "`dirname \"$dir\"`"

  parent="`pwd`"
  dir="$parent/`basename \"$dir\"`"

  cd "$parent"
fi

arx valid-package-name -e create-branch --tolerant -- "$old_revision"
arx valid-package-name -e create-branch --tolerant -- "$new_version"

old_archive="`arx parse-package-name --arch \"$old_revision\"`"
old_spec="`arx parse-package-name --non-arch \"$old_revision\"`"
new_archive="`arx parse-package-name --arch \"$new_version\"`"
new_spec="`arx parse-package-name --non-arch \"$new_version\"`"

################################################################
# Greetings
# 

if test "(" -z "$__restart" -a $verbosity -ge $report ")" -o $verbosity -ge $verbose ; then
  arx heading "create-branch\\n"
  printf "arguments: %s\\n"  "$command_line" | fold -w 60 | arx body-indent
  if test $verbosity -ge $verbose ; then
    arx heading --sub "create-branch start time: %s\\n" "`date`"
    arx heading --sub "old archive: %s\\n" "$old_archive"
    arx heading --sub "old revision spec: %s\\n" "$old_spec"
    arx heading --sub "new archive: %s\\n" "$new_archive"
    arx heading --sub "new version spec: %s\\n" "$new_spec"
    if test -z "$in_place" ; then
      arx heading --sub "output directory: %s\\n" "$dir"
    fi
  fi
fi

################################################################
# Ensure that We Have an Archive Connection 
# 

if test "$WITHARCHIVE" != "$old_archive" ; then

  if test $verbosity -ge $verbose ; then
    arx heading --sub "restarting with connection to archive\\n"
  fi

  if test -z "$in_place" ; then
    exec arx with-archive -A "$old_archive"  \
              arx create-branch $verbose_opt $debug_opt \
                   $non_interactive ----restart \
                   "$old_archive/$old_spec" "$new_archive/$new_spec" "$dir"
  else
    exec arx with-archive -A "$old_archive"  \
              arx create-branch $verbose_opt $debug_opt \
                   $in_place $non_interactive ----restart \
                   "$new_archive/$new_spec"
  fi
fi

################################################################
# What Revision are we Branching From
# 

old_revision="`arx indicated-revision -e create-branch \"$old_archive/$old_spec\"`"
old_revision="`arx parse-package-name --non-arch \"$old_revision\"`"

if test $verbosity -ge $verbose ; then
  arx heading --sub "old revision: %s\\n" "$old_revision"
fi

################################################################
# What Version are we Branching To
# 

if arx valid-package-name --vsn "$new_spec" ; then
  new_version="$new_archive/$new_spec"
else
  new_version="`arx versions --reverse \"$new_archive/$new_spec\" | head -1`"
fi

if test -z "$new_version" ; then
  if test $verbosity -ge $quiet ; then
    printf "\\n" 1>&2
    printf "create-branch: no versions found\\n" 1>&2
    printf "  archive: %s\\n" "$new_archive" 1>&2
    printf "  version spec: %s\\n" "$new_spec" 1>&2
    printf "\\n" 1>&2
  fi
  exit 1
fi

if test $verbosity -ge $verbose ; then
  arx heading --sub "new version: %s\\n" "$new_version"
fi

################################################################
# Do It
# 

if test -z "$in_place" ; then

  if test $verbosity -ge $report ; then
    arx heading --sub "fetching the BRANCH FROM revision\\n"
  fi

  arx nested --sub arx get --quiet $verbose_opt $debug_opt \
                       "$old_archive/$old_revision" "$dir"

  if test $verbosity -ge $report ; then
    arx heading --sub "switching new directory to branch version\\n"
  fi

  cd "$dir"
fi

arx add-log "$new_version"
arx set-tree-version "$new_version"

if test $verbosity -ge $report ; then
  arx heading --sub "creating log file template\\n"
fi

if test -n "$logname" ; then
  arx make-log $non_interactive -L "$logname"
  log_template="$logname"
else
  arx make-log $non_interactive
  log_template="`arx make-log --non-interactive`"
fi
printf "branched from %s\\n" "$old_archive/$old_revision" >> "$log_template"
printf "\\n" >> "$log_template"

rm -f "{arch}/++branch-revision"

printf "%s\\n" "$old_archive/$old_revision" > "{arch}/++branch-revision"

if test $verbosity -ge $verbose ; then
  arx heading --sub "create-branch finish time: %s\\n" "`date`"
fi

# tag: Tom Lord Mon Dec 10 19:15:50 2001 (branching-and-merging/prepare-branch.sh)
#
