# 
# create-version.sh - make the current tree into a new version
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry
#                          and the Regents of the University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "make the current tree into a new version\\n"
                printf "usage: create-version [options] [archive/]version\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " -L --log-file file             specify the name of the log-file\\n"
                printf " -d --dir DIR                  cd to DIR first\\n"
		printf " --non-interactive             Don't pop up an editor for the log file\\n"
                printf " --nested                      create a nested project tree\\n"
                printf "\\n"
                printf "Sets things up so that a new version will\\n"
                printf "be created when you commit (either as a continuation of the current\\n"
                printf "version, or as a completely new line of development.\\n"
                printf "\\n"
                printf "It invokes \"arx make-log\" which pops up an editor to edit a log file.\\n"
		printf "With the --non-interactive option, the name of the new log file is\\n"
                printf "printed to standard output.\\n"
                printf "\\n"
                exit 0
                      ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

dir=.
new_version=
nested=
non_interactive=

while test $# -ne 0 ; do

  case "$1" in 

    -d|--dir)                shift
                        if test $# -eq 0 ; then
                          printf "create-version: -d and --dir require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        dir="$1"
                        shift
                        ;;

    --nested)             shift
                        nested=--nested
                        ;;

    --non-interactive)  shift
                        non_interactive=--non-interactive
                        ;;

    -L|--log-file)        shift
                            if test $# -eq 0 ; then
                          printf "create-version: -L and --log-file require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        logname="$1"
                        shift
                        ;;




    -*)                        printf "create-version: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                        break
                            ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 1 ; then
  printf "usage: create-version [options] [archive/]version\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

new_version="$1"
shift


################################################################
# Sanity Check and Process Defaults
# 

here="`pwd`"
cd "$dir"
dir="`pwd`"

# If making a new version, either initialise the tree for import, or
# mark the tree as a branch.  Very non-transactional.

if test -n "$nested" || ! arx tree-root --silent > /dev/null ; then
  options=
  if test -n "$nested" ; then
    options="$nested"
  fi
  if test -n "$options" ; then
    arx init-tree $options "$new_version"
  else
    arx init-tree "$new_version"
  fi
else
  if test -f "{arch}/++branch-revision"; then
    printf "{arch}/++branch-revision already exists.\\n"
    exit 1
  fi
  arx tree-version > "{arch}/++branch-revision"
  arx add-log "$new_version"
  arx set-tree-version "$new_version"
fi

arx make-log $non_interactive "$new_version"
