# 
# delta-patch.sh: compute and apply an arbitrary patch 
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Modifications 2002 Alexander Deruwe
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the University
# of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 
command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#       --version | -V
#       --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "compute and apply an arbitrary patch\\n"
                printf "usage: delta-patch [options] from to upon output-dir\\n"
                printf "usage: delta-patch [options] --in-place from to upon\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " --in-place                    modify an existing directory\\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf "Construct a patch from FROM to TO and apply to UPON creating\\n"
                printf "OUTPUT-DIR\\n"
                printf "\\n"
                printf "FROM, TO, and UPON may each be revisions or project trees.\\n"
                printf "\\n"
                printf "If UPON is a project tree, the \"--in-place\" option can be used\\n"
                printf "to modify that tree directly, rather than creating a new tree.\\n"
                printf "\\n"
                printf "The exit status reflects the exit status of \"dopatch\": 0 for a\\n"
                printf "conflictless patch application, 1 if conflicts occurred, 2 if an\\n"
                printf "internal error occured.\\n"
                printf "\\n"
                exit 0
                ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=

list=
in_place=


silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=


while test $# -ne 0 ; do

  case "$1" in 

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "delta-patch: debugging output enabled\\n"
                    set -x
                debug_opt=--debug
                ;;


    --in-place)                shift
                            in_place=--in-place
                        ;;

    --)                        shift
                            break
                        ;;
                        
    -*)                 printf "delta-patch: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 2
                        ;;

    *)                  break
                        ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test ! -z "$in_place" ; then
  n_dir_args=0
else
  n_dir_args=1
fi

n_args=$(($n_dir_args + 3))

if test $# -ne $n_args ; then
  printf "usage: delta-patch [options] from to upon output-dir\\n" 1>&2
  printf "       delta-patch [options] --in-place from to upon\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

A="$1"
shift

B="$1"
shift

C="$1"
shift

if test -z "$in_place" ; then
  output_dir="$1"
  shift
else
  output_dir="$C"
fi

################################################################
# Sanity Check and Process Defaults
# 

here="`pwd`"

abs_dir()
{
  cd "$here"
  cd "`dirname \"$1\"`"
  printf "%s\\n" "`pwd`/`basename \"$1\"`"
}


if test -z "$in_place" -a -e "$output_dir" ; then
  printf "\\n" 1>&2
  printf "delta-patch: output directory already exists\\n" 1>&2
  printf "  dir %s\\n" "$output_dir"1>&2
  printf "\\n" 1>&2
  printf "Do you mean \"--in-place\"?\\n" 1>&2
  printf "\\n" 1>&2
  printf "Try \"delta-patch --help\"\\n" 1>&2
  printf "\\n" 1>&2
  exit 2
fi

output_dir="`abs_dir \"$output_dir\"`"

if arx valid-package-name --tolerant "$A" ; then

  A_is=revision

elif test -d "$A" ; then

  A_is=tree
  A="`abs_dir \"$A\"`"

else

  set +e
  arx valid-package-name --tolerant --errname delta-patch "$A"
  exit 2

fi

if arx valid-package-name --tolerant "$B" ; then

  B_is=revision

elif test -d "$B" ; then

  B_is=tree
  B="`abs_dir \"$B\"`"

else

  set +e
  arx valid-package-name --tolerant --errname delta-patch "$B"
  exit 2

fi

if test -z "$in_place" && arx valid-package-name --tolerant "$C" ; then

  C_is=revision

elif test -d "$C" ; then

  C_is=tree
  C="`abs_dir \"$C\"`"


elif test -z "$in_place" ; then

  set +e
  arx valid-package-name --tolerant --errname delta-patch "$C"
  exit 2

else

  printf "\\n" 1>&2
  printf "delta-patch: output directory does not exist\\n" 1>&2
  printf "  dir %s\\n" "$output_dir"1>&2
  printf "\\n" 1>&2
  printf "Try \"delta-patch --help\"\\n" 1>&2
  printf "\\n" 1>&2
  exit 2

fi


################################################################
# Greetings
# 

if test $verbosity -ge $report ; then
  arx heading "delta-patch\\n"
  printf "arguments: %s\\n"  "$command_line" | fold -w 60 | arx body-indent
  if test $verbosity -ge $verbose ; then
    arx heading --sub "delta-patch start time: %s\\n" "`date`"
    arx heading --sub "from: %s" "$A" 
    arx heading --sub "to: %s" "$B" 
    arx heading --sub "upon: %s" "$C" 
    if test -z "$in_place" ; then
      arx heading --sub "output to: %s\\n" "$output_dir"
    else
      arx heading --sub "NOTE: modifying tree in place (--in-place)\\n"
      arx heading --sub --sub "tree: %s\\n" "$C"
    fi
  fi
fi


################################################################
# Make a Temporary Directory
# 

tmpdirparent="`pwd`"
tmpdir="$tmpdirparent/,,delta-patch.$$"

bail()
{
  cd "$tmpdirparent"
  rm -rf "$tmpdir"
  exit 2
}

trap "printf \"delta-patch: interrupted -- cleaning up\\n\" 1>&2 ; bail" INT
rm -rf "$tmpdir"
mkdir "$tmpdir"

################################################################
# Find or Make Trees for From and To
# 

if test "$A_is" = tree ; then
  A_tree="$A"

elif arx library-find --silent "$A" ; then

  A_tree="`arx library-find "$A"`"

else

  cd "$tmpdir"
  mkdir A
  cd A
  arx nested arx build-revision --cache "$here" --quiet $verbose_opt $debug_opt -- "$A"
  A_tree="$tmpdir/A"

fi

if test "$B_is" = tree ; then

  B_tree="$B"

elif arx library-find --silent "$B" ; then

  B_tree="`arx library-find "$B"`"

else

  cd "$tmpdir"
  mkdir B
  cd B
  arx nested arx build-revision --cache "$here" --quiet $verbose_opt \
      $debug_opt -- "$B"
  B_tree="$tmpdir/B"

fi

################################################################
# Compute the Patch Set
# 

cd "$tmpdir" 
arx nested arx mkpatch --quiet $verbose_opt $debug_opt "$A_tree" "$B_tree" the-patch

patch="$tmpdir/the-patch"

################################################################
# Establish the Output Directory
# 

if test -z "$in_place" ; then

  if test "$C_is" = tree ; then

    mkdir "$output_dir"
    cd "$C"
    arx inventory --source --precious --control --nested --files \
    | $ARCH_AWK '{ print $2 }' \
    | copy-file-list -- - . "$output_dir"

  else

    arx nested arx get  --quiet $verbose_opt $debug_opt "$archive" -- "$C" "$output_dir"

  fi

fi

################################################################
# Apply the Patch
# 

set +e
arx nested arx dopatch "$patch" "$output_dir"
patch_status=$?
set -e

################################################################
# Bye (with Patch Status)
# 

cd "$tmpdirparent"
rm -rf "$tmpdir"

if test $patch_status -eq 1 ; then
  arx heading --sub "NOTICE: merge conflicts occured\\n"
  printf "  Look for \".rej\" files in %s\\n" "`basename \"$output_dir\"`" | arx body-indent --sub
fi

if test $verbosity -ge $verbose ; then
  arx heading --sub "delta-patch finish time: %s\\n" "`date`"
fi

exit "$patch_status"

# tag: Tom Lord Sat Jan  5 12:49:36 2002 (branching-and-merging/delta-patch.sh)
#
