# config-history.sh: report the history of a configuration
# 
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the
#                                  University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 
command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "report the history of a configuration\\n"
                printf "usage: config-history [options] config-name [archive/]config-package"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " -r --reverse                  list revisions from newest to oldest\\n"
                printf "\\n"
                printf "Report the history of a particular configuration in a\\n"
                printf "particular branch or version.\\n"
                printf "\\n"
                printf "The second parameter, CONFIG-PACKAGE, may be a category,\\n"
                printf "branch, version, or revision of a project tree which contains\\n"
                printf "a config directory (see \"arx build-config --help\").\\n"
                printf "\\n"
                printf "For all indicated revisions of the CONFIG-PACKAGE (e.g., all in\\n"
                printf "the category, or all in a particular branch), a series of output\\n"
                printf "lines are produced, each having the format:\\n"
                printf "\\n"
                printf "        \"%%s\\t%%s\" \$config_revision \$package_revision \\n"
                printf "\\n"
                printf "where \"\$package_revision\" is a revision spec of a nested project\\n"
                printf "tree from the config file \"\$config_revision\" of CONFIG-PACKAGE.\\n"
                printf "\\n"
                printf "One additional line is produced with the format:\\n"
                printf "\\n"
                printf "        \"%%s\\t%%s\" \$config_revision \$package_revision \\n"
                printf "\\n"
                printf "Note that this command obtains its data from your revision library,\\n"
                printf "not from the archive directly.  (See \"arx my-revision-library --help.)\"\\n"
                printf "\\n"
                exit 0
                      ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=
reverse=

while test $# -ne 0 ; do

  case "$1" in 

    -r|--reverse)        shift
                            reverse=-r
                        ;;

    --)                        shift
                            break
                        ;;

    -*)                        printf "config-history: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                        break
                            ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 2 ; then
  printf "usage: config-history [options] config-name [archive/]config-package\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

config_name="$1"
shift

config_package="$1"
shift

################################################################
# Sanity Check and Process Defaults
# 

arx my-revision-library -e config-history > /dev/null

arx valid-package-name -e config-history --basename --tolerant "$config_package"


archive="`arx parse-package-name --arch \"$config_package\"`"
category="`arx parse-package-name --basename \"$config_package\"`"

if arx valid-package-name --tolerant "$config_package" ; then
  branch="`arx parse-package-name \"$config_package\"`"
else
  branch=
fi

if arx valid-package-name --vsn --tolerant "$config_package" ; then
  version="`arx parse-package-name --package-version \"$config_package\"`"
else
  version=
fi

if arx valid-package-name --lvl "$config_package" ; then
  revision="`arx parse-package-name --non-arch \"$config_package\"`"
else
  revision=
fi

arx valid-config-name -e config-history "$config_name"

################################################################
# Iterate Over Selected Revisions
#

if test -z $branch ; then 
  branches="`arx library-branches \"$archive/$category\"`"
else
  branches="$branch"
fi

for branch in $branches ; do

  if test -z "$version" ; then 
    versions="`arx library-versions $reverse \"$archive/$branch\"`"
  else
    versions="$version"
  fi

  for version in $versions ; do

    if test -z "$revision" ; then 
      revisions="`arx library-revisions --full $reverse \"$archive/$version\"`"
    else
      revisions="$revision"
    fi

    for revision in $revisions ; do

      printf "%s\\t%s\\n" "$revision" "$archive/$revision" 

      if arx library-file --silent "configs/$config_name" "$archive/$revision" > /dev/null ; then
        arx cat-library-file  "configs/$config_name" "$archive/$revision" \
        | arx file-syntax-filter --sh-comments --blank-lines --trailing-spaces \
        | $ARCH_AWK -v revision="$revision" '{print revision "\t" $2}'
        
      fi

    done


  done


done




# tag: Tom Lord Sat Jan 26 18:43:39 2002 (configurations/config-history.sh)
#
