# record-config.sh: record a revision-specific configuration
# 
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the
#                                  University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 
command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "record a revision-specific configuration\\n"
                printf "usage: record-config [options] config-name new-config-name\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf " -d --dir DIR                  cd to DIR first\\n"
                printf " --config-dir CFG              get config from directory CFG\\n"
                printf "\\n"
                printf " -f --force                    overwrite an existing config\\n"
                printf "\\n"
                printf "Record a new configuration based on the old one.\\n"
                printf "\\n"
                exit 0
                      ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

dir=.
config_dir=

silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=

force=

while test $# -ne 0 ; do

  case "$1" in 

    -f|--force) shift
                    force=--force
                ;;

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "record-config: debugging output enabled\\n"
                    set -x
                debug_opt=--debug
                ;;

    -d|--dir)                shift
                            if test $# -eq 0 ; then
                          printf "record-config: -d and --dir require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        dir=$1
                        shift
                        ;;

    --config-dir)       shift
                        if test $# -eq 0 ; then
                          printf "record-config: --config-dir requires an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        config_dir="$1"
                        shift
                        ;;

    --)                        shift
                            break
                        ;;

    -*)                        printf "record-config: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                        break
                            ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 2 ; then
  printf "usage: record-config [options] config-name new-config-name\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

name="$1"
shift
new_name="$1"
shift

################################################################
# Sanity Check and Process Defaults
# 

cd "$dir"
dir="`pwd`"

wdroot="`arx tree-root --accurate`"
cd "$wdroot"

config_file="`arx indicated-config-file -e record-config --config-dir \"$config_dir\" --dir \"$wdroot\" --exists \"$name\"`"
new_config_file="`arx indicated-config-file -e record-config --config-dir \"$config_dir\" --dir \"$wdroot\" --new $force \"$new_name\"`"

################################################################
# Greetings
# 

if test $verbosity -ge $report ; then
  arx heading "record-config\\n"
  printf "arguments: %s\\n" "$command_line" | fold -w 60 | arx body-indent
  if test $verbosity -ge $verbose ; then
    arx heading --sub "record-config start time: %s\\n" "`date`"
    arx heading --sub "project tree: %s\\n" "$wdroot"
    arx heading --sub "input configuration: %s\\n" "$name"
    cat "$config_file" | arx body-indent --sub
  fi
fi

################################################################
# Compute the New Configuration
# 

cd "$wdroot"

if test $verbosity -ge $verbose ; then
  arx heading --sub "output configuration: %s\\n" "$new_name"
fi

rm -f "$new_config_file"

if test $verbosity -ge $default_output ; then
  output_filter="tee \"$new_config_file\" | arx body-indent --sub"
else
  output_filter="cat > \"$new_config_file\" "
fi

( \
  printf "#\\n" ; \
  printf "# automatically generated configuration\\n" ; \
  printf "#\\n" ; \
  printf "#  derived from %s\\n" "$name" ; \
  printf "#\\n" ; \
  printf "\\n" ; \
  
  cat "$config_file" \
  | arx file-syntax-filter --sh-comments --blank-lines --trailing-spaces \
  | sort -k 1 \
  | $ARCH_AWK -v here="$wdroot" \
        -v verbosity=$verbosity \
        -v quiet=$quiet \
       '{
          print "cd \"" here "\"" ; 
          print "printf \"%-30s\\t\" \"" $1 "\""  ;
          print "cd \"" $1 "\"" ; 
          print "x=\"`arx log-ls --full --reverse \\\"" $2 "\\\"`\"" ; 
          print "if test -z \"$x\" ; then"
          print "  if test $verbosity -ge $quiet ; then
          print "    printf \"record-config: no log entries for %s\\\\n\" \"" $2 "\" 1>&2"
          print "  fi
          print "  exit 1"
          print "else"
          print "  arx log-ls --full --reverse \"" $2 "\" | head -1"
          print "fi"
        }' \
  | $ARCH_SHELL -e \
) \
| $output_filter

if test $verbosity -ge $verbose ; then
  arx heading --sub "record-config finish time: %s\\n" "`date`"
fi


# tag: Tom Lord Sun Dec 30 17:46:10 2001 (configurations/record-config.sh)
#
