# 
# update-config.sh: update a multi-project tree
# 
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the
#                                  University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 
command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
          printf "update a multi-project tree\\n"
          printf "usage: update-config [options] config-name\\n"
          printf "\\n"
          printf " -V --version                  print version info\\n"
          printf " -h --help                     display help\\n"
          printf "\\n"
          printf " --silent                      no output\\n"
          printf " --quiet                       only output errors\\n"
          printf " --default-output              default output\\n"
          printf " --report                      slightly verbose output\\n"
          printf " --verbose                     maximal output\\n"
          printf " --debug                       debugging output\\n"
          printf "\\n"
          printf " -d --dir DIR                  cd to DIR first\\n"
          printf " --config-dir CFG              get config from directory CFG\\n"
          printf "\\n"
          printf " --force                       pass the --force flag to update\\n"
          printf "\\n"
          printf "Update the named configuration based on the specification\\n"
          printf "in the CFG directory (or \"configs\" subdirectory\\n"
          printf "at the root of the project tree containing DIR).\\n"
          printf "\\n"
          exit 0
          ;;

      *)
          ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

dir=.
config_dir=

silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=

force_opt=

while test $# -ne 0 ; do

  case "$1" in 

    --force)    shift
                force_opt=--force
                ;;

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "update-config: debugging output enabled\\n"
                    set -x
                debug_opt=--debug
                ;;

    -d|--dir)                shift
                            if test $# -eq 0 ; then
                          printf "update-config: -d and --dir require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        dir=$1
                        shift
                        ;;

    --config-dir)       shift
                        if test $# -eq 0 ; then
                          printf "update-config: --config-dir requires an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        config_dir="$1"
                        shift
                        ;;

    --)                        shift
                            break
                        ;;

    -*)                        printf "update-config: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                        break
                            ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 1 ; then
  printf "usage: update-config [options] name\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

name="$1"
shift

################################################################
# Sanity Check and Process Defaults
# 

cd "$dir"
dir="`pwd`"

wdroot="`arx tree-root --accurate`"
cd "$wdroot"

config_file="`arx indicated-config-file -e update-config --config-dir \"$config_dir\" --dir \"$wdroot\" --exists \"$name\"`"

################################################################
# Greetings
# 

if test $verbosity -ge $report ; then
  arx heading "update-config\\n"
  printf "arguments: %s\\n" "$command_line" | fold -w 60 | arx body-indent
  arx heading --sub "update-config start time: %s\\n" "`date`"
  arx heading --sub "project tree: %s\\n" "$wdroot"
  arx heading --sub "configuration: %s\\n" "$name"
  cat "$config_file" | arx body-indent --sub
fi

################################################################
# Update Requested Directories
# 

cd "$wdroot"

cat "$config_file" \
| arx file-syntax-filter --sh-comments --blank-lines --trailing-spaces \
| sort -k 1 \
| $ARCH_AWK -v debug_opt="$debug_opt" \
      -v force_opt="$force_opt" \
      -v verbose_opt="$verbose_opt" \
      '{
          printf("arx nested --sub arx update --quiet %s %s %s  --in-place -- %s %s\n", force_opt, debug_opt, verbose_opt, $1, $2);
       }' \
| $ARCH_SHELL

# tag: Tom Lord Sat Dec 29 23:20:42 2001 (configurations/update-configuration.sh)
#
