# my-revision-library.sh: the user's revision library
# 
################################################################
# Copyright (C) 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the
#                                  University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "print or set your revision library path\\n"
                printf "usage: my-revision-library [options] [dir]\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " -e --errname prog             specify program name for errors\\n"
                printf " -d --delete                   unspecify your library location\\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf "With no argument, and without -d, print the path to your revision\\n"
                printf "library.\\n"
                printf "\\n"
                printf "With an argument, record DIR as the path to your revision library\\n"
                printf "in ~/.arch-params/=revision-library\\n"
                printf "\\n"
                printf "With the option -d (--delete) and no argument, ensure that\\n"
                printf "you do not have a revision library path set in ~/.arch-params.\\n"
                printf "\\n"
                printf "If no revision library is set, the program exits an error.\\n"
                printf "\\n"
                exit 0
                      ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=

delete=
errname=my-revision-library

while test $# -ne 0 ; do

  case "$1" in 

    -e|--errname)        shift
                        if test $# -eq 0 ; then
                          printf "my-revision-library: -e and --errname require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        errname="$1"
                        shift
                        ;;

    -d|--delete)        shift
                            delete=--delete
                        ;;

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "my-revision-library: debugging output enabled\\n"
                    set -x
                ;;

    -*)                        printf "my-revision-library: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                        break
                            ;;

  esac

done



################################################################
# Ordinary Arguments
# 

if test ! -z "$delete" ; then

  if test $# -ne 0 ; then
    printf "usage: my-revision-library [options] -d|--delete\\n" 1>&2
    printf "try --help\\n" 1>&2
    exit 1
  fi

  type=delete

else

  case $# in 

    0)        type=print
        ;;

    1)        type=set
          newdflt="$1"
        # If the new revision library is an empty string,
        # then just print the archive
        if test -z "$newdflt" ; then
            type=print
        fi
        ;;

    *)        if test $# -gt 1 ; then
          printf "usage: my-revision-library [options] [dir]\\n" 1>&2
          printf "try --help\\n" 1>&2
          exit 1
        fi
        ;;
  esac

fi


################################################################
# Sanity Check
# 

if test $type = set ; then
  if ! test -d "$newdflt" ; then
    if test $verbosity -ge $quiet ; then
      printf "\\n" 1>&2
      printf "my-revision-library: path is not a directory\\n" 1>&2
      printf "  path: %s\\n" "$newdflt" 1>&2
      printf "\\n" 1>&2
    fi
    exit 1
  fi
fi


################################################################
# Do It
# 

case $type in

  delete)        mkdir -p ~/.arch-params
                  cd ~/.arch-params

                if test -e =revision-library ; then
                  rm =revision-library
                 fi

                if test $verbosity -ge $report ; then
                  printf "revision library location removed\\n"
                fi

                exit 0
                ;;

  set)                mkdir -p ~/.arch-params/
                  printf "%s\\n" "$newdflt" > ~/.arch-params/=revision-library

                if test $verbosity -ge $report ; then
                  printf "revision library location set (%s)\\n" "$newdflt"
                fi

                exit 0
                ;;

  print)        if test ! -e ~/.arch-params/=revision-library ; then
                  if test $verbosity -ge $quiet ; then
                    printf "\\n" 1>&2
                    printf "%s: no revision library location set\\n" "$errname" 1>&2
                    printf "  try \"arx my-revision-library --help\"\\n" 1>&2
                    printf "\\n" 1>&2
                  fi
                  exit 1
                fi

                if test $verbosity -ge $default_output ; then
                  cat ~/.arch-params/=revision-library
                fi
                ;;


esac

# tag: Tom Lord Sat Jan 19 21:43:52 2002 (library/my-revision-library.sh)
#
