# 
# copy-from-cache.sh - try to copy a revision from the cache
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the University
#                          of California
# Minor bug fix 2003 Chris Gray
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "try to copy a revision from a cache\\n"
                printf "usage: copy-from-cache [options] cachedir revision\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf "Look for a cached copy of REVISION in the project tree\\n"
                printf "which are immediate subdirectories of CACHEDIR.  If one is found,\\n"
                printf "copy it to the current directory and exit with status 0.  Otherwise,\\n"
                printf "exit with status 1.\\n"
                printf "\\n"
                exit 0
                      ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=

silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=

while test $# -ne 0 ; do

  case "$1" in 


    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "copy-from-cache: debugging output enabled\\n"
                    set -x
                    debug_opt=--debug
                    ;;

    --)                        shift
                        break
                        ;;

    -*)                        printf "copy-from-cache: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                        break
                            ;;

  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 2 ; then
  printf "usage: copy-from-cache [options] cachedir revision\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

cachedir="$1"
shift

revision="$1"
shift

################################################################
# Sanity Check and Process Defaults
# 

arx valid-package-name -l -e copy-from-cache -- "$revision"

archive=`arx parse-package-name --arch $revision`
category=`arx parse-package-name -b $revision`
branch=`arx parse-package-name $revision`
vsn=`arx parse-package-name -v $revision`
lvl=`arx parse-package-name -l $revision`

here="`pwd`"
cd "$cachedir"
cachedir="`pwd`"

################################################################
# Greetings
# 
#  This is an internal command.

if test $verbosity -ge $report; then
  ARCH__OUTLINE_DEPTH="$ARCH__OUTLINE_DEPTH*"
  arx heading "copy-from-cache\\n"
  printf "arguments: %s\\n" "$command_line" | fold -w 60 | arx body-indent
  if test $verbosity -ge $verbose; then
    arx heading --sub "copy-from-cache start time: %s\\n" "`date`"
    arx heading --sub "seeking revision: %s\\n" "$branch--$vsn--$lvl"
    arx heading --sub "to copy to: %s\\n" "$here"
    arx heading --sub "cache directory: %s\\n" "$cachedir"
  fi
fi

################################################################
# Look for Cached Copies
# 


for wd in [=a-zA-Z0-9+]* ; do
  for state in locked unlocked ; do
    if test -d "$wd/{arch}/++pristine-trees/$state/$category/$branch/$branch--$vsn/$archive/$branch--$vsn--$lvl" ; then
  
      if test $verbosity -ge $report; then
        arx heading --sub "copying revision from local cache: %s\\n" $branch--$vsn--$lvl
	arx heading --sub --sub "from project tree: %s\\n" "$wd"
      fi
  
      cd "$wd/{arch}/++pristine-trees/$state/$category/$branch/$branch--$vsn/$archive/$branch--$vsn--$lvl"
      
      find . | copy-file-list -- - . "$here"
  
      if test $verbosity -ge $verbose; then
        arx heading --sub "copy-from-cache finish time: %s\\n" "`date`"
      fi
  
      exit 0
  
    fi
  done
done

if test $verbosity -ge $quiet; then
  arx heading --sub "no cached copy found\\n"
  if test $verbosity -ge $verbose; then
    arx heading --sub "copy-from-cache finish time: %s\\n" "`date`"
  fi
fi

exit 1

