# 
# old-file.sh: compare file with pristine tree
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the
#                                  University of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#       --version | -V
#       --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "compare file with cached pristine revision\\n"
                printf "usage: old-file [options] file [[archive/]revision]\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf "  --cache DIR                  cache directory for locally cached\\n"
                printf "\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf "Write the contents of the version of FILE found in a cached pristine\\n"
                printf "copy of REVISION.\\n"
                printf "\\n"
                printf "(See \"arx add-pristine\".)\\n"
                exit 0
                ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archroot=
archive=
cache_dir=
html=

debug_opt=

while test $# -ne 0 ; do

  case "$1" in 

    --debug)        shift
                    arx heading "old-file: debugging output enabled\\n"
                    set -x
                debug_opt=--debug
                ;;

    --cache)            shift
                        if test $# -eq 0 ; then
                          printf "old-file: --cache requires an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 2
                        fi
                        cache_dir="$1"
                        shift
                        ;;

    --)                        shift
                            break
                        ;;

    -*)                 printf "old-file: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 2
                        ;;

    *)                  break
                        ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -lt 1 -o  $# -gt 2 ; then
  printf "usage: old-file [options] file [[archive/]revision]\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 2
fi

file="$1"
shift

if test $# -ne 0 ; then
  rvnspec="$1"
  shift
else
  rvnspec=
fi

################################################################
# Sanity Check and Process Defaults
# 


dir="`dirname \"$file\"`"

cd "$dir"
dir="`pwd`"
wdroot="`arx tree-root`"

file_base="`basename \"$file\"`"
rel_path=".${dir#$wdroot}/$file_base"

cd "$wdroot"

if test -z "$rvnspec" ; then
  rvnspec="`arx tree-version`"
fi

arx valid-package-name -e old-file -t "$rvnspec"

archive="`arx parse-package-name --arch \"$rvnspec\"`"
category="`arx parse-package-name --basename \"$rvnspec\"`"
branch="`arx parse-package-name --package \"$rvnspec\"`"

if arx valid-package-name --vsn -t "$rvnspec" ; then
  version="`arx parse-package-name --package-version \"$rvnspec\"`"
else
  version="`arx logs -r \"$branch\" | head -1`"
  if test -z "$version" ; then
    printf "old-file: no logs for that branch\\n" 1>&2
    printf "  branch: %s\\n" "$branch" 1>&2
    printf "\\n"
    exit 2
  fi
  version="`arx parse-package-name --package-version \"$version\"`"
fi

if arx valid-package-name --patch-level -t "$rvnspec" ; then
  revision="`arx parse-package-name --non-arch \"$rvnspec\"`"
else
  lvl="`arx log-ls -r \"$version\" | head -1`"
  if test -z "$lvl" ; then
    printf "old-file: no patch level for that version\\n" 1>&2
    printf "  branch: %s\\n" "$version" 1>&2
    printf "\\n"
    exit 2
  fi
  revision="$version--$lvl"
fi

if test -z "$cache_dir" ; then
  cd "$wdroot/.."
  cache_dir="`pwd`"
  cd "$wdroot"
fi

if arx library-find --silent $verbose_opt $debug_opt "$archive/$revision" ; then

  revision_tree="`arx library-find \"$archive/$revision\"`"
  revision_index="$revision_tree/,,index"

else
  set +e
  revision_tree="`arx find-in-cache \"$cache_dir\" \"$archive/$revision\"`"
  status=$?
  set -e

  if test $status -ne 0 ; then
    printf "old-file: revision not in library or local cache\\n" 1>&2
    printf "  revision: %s\\n" "$revision" 1>&2
    printf "\\n" 1>&2
    printf "See \"arx add-pristine\"\\n" 1>&2
    printf "\\n" 1>&2
    exit 2
  fi

  revision_index="`arx cached-index \"$revision_tree\"`"

fi



################################################################
# Identify the File
# 

tagging_method="`arx tagging-method`"

set +e

cd "$wdroot"

case "$tagging_method" in

  names)        tag="?$rel_path"
                    status=0
                  ;;

  implicit)        tag="`file-tag --implicit \"$rel_path\"`"
                  status=$?
                tag="`printf \"%s\\\\n\" \"$tag\" | cut -s -f 2`"
                ;;

  explicit)        tag="`file-tag --explicit \"$rel_path\"`"
                  status=$?
                tag="`printf \"%s\\\\n\" \"$tag\" | cut -s -f 2`"
                ;;

  *)                printf "old-file: internal error\\n" 1>&2
                  printf "  unrecognized tagging method (%s)\\n" "$tagging_method" 1>&2
                printf "\\n" 1>&2
                exit 2
                ;;
esac

if test "$status" -ne 0 ; then
  printf "old-file: unable to compute file tag\\n" 1>&2
  printf "  file: %s\\n" "$wdroot/$rel_path" 1>&2
  printf "\\n" 1>&2
  exit 2
fi

################################################################
# Find the Corresponding File
# 

revision_path="`printf \"%s\\n\" \"$tag\" | join -o 2.1 -1 1 -2 2 - \"$revision_index\"`"


################################################################
# Output Old File
# 

cat "$revision_tree/$revision_path"

# tag: Tom Lord Fri Dec 14 03:01:38 2001 (local-cache/old-file.sh)
#
