# 
# log-ls.sh - print version patches recorded for a project tree
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents
#                          of the University of California
# Minor bug fix 2003 Stig Brautaset
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

################################################################
# special options
# 
# Some options are special:
# 
#        --version | -V
#        --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "print version patches in a project tree\\n"
                printf "usage: log-ls [options] [[archive/]version] ...\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " -d --dir DIR                  cd to DIR first\\n"
                printf "\\n"
                printf " -r --reverse                  sort from newest to oldest\\n"
                printf " -s --summary                  print a summary of each patch\\n"
                printf " -c --creator                  print the creator id of each patch\\n"
                printf " -D --date                     print the date of each patch\\n"
                printf " --full                        print full patch level names\\n"
                printf "\\n"
                printf "Print the list of patches applied to a project tree\\n"
                printf "DIR (or the current directory) from VERSION."
                printf "\\n"
                printf "Patch names are printed simply:\\n"    
                printf "\\n"
                printf "        \"%%s\\\\n\"  \$patch_level\\n"
                printf "\\n"
                exit 0
                      ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archroot=
archive=
reverse=
summary=
creator=
date=
full=
dir=
debug_opt=

while test $# -ne 0 ; do

  case "$1" in 

    --debug)                shift
                            debug_opt=--debug
                        printf "\n" 1>&2
                        printf "log-ls: DEBUGGING ACTIVATED\n" 1>&2
                        printf "\n" 1>&2
                        set -x
                        ;;

    -d|--dir)           shift
                        if test $# -eq 0 ; then
                          printf "log-ls: -d and --dir require an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        dir="$1"
                        shift
                        ;;

    -r|--reverse)        shift
                            reverse=-r
                        ;;

    -s|--summary)        shift
                            summary=--summary
                        ;;

    -c|--creator)        shift
                            creator=--creator
                        ;;

    -D|--date)                shift
                            date=--date
                        ;;

    --full)                shift
                            full=--full
                        ;;

    --)                        shift
                            break
                        ;;

    ---)                break
                            ;;

    -*)                        printf "log-ls: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                        break
                            ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ge 1 ; then
  archive_version="$1"
  shift
else
  archive_version=
fi


################################################################
# Sanity Check and Process Defaults
# 

if test "$archive_version" = --- ; then
  exit 0
fi

if test -z "$dir" ; then
  dir="."
fi

cd "$dir"
dir="`pwd`"

cd "$dir"
wdroot="`arx tree-root --accurate`"
cd "$wdroot"

if test -z "$archive_version" ; then
  archive_version=`arx tree-version`
else
  arx valid-package-name -e log-ls --tolerant -- "$archive_version"
fi

archive=`arx parse-package-name --arch $archive_version`
spec=`arx parse-package-name --non-arch $archive_version`
category=`arx parse-package-name --basename $spec`
branch=`arx parse-package-name $spec`

if arx valid-package-name --vsn --tolerant "$spec" ; then
  version="`arx parse-package-name --package-version \"$spec\"`"
else
  cd "$wdroot"
  version="`arx logs $debug_opt --reverse -a \"$archive\" -b \"$branch\" | head -1`"
  if test -z "$version" ; then
    printf "\\n" 1>&2
    printf "log-ls: no patch logs for branch\\n" 1>&2
    printf "  tree: %s\\n" "$wdroot" 1>&2
    printf "  branch: %s\\n" "$archive/$branch" 1>&2
    printf "\\n" 1>&2
    exit 1
  else
    version="`arx parse-package-name --non-arch \"$version\"`"
  fi
fi

if arx valid-package-name --lvl "$spec" ; then
  lvl="`arx parse-package-name --lvl \"$spec\"`"
else
  lvl=[0-9]
fi


################################################################
# Print the list of patches as patch-level names.
# 

cd "$wdroot/{arch}"

logdir="$category/$branch/$version/$archive/patch-log"

if test ! -d "$logdir" ; then
  printf "log-ls: project tree has no patch-log for version\\n" 1>&2
  printf "\\n" 1>&2
  printf "  project tree: %s\\n" "$wdroot" 1>&2
  printf "  version: %s\\n" "$version" 1>&2
  printf "  archive: %s\\n" "$archive" 1>&2
  printf "\\n" 1>&2
  printf "try \"arx add-log --help\"\\n" 1>&2
  exit 1
fi

patchlvlre="base-0|patch-[0-9]+|version-0|versionfix-[0-9]+"

cd "$logdir"

if test $# -gt 0 -a -z "$full" ; then
  printf "%s:\\n" "$archive/$version"
  printf "\\n"
fi


if test -z "$summary" -a -z "$creator" -a -z "$date" ; then

  if test -z "$full" ; then
    ls | grep -E "^($patchlvlre)\$" | grep -e "$lvl$" | sort -t- -k 1,1${reverse#-} -k 2,2${reverse#-}n 
  else
    ls | grep -E "^($patchlvlre)\$" | grep -e "$lvl$" | sort -t- -k 1,1${reverse#-} -k 2,2${reverse#-}n \
    | $ARCH_AWK -v arch="$archive" -v vsn="$version" '{ printf("%s/%s--%s\n", arch, vsn, $1); }'
  fi



else

  for patch in `ls | grep -E "^($patchlvlre)\$" | grep -e "$lvl$" | sort -t- -k 1,1${reverse#-} -k 2,2${reverse#-}n` ; do

    if test -z "$full" ; then
      printf "%s\\n" "$patch"
    else
      printf "%s/%s--%s\\n" "$archive" "$version" "$patch"
    fi

    cat $patch \
    | $ARCH_AWK -v summary="$summary" \
          -v creator="$creator" \
          -v date="$date" \
          -f "$ARCH_SUBCMD_ROOT/patch-logs/patch-list-description.awk"

  done

fi

if test $# -gt 0 ; then
  if test -z "$full" ; then
    printf "\\n"
    printf "\\n"
  fi
  arx log-ls $debug_opt --dir "$dir" $reverse $summary $full "$@" ---
fi

