# update-distributions.sh: build or update an FTP area
# 
################################################################
# Copyright (C) 2001, 2002 Tom Lord
# Copyright (C) 2002, 2003 Walter Landry and the Regents of the University
# of California
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
# 

set -e 

command_line="$*"

################################################################
# special options
# 
# Some options are special:
# 
#       --version | -V
#       --help | -h
# 
if test $# -ne 0 ; then

  for opt in "$@" ; do
    case $opt in

      --version|-V) exec arx --version
                    ;;


      --help|-h)
                printf "build or update a tar ball\\n"
                printf "usage: update-distributions [options] [archive/]revision dist-name dir\\n"
                printf "\\n"
                printf " -V --version                  print version info\\n"
                printf " -h --help                     display help\\n"
                printf "\\n"
                printf " --control                     include control files in the tar file.\\n"
                printf " --changelog                   include ChangeLog's in the tar file.\\n"
                printf " --cache DIR                   cache directory for locally cached\\n"
                printf "                               revisions\\n"
                printf " --config NAME                 after checking out the revision, pass\\n"
                printf "                               NAME to \"build-config\" to complete\\n"
                printf "                               the source tree.\\n"
                printf "\\n"
                printf " --silent                      no output\\n"
                printf " --quiet                       only output errors\\n"
                printf " --default-output              default output\\n"
                printf " --report                      slightly verbose output\\n"
                printf " --verbose                     maximal output\\n"
                printf " --debug                       debugging output\\n"
                printf "\\n"
                printf "Ensure that DIR exists and is populated with the indicated revision\\n"
                printf "from the archive.\\n"
                printf "\\n"
                printf "The tar file will be named:\\n"
                printf "\\n"
                printf "        DIST-NAME-\${VERSION}\${SUFFIX}\\n"
                printf "\\n"
                printf "where VERSION is the version number of the revision being packaged, and\\n"
                printf "SUFFIX is derived from the patch-level name of the revision.\\n"
                printf "\\n"
                printf "The value of \${SUFFIX} depends on the revision's patch level, as in\\n"
                printf "these examples:\\n"
                printf "\\n"
                printf "        revision:                        	tar file:\\n"
                printf "\\n"
                printf "        arch--release--1.0--base-0                arch-1.0pre0.tar.gz\\n"
                printf "        arch--release--1.0--patch-3                arch-1.0pre3.tar.gz\\n"
                printf "        arch--release--1.0--version-0                arch-1.0.tar.gz\\n"
                printf "        arch--release--1.0--versionfix-5        arch-1.0.5.tar.gz\\n"
                printf "\\n"
                printf "The top-level directory in the tar file has the same name as the tar\\n"
                printf "file (but without the \".tar.gz\" suffix).\\n"
                printf "\\n"
                exit 0
                ;;

      *)
                ;;
    esac
  done
fi

################################################################
# Ordinary Options
# 
# 

archive=
cache_dir=

config=

control_opt=
changelog_opt=

silent=1
quiet=2
default_output=3
report=4
verbose=5

verbosity=$default_output
verbose_opt=

debug_opt=
__restart=

while test $# -ne 0 ; do

  case "$1" in 

    ----restart) shift
                    __restart=----restart
                ;;

    --silent)           shift
                        verbosity=$silent
                        verbose_opt=--silent
                        ;;

    --quiet)            shift
                        verbosity=$quiet
                        verbose_opt=--quiet
                        ;;

    --default-output)   shift
                        verbosity=$default_output
                        verbose_opt=--default-output
                        ;;

    --report)           shift
                        verbosity=$report
                        verbose_opt=--report
                        ;;

    --verbose)          shift
                        verbosity=$verbose
                        verbose_opt=--verbose
                        ;;

    --debug)        shift
                    arx heading "update-distributions: debugging output enabled\\n"
                    set -x
                debug_opt=--debug
                ;;

    --control)          shift
                        control_opt=--control
                        ;;

    --changelog)          shift
                        changelog_opt=--changelog
                        ;;

    --cache)            shift
                        if test $# -eq 0 ; then
                          printf "update-distributions: --cache requires an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        cache_dir="$1"
                        shift
                        ;;

    --config)           shift
                        if test $# -eq 0 ; then
                          printf "update-distributions: --config requires an argument\\n" 1>&2
                          printf "try --help\\n" 1>&2
                          exit 1
                        fi
                        config="$1"
                        shift
                        ;;

    --)                        shift
                            break
                        ;;

    -*)                 printf "update-distributions: unrecognized option (%s)\\n" "$1" 1>&2
                        printf "try --help\\n" 1>&2
                        exit 1
                        ;;

    *)                  break
                        ;;
  esac

done



################################################################
# Ordinary Arguments
# 

if test $# -ne 3 ; then
  printf "usage: update-distributions [options] [archive/]revision dist-name dir\\n" 1>&2
  printf "try --help\\n" 1>&2
  exit 1
fi

arch_rev_spec="$1"
shift

dist_name="$1"
shift

ftp_dir="$1"
shift

################################################################
# Sanity Check and Process Defaults
# 
  
here="`pwd`"
mkdir -p "$ftp_dir"
cd "$ftp_dir"
ftp_dir="`pwd`"
cd "$here"

arch_rev="`arx indicated-revision -- \"$arch_rev_spec\"`"

archive="`arx parse-package-name --arch \"$arch_rev\"`"
revision="`arx parse-package-name --non-arch \"$arch_rev\"`"

################################################################
# Greetings
# 

if test $verbosity -ge $report ; then
  arx heading "update-distributions\\n"
  printf "arguments: %s\\n" "$command_line" | fold -w 60 | arx body-indent
  if test $verbosity -ge $verbose ; then
    arx heading --sub "update-distributions start time: %s\\n" "`date`"
    arx heading --sub "ftp dir: %s\\n" "$ftp_dir"
    arx heading --sub "archive: %s\\n" "$archive"
    arx heading --sub "version: %s\\n" "$version"
  fi
fi


################################################################
# Make a temp directory
# 

tmpdir="$here/,,update-distributions.$$"

bail()
{
  cd "$here"
  rm -rf "$tmpdir"
  exit 1
}

trap "printf \"update-distributions: interrupted -- cleaning up\\n\" 1>&2 ; bail" INT

rm -rf "$tmpdir"
mkdir "$tmpdir"

################################################################
# What Revision is the Latest?
# 


full_dist_name="`arx distribution-name --dist \"$dist_name\" \"$revision\"`"
tar_file="$full_dist_name.tar.gz"

################################################################
# Is it Already There?
# 

if test -e "$ftp_dir/$tar_file" ; then

  if test $verbosity -ge $default_output ; then
    arx heading --sub "latest revision already available\\n"
    printf "%s\n" "$tar_file" | arx body-indent --sub
  fi

else

  ################################################################
  # Get a Copy of the Revision
  # 
  
  cd "$tmpdir"

  if test $verbosity -ge $report ; then
    arx heading --sub "getting revision\\n"
  fi

  arx nested --sub \
       arx get --quiet $verbose_opt $debug_opt \
                --no-pristine \
               "$archive/$revision" "$full_dist_name"  

  ################################################################
  # Build a Configuration, if Requested
  # 
  if test ! -z "$config" ; then
    if test $verbosity -ge $report ; then
      arx heading --sub "building configuration: %s\\n" "$config"
    fi

    arx nested --sub \
         arx build-config --quiet $verbose_opt $debug_opt \
                 --dir "$tmpdir/$full_dist_name" \
                "$config"
  fi

  ################################################################
  # Make tar File
  # 
  
  cd "$tmpdir"
  arx inventory --source $control_opt --nested "$full_dist_name" > ,,files
  if test -n "$control_opt" ; then
    arx inventory --precious --control --nested "$full_dist_name" \
      | grep "\+\+default-version" >> ,,files
  fi

  # Add in the ChangeLog's
  if test -n "$changelog_opt" ; then
    # Do all of the subtrees
    for tree in "$full_dist_name" `arx inventory --nested --trees "$full_dist_name"` ; do
      if test -e "$tree/ChangeLog.d" ; then
        if $verbosity -ge $default_output ; then
          printf "WARNING: ChangeLog directory %s/ChangeLog.d already exists.\\n" "$tree"
          printf "Skipping this tree.\\n"
        fi
      else
        for patch_log in `arx logs -d "$tree"` ; do
          patch_archive=`parse-package-name --arch "$patch_log"`
          mkdir -p "$tree/ChangeLog.d/$patch_archive"
          arx changelog -D "$tree" "$patch_log" \
              > "$tree/ChangeLog.d/$patch_log"
          printf "%s/ChangeLog.d/%s\\n" "$tree" "$patch_log" >> ,,files
        done
      fi
    done
  fi
  
  if test $verbosity -ge $verbose ; then
    arx heading --sub "tar file"
    $ARCH_TAR -T ,,files --no-recursion -zcvf "$tar_file" "$full_dist_name" \
    | arx body-indent --sub
  else
    $ARCH_TAR -T ,,files --no-recursion -zcf  "$tar_file" "$full_dist_name" 
  fi
  
  mv "$tar_file" "$ftp_dir"

  cd "$here"
  rm -rf "$tmpdir"

fi

################################################################
# Bye
# 

if test $verbosity -ge $verbose ; then
  arx heading --sub "update-distributions finish time: %s\\n" "`date`"
fi
  
# tag: Tom Lord Thu Jan 10 06:26:49 2002 (web/update-distributions.sh)
#
