/* Add an explicit tag to a file.

   Copyright (C) 2001, 2002 Tom Lord
   Copyright (C) 2002, 2003 Walter Landry and the Regents of the
                                  University of California

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include <iostream>
#include "parse_common_options.hpp"
#include "parse_unknown_options.hpp"
#include "check_extra_args.hpp"
#include "command_initializer.hpp"
#include "get_config_option.hpp"
#include "arx_error.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/exception.hpp"
#include "boost/date_time/posix_time/posix_time.hpp"

using namespace std;
using namespace boost;
using namespace boost::posix_time;
namespace fs=boost::filesystem;
using fs::path;

int add(list<string> &argument_list, const command &cmd);

static command_initializer move_init(command("add",
"Add an explicit tag to a file",
"usage: add [options] source...",
"Add an explicit tag to a file or directory in a project tree.",
add,"Inventory",true));

int add(list<string> &argument_list, const command &cmd)
{
  Command_Info info(cmd);

  while(parse_common_options(argument_list,info))
    {}
  parse_unknown_options(argument_list,info);

  if(argument_list.empty())
    {
      throw arx_error("Need at least one file name",info);
    }

  int count=0;
  for(list<string>::iterator file=argument_list.begin();
      file!=argument_list.end(); ++file)
    {
      const path filename(*file);
      path id_directory, id_file_path;

      /* Id file location and name is different for files and
         directories. */

      if(!exists(filename))
        {
          throw arx_error("Can't add\n\t" + filename.native_file_string()
                          + "\nIt doesn't exist",info);
        }
      if(is_directory(filename))
        {
          id_directory=filename / ".arch-ids";
          id_file_path=id_directory / "=id";
        }
      else
        {
          id_directory=filename.branch_path() / ".arch-ids";
          id_file_path=id_directory / (filename.leaf() + ".id");
        }

      if(!exists(id_directory))
        {
          try
            {
              create_directory(id_directory);
            }
          catch (fs::filesystem_error &fs_error)
            {
              throw arx_error("Can't create id directory: "
                              + id_directory.native_file_string(),info);
            }
        }
      
      if(exists(id_file_path) && info.verbosity>=default_output)
        cerr << *file << " already added\n";
      else
        {
          try
            {
              fs::ofstream id_file(id_file_path);
              /* We use the id, the complete file name, the time, and
                 a count to prevent duplicate tags.  It is probably
                 better to use a hash of all of these things and a
                 random number. */
              id_file << get_config_option("=id",info) << " "
                      << fs::complete(filename).string() << " "
                      << to_iso_extended_string(ptime
                                                (second_clock::universal_time()))
                      << "." << count << endl;
              ++count;
            }
          catch (fs::filesystem_error &fs_error)
            {
              throw arx_error("Can't open id file: "
                              + id_file_path.native_file_string(),info);
            }
        }
    }
  return 0;
}
