/* Apply patches to files and directories.

  Copyright (C) 2003 Walter Landry
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 dated June, 1991.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */


#include "boost/filesystem/operations.hpp"
#include <string>
#include "Command_Info.hpp"
#include "arx_error.hpp"
#include "null_patch.hpp"
#include "../Moved_Path.hpp"
#include "inventory_types.hpp"
#include "../inventory_tag_type.hpp"
#include "Inventory.hpp"
#include "copy_missing_patch.hpp"
#include "check_symlink_hierarchy.hpp"
#include <iostream>

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

extern void patch_directory(const path &patch_dir, const path &destination,
                            const string &orig, const string &mod,
                            const Command_Info &info);

extern void patch_file(const path &patch_file, const path &destination,
                       const bitset<9> &destination_permissions,
                       const string &orig, const string &mod,
                       const string &original,
                       const string &modified,
                       const string &reverse,
                       list<pair<string,string> > &patch_conflicts,
                       const Command_Info &info);

void apply_patches(list<pair<string,string> > patched[2],
                   const Inventory &source_inventory,
                   const bool &reverse,
                   const path &patch, const path &destination,
                   list<string> &missing_patches,
                   list<pair<string,string> > &patch_conflicts,
                   const Command_Info &info)
{
  string orig("orig"), mod("mod"), original("original"), modified("modified"),
    reverse_option;
  if(reverse)
    {
      reverse_option=" --reversed";
      orig="mod";
      mod="orig";
      original="modified";
      modified="original";
    }

  for(int m=0; m<2; ++m)
    {
      for(list<pair<string,string> >::iterator i=patched[m].begin();
          i!=patched[m].end(); ++i)
        {
          if(info.verbosity>=report)
            cout << "patched " << i->first << endl;

          /* Find the path with the same tag. */
          
          inventory_types inv_type(inventory_tag_type(i->second,info));
          
          /* Don't worry about empty patches.  This is a fix for broken
             Changelogs.  */

          if(!null_patch((patch/i->first).string(),m==arx_dir))
            {
              const_Inventory_iterator
                j(find_if(source_inventory[inv_type][m].begin(),
                          source_inventory[inv_type][m].end(),
                          bind2nd(ptr_fun(Tom_encoded_tag_eq_string),
                                  i->second)));
              
              if(j==source_inventory[inv_type][m].end())
                {
                  /* Handle non-existant destinations for patches. */
                  missing_patches.push_back(i->first);
                  copy_missing_patch(i->first,patch,
                                     destination/"==missing-patches",
                                     m==arx_dir);
                }
              else
                {
                  /* Make sure we aren't applying patches through
                     symlinks */

                  check_symlink_hierarchy(destination,
                                          destination/j->file_path);

                  if(info.verbosity>report)
                    cout << "Patching: "
                         << (patch/i->first).native_file_string() << "\t"
                         << (destination/j->file_path).native_file_string()
                         << endl;

                  if(m==arx_dir)
                    {
                      /* The only thing that can happen is metadata
                         changes.  These can never conflict. */
                      patch_directory(patch/i->first,destination/j->file_path,
                                      orig,mod,info);
                    }
                  else
                    {
                      /* Don't apply ordinary patches to symlinks */
                      if(fs::exists(patch/(i->first+".patch"))
                         && !fs::is_empty(patch/(i->first+".patch"))
                         && is_link(destination/j->file_path))
                        {
                          missing_patches.push_back(i->first);
                          copy_missing_patch(i->first,patch,
                                             destination/"==missing-patches",
                                             m==arx_dir);
                        }
                      else
                        {
                          patch_file(patch/i->first,destination/j->file_path,
                                     j->permissions,
                                     orig,mod,original,modified,reverse_option,
                                     patch_conflicts,info);
                        }
                    }
                }
            }
        }
    }
}

