/* Apply a patches to one file.

  Copyright (C) 2003 Walter Landry
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 dated June, 1991.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

#include "config.h"
#include "boost/filesystem/operations.hpp"
#include <string>
#include "Command_Info.hpp"
#include "arx_error.hpp"
#include "get_option_from_file.hpp"
#include "get_permissions_from_file.hpp"
#include "set_permissions.hpp"
#include "../move_original_away.hpp"
#include <bitset>
#include <iostream>
#include <sstream>

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

void patch_file(const path &patch_file, const path &destination,
                const bitset<9> &destination_permissions,
                const string &orig, const string &mod,
                const string &original, const string &modified,                
                const string &reverse_option,
                list<pair<string,string> > &patch_conflicts,
                const Command_Info &info)
{
  /* Apply regular old patches.  If the patch exists but is empty, we
     skip it */
  if(fs::exists(patch_file.string()+".patch")
     && !fs::is_empty(patch_file.string()+".patch"))
    {
      /* Apply the patches out-of-place, in case of hard links */
      fs::rename(destination,",,dopatch." + destination.leaf());
      fs::remove(",,patch_output");

      /* Get a name for a reject file. */
      stringstream rej(".rej");
      int counter(0);

      while(fs::exists(destination.string() + rej.str()))
        {
          ++counter;
          rej.str("");
          rej << ".rej" << counter;
        }

      const int patch_stat
        (system((ARXPATCH + reverse_option
                 + " -f -s --posix -i \""
                 + patch_file.native_file_string()
                 + ".patch\" -o \""
                 + destination.native_file_string()
                 + "\" -r \""
                 + destination.native_file_string() + rej.str()
                 + "\" \",,dopatch." + destination.leaf()
                 + "\" > ,,patch_output 2>&1 < /dev/null")
                .c_str()) >> 8);
      
      /* We have to reset the permissions because they might be
         mangled by "patch". */
      set_permissions(destination,destination_permissions);
      
      switch(patch_stat)
        {
        case 0:
          fs::remove(",,dopatch." + destination.leaf());
          fs::remove(",,patch_output");
          break;
        case 1:
          /* Handle patch conflicts. */
          move_original_away(",,dopatch." + destination.leaf(),
                             destination.string(),patch_conflicts);
          
          if(info.verbosity>=verbose)
            {
              string temp;
              fstream patch_output(",,patch_output");
              while(getline(patch_output,temp))
                {
                  cerr << temp << endl;
                }
            }
          fs::remove(",,patch_output");
          break;
        default:
          /* Serious trouble */
          throw arx_error("INTERNAL ERROR: patch with status > 1 with files\n\t"
                          + patch_file.native_file_string() + ".patch\n\t"
                          + destination.native_file_string()
                          + "\n\t,,dopatch." + destination.leaf()
                          + "\n\t,,patch_output",info);
          break;
        }
      fs::remove(",,patch_output");
    }
  
  /* Handle symlinks and places where files are completely replaced
     instead of being patched. */
  
  if(fs::exists(patch_file.string() + ".link-" + orig)
     || fs::exists(patch_file.string() + "." + original))
    {
      fs::remove(destination);
      if(fs::exists(patch_file.string() + ".link-" + mod))
        {
          /* Replaced by a symlink */
          fs::symlink(destination,
                      get_option_from_file(patch_file.string()
                                           + ".link-" + mod, info));
        }
      else if(fs::exists(patch_file.string() + "." + modified))
        {
          /* Replaced by a complete file */
          fs::rename(patch_file.string() + "." + modified,destination);
        }
      else
        {
          throw arx_error
            ("Bad patch: One of\n\t"
             + patch_file.native_file_string() + ".link-" + orig + "\n\t"
             + patch_file.native_file_string() + "." + original
             + "\nexists, but can't find either of\n\t"
             + patch_file.native_file_string() + ".link-" + mod + "\n\t"
             + patch_file.native_file_string() + "." + modified,
             info);
        }
    }
  
  /* Apply any permissions changes. */
  if(fs::exists(patch_file.string() + ".meta-" + orig))
    {
      if(fs::exists(patch_file.string() + ".meta-" + mod))
        {
          set_permissions(destination,
                          get_permissions_from_file(patch_file.string()
                                                    + ".meta-" + mod));
        }
      else
        {
          throw arx_error
            ("Bad patch: Either both or neither of the following files must exist:\n\t"
             + patch_file.native_file_string() + ".meta-" + orig
             + "\n\t"
             + patch_file.native_file_string() + ".meta-" + mod,
             info);
        }
    }
}
