/* Apply a patch to a source tree.

  Copyright (C) 2001, 2002 Tom Lord
  Copyright (C) 2002, 2003 Walter Landry and the Regents
                           of the University of California
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 dated June, 1991.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/exception.hpp"
#include "boost/date_time/posix_time/posix_time.hpp"
#include "file_attributes.hpp"
#include "arx_error.hpp"
#include "Command_Info.hpp"
#include "Inventory_Flags.hpp"
#include "Inventory.hpp"
#include "inventory_directory.hpp"
#include <list>
#include <utility>
#include "Moved_Path.hpp"
#include "Current_Path.hpp"
#include "recursively_delete_paths.hpp"

using namespace std;
using namespace boost;
using namespace boost::posix_time;
namespace fs=boost::filesystem;
using fs::path;

extern void read_and_set_directory_permissions(const path &permissions_path,
                                               const path &patches,
                                               const Command_Info &info);

extern void output_patching_errors(list<pair<string,string> > &move_conflicts,
                                   list<pair<string,string> > &patch_conflicts,
                                   list<Moved_Path> &missing_moves,
                                   list<string> &missing_patches,
                                   const Command_Info &info);

extern void apply_patches(list<pair<string,string> > patched[2],
                          const Inventory &source_inventory,
                          const bool &reverse,
                          const path &patch, const path &destination,
                          list<string> &missing_patches,
                          list<pair<string,string> > &patch_conflicts,
                          const Command_Info &info);

extern void move_to_destination(list<Moved_Path> &moved,
                                const path &deleted_path,
                                const path &source,
                                list<pair<string,string> > &move_conflicts,
                                const Command_Info &info);

extern void set_aside_paths(const path &deleted_path, const path &source,
                            list<Moved_Path> &moved,
                            list<Moved_Path> &missing_moves,
                            Inventory &source_inventory,
                            const Command_Info &info);

extern void read_and_sort_index(const path &file_path,
                                list<pair<string,string> > &file_list,
                                const Command_Info &info);

extern void categorize_files(const list<pair<string,string> > original[2],
                             const list<pair<string,string> > modified[2],
                             list<pair<string,string> > patched[2],
                             list<Moved_Path> &moved,
                             list<Moved_Path> &mod_only,
                             const bool &reversed);

extern void add_files_and_directories(list<Moved_Path> &new_paths,
                                      const path &patch, const path &source,
                                      const path &temp_path,
                                      list<Moved_Path> &moved,
                                      const Command_Info &info);

void do_patch(const path &patch, const path &source, const bool &reverse,
              const bool &delete_removed, const Command_Info &info)
{
  try
    {
      if(!exists(patch))
        {
          throw arx_error("Patch directory does not exist: "
                          + patch.native_file_string(),info);
        }
      if(!exists(source))
        {
          throw arx_error("Source directory does not exist: "
                          + source.native_file_string(),info);
        }

      /* Create a directory where I can put deleted things and other
         things temporarily. */

      path deleted_path;
      deleted_path=source / ("++removed-by-dopatch-" + source.leaf() + "--"
                             + to_iso_extended_string
                             (ptime(second_clock::universal_time())));
      if(exists(deleted_path))
        {
          throw arx_error("Deleted items directory already exists\n\t"
                          + deleted_path.native_file_string(),info);
        }
      else
        {
          fs::create_directory(deleted_path);
        }

      /* Strings to parametrize whether a patch is to be reversed. */
      string orig("orig"), mod("mod"), modified_string("modified");
      if(reverse)
        {
          orig="mod";
          mod="orig";
          modified_string="original";
        }
          
      /* Inventory the source tree. */

      Inventory source_inventory;
      {
        Current_Path current_path(source);
        Inventory_Flags flags;
        flags.source=flags.control=true;
        flags.files=flags.directories=true;
        flags.inv_method=Inventory_Method::get(".",flags.regexes,info); 
        inventory_directory(".", source_inventory, flags, info);
      }
      /* Sort by tags to allow efficient searching later. */
      for(int m=0;m<2;++m)
        for(int l=0;l<num_inventory_types;++l)
          {
            source_inventory[l][m].sort(inventory_tag_cmp);
          }

      /* Read the patch indices and categorize files. */

      list<pair<string,string> > original[2], modified[2];

      read_and_sort_index(patch/(orig + "-files-index"),original[arx_file],
                          info);
      read_and_sort_index(patch/(orig + "-dirs-index"),original[arx_dir],info);
      read_and_sort_index(patch/(mod + "-files-index"),modified[arx_file],
                          info);
      read_and_sort_index(patch/(mod + "-dirs-index"),modified[arx_dir],info);

      read_and_set_directory_permissions
        (patch/(modified_string + "-only-dir-metadata"),
         patch/"new-files-archive",info);

      list<pair<string,string> > patched[2];
      list<Moved_Path> mod_only, moved;
      categorize_files(original,modified,patched,moved,mod_only,reverse);

      /* Do the real meat of deleting, renaming, adding and patching.
         We do the set asides because, among other things, a file may
         be renamed into a directory that has just been created or
         renamed.  */

      list<pair<string,string> > move_conflicts, patch_conflicts;
      list<Moved_Path> missing_moves;
      list<string> missing_patches;
        
      set_aside_paths(deleted_path,source,moved,missing_moves,
                      source_inventory,info);
      add_files_and_directories(mod_only,patch,source,deleted_path,moved,info);
      move_to_destination(moved,deleted_path,source,move_conflicts,info);
      apply_patches(patched,source_inventory,reverse,patch/"patches",source,
                    missing_patches,patch_conflicts,info);
      output_patching_errors(move_conflicts,patch_conflicts,missing_moves,
                             missing_patches,info);

      /* Remove the deleted paths if the option is set or the
         directory is empty. */

      if(delete_removed
         || fs::directory_iterator(deleted_path)==fs::directory_iterator())
        recursively_delete_paths(deleted_path);
    }
  catch (fs::filesystem_error &fs_error)
    {
      throw arx_error("Problem opening or writing a file:\n"
                      + fs_error.who() + "\n"
                      + fs_error.path1().native_file_string() + "\n"
                      + fs_error.path2().native_file_string() + "\n",
                      info);
    }
}
