/* Print out an inventory of a project tree.  This mostly parses
   options and then passes the work to inventory_directory and
   output_inventory.

   Copyright (C) 2001, 2002 Tom Lord
   Copyright (C) 2002, 2003 Walter Landry and the Regents of the
                                  University of California

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */


#include "parse_common_options.hpp"
#include "parse_unknown_options.hpp"
#include "check_extra_args.hpp"
#include "command_initializer.hpp"
#include "arx_error.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/exception.hpp"
#include "Inventory_Flags.hpp"
#include "inventory_directory.hpp"
#include "file_attributes.hpp"
#include <list>
#include "Inventory.hpp"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

extern void output_inventory(Inventory &inventory_result,
                             const Inventory_Flags &flags,
                             const Command_Info &info);

int inventory(list<string> &argument_list, const command &cmd);

static command_initializer inventory_init(command("inventory",
"Print out an inventory of a project tree",
"usage: inventory [options] [dir]",
" -s --source                   list source files\n\
 -p --precious                 list precious files\n\
 -b --backups                  list backup files\n\
 -j --junk                     list junk files\n\
 -u --unrecognized             list unrecognized files\n\
 -t --trees                    list roots of nested trees\n\
\n\
 -d --directories              list directories\n\
 -f --files                    list non-directories\n\
 --kind                        show file kinds\n\
 --control                     include arch control files\n\
 --nested                      include nested trees\n\
 --non-source                  recursively inventory non-source directories\n\
\n\
 --tags                        list with id tags\n\
                               (only applies to source)\n\
 --untagged                    include untagged files\n\
\n\
 --explicit                    require an explicit file tag\n\
 --implicit                    use the implicit tagging method\n\
 --names                       use the name-based tagging method\n\
\n\
With no arguments, print a human-readable inventory report.\n\
\n\
With -r or --report, category options (--source etc)\n\
limit the report to just those files.  With no other options,\n\
the report includes printf all sections and files.\n\
\n\
The options -d and -f, stack on each other.  Thus \"inventory -d -f\"\n\
is the same as \"inventory\".\n\
\n\
If a directory is precious, junk, or unrecognized, only the\n\
directory name itself is printed -- its contents are not\n\
searched.",
inventory,"Inventory",true));

int inventory(list<string> &argument_list, const command &cmd)
{
  Command_Info info(cmd);
  bool done=false;
  Inventory_Flags flags;
  int disambiguate=-1;

  while(!argument_list.empty() && !done)
    {
      if(!parse_common_options(argument_list,info))
        {
          list<string>::iterator arg=argument_list.begin();
          if(*arg=="-s" || *arg=="--source")
            {
              flags.source=true;
              argument_list.pop_front();
              ++disambiguate;
            }
          else if (*arg=="-p" || *arg=="--precious")
            {
              flags.precious=true;
              argument_list.pop_front();
              ++disambiguate;
            }
          else if (*arg=="-b" || *arg=="--backups")
            {
              flags.backups=true;
              argument_list.pop_front();
              ++disambiguate;
            }
          else if (*arg=="-j" || *arg=="--junk")
            {
              flags.junk=true;
              argument_list.pop_front();
              ++disambiguate;
            }
          else if (*arg=="-u" || *arg=="--unrecognized")
            {
              flags.unrecognized=true;
              argument_list.pop_front();
              ++disambiguate;
            }
          else if (*arg=="-t" || *arg=="--trees")
            {
              flags.trees=true;
              argument_list.pop_front();
              ++disambiguate;
            }
          else if (*arg=="-r" || *arg=="--report")
            {
              flags.report=true;
              argument_list.pop_front();
            }
          else if (*arg=="-d" || *arg=="--directories")
            {
              flags.directories=true;
              argument_list.pop_front();
            }
          else if (*arg=="-f" || *arg=="--files")
            {
              flags.files=true;
              argument_list.pop_front();
            }
          else if (*arg=="--kind")
            {
              flags.kind=true;
              argument_list.pop_front();
            }
          else if (*arg=="--control")
            {
              flags.control=flags.control=true;
              argument_list.pop_front();
            }
          else if (*arg=="--nested")
            {
              flags.nested=true;
              argument_list.pop_front();
            }
          else if (*arg=="--tags")
            {
              flags.tags=true;
              argument_list.pop_front();
            }
          else if (*arg=="--untagged")
            {
              flags.untagged=true;
              argument_list.pop_front();
            }
          else if (*arg=="--explicit")
            {
              flags.inv_method=Inventory_Method::explicit_inventory;
              flags.use_default_inventory_method=false;
              argument_list.pop_front();
            }
          else if (*arg=="--implicit")
            {
              flags.inv_method=Inventory_Method::implicit_inventory;
              flags.use_default_inventory_method=false;
              argument_list.pop_front();
            }
          else if (*arg=="--names")
            {
              flags.inv_method=Inventory_Method::names_inventory;
              flags.use_default_inventory_method=false;
              argument_list.pop_front();
            }
          else if (*arg=="--non-source")
            {
              flags.inventory_non_source=true;
              argument_list.pop_front();
            }
          else
            {
              parse_unknown_options(argument_list,info);
              done=true;
            }
        }
    }
  
  path directory(".");
  if(!argument_list.empty())
    {
      directory=path(*(argument_list.begin()));
      if(!exists(directory) || !is_directory(directory))
        {
          throw arx_error("Argument must be a directory: "
                          + *(argument_list.begin()),info);
        }
      argument_list.pop_front();
    }
  /* Set the inventory method if none is specified. */
  if(flags.use_default_inventory_method)
    {
      flags.inv_method=Inventory_Method::get(directory,flags.regexes,info);
    }
  else
    {
      Inventory_Method::get(directory,flags.regexes,info);
    }
  check_extra_args(argument_list,info);

  /* If nothing is specified, print out everything */
    
  if(flags.report)
    {
      flags.disambiguate=false;
    }
  else
    {
      flags.disambiguate=disambiguate!=0;
    }

  if(!flags.source && !flags.precious && !flags.backups && !flags.junk
     && !flags.unrecognized && !flags.trees)
    {
      flags.source=flags.precious=flags.backups=flags.junk=flags.unrecognized=
        flags.trees=flags.report=true;
      flags.disambiguate=false;
    }
  if(!flags.directories && !flags.files)
    {
      flags.directories=flags.files=true;
    }

  Inventory inventory_result;

  /* Traverse the directory */

  inventory_directory(directory,inventory_result,flags,info);

  /* and print it all out */

  if(info.verbosity>silent)
    output_inventory(inventory_result,flags,info);

  return 0;
}
