/* Looks recursively through a directory, categorizing what is there.
   When looking at directories, it asks, in order:

   1) Is it a nested tree
   2) Does it match the regex for a control directory
   3) Is it an explicitly tagged directory
   4) Does it match the regex for junk, then backup, then precious,
      then unrecognized, then source

   If none of these things are true, then it categorizes the directory
   as unrecognized.

   For files, it is the same except there are no such thing as nested
   trees, and it checks for implicit tags as well.

   Copyright (C) 2003 Walter Landry

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include <string>
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "Inventory.hpp"
#include "Inventory_Flags.hpp"
#include "Inventory_Method.hpp"
#include "inventory_types.hpp"
#include "file_attributes.hpp"
#include "inventory_directory_as.hpp"
#include "Command_Info.hpp"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;


extern string get_implicit_directory_tag(const path &dir_path,
                                         const Inventory_Flags &flags,
                                         const Command_Info &info);

extern bool inventory_control_directory(const path &file_path,
                                        Inventory &result,
                                        const Inventory_Flags &flags,
                                        const Command_Info &info);

extern bool push_if_tagged_directory(const path &file,
                                     Inventory &result,
                                     const Inventory_Flags &flags,
                                     const Command_Info &info);

extern bool push_if_tagged_file(const path &file_path,
                                Inventory &result,
                                const Inventory_Flags &flags,
                                const Command_Info &info);

extern bool push_if_implicit_or_names(const path &file_path,
                                      Inventory &result,
                                      const Inventory_Flags &flags,
                                      const Command_Info &info);

void inventory_directory(const path &directory,
                         Inventory &result,
                         const Inventory_Flags &flags,
                         const Command_Info &info)
{
  fs::directory_iterator end;
  for(fs::directory_iterator i(directory); i!=end; ++i)
    {
      /* If it is a directory, we may have to recurse through it.
         Symbolic links are always considered files, even if they
         point to directories. */

      if(!is_link(*i) && is_directory(*i))
        {
          /* If it is a nested project tree and we are doing a nested
             inventory, then we need to use a different
             inventory_method */

          if(exists(*i / "{arch}"))
            {
              if(flags.trees && flags.directories)
                {
                  result[nested_tree][arx_dir].push_back(*i);
                }
              if(flags.nested)
                {
                  Inventory_Flags new_flags(flags);
                  if(flags.use_default_inventory_method)
                    {
                      new_flags.inv_method=
                        Inventory_Method::get(*i,new_flags.regexes,info);
                    }
                  inventory_directory(*i,result,new_flags,info);
                }
            }
          else if(inventory_control_directory(*i,result,flags,info))
            {}
          else if(push_if_tagged_directory(*i,result,flags,info))
            {
              inventory_directory(*i,result,flags,info);
            }
          else if(regex_match(i->leaf(),flags.regexes.junk))
            {
              if(flags.junk && flags.directories)
                {
                  result[junk][arx_dir].push_back(*i);
                  /* If we're inventorying everything, then all files in a
                     junk directory are junk. */
                  if(flags.inventory_non_source)
                    inventory_directory_as(*i,result,junk,flags,info);
                }
            }
          else if(regex_match(i->leaf(),flags.regexes.precious))
            {
              if(flags.precious && flags.directories)
                {
                  result[precious][arx_dir].push_back(*i);
                  /* If we're inventorying everything, then all files in a
                     precious directory are precious. */
                  if(flags.inventory_non_source)
                    inventory_directory_as(*i,result,precious,flags,info);
                }
            }
          else if(regex_match(i->leaf(),flags.regexes.backup))
            {
              if(flags.backups && flags.directories)
                {
                  result[backup][arx_dir].push_back(*i);
                  /* If we're inventorying everything, then all files in a
                     backup directory are backup. */
                  if(flags.inventory_non_source)
                    inventory_directory_as(*i,result,backup,flags,info);
                }
            }
          else if(regex_match(i->leaf(),flags.regexes.unrecognized))
            {
              if(flags.unrecognized && flags.directories)
                {
                  result[unrecognized][arx_dir].push_back(*i);
                  /* If we're inventorying everything, then all files in an
                     unrecognized directory are unrecognized. */
                  if(flags.inventory_non_source)
                    inventory_directory_as(*i,result,unrecognized,flags,info);
                }
            }
          /* Naming conventions for source only if not using
             explicit_inventory. */
          else if(flags.inv_method!=Inventory_Method::explicit_inventory
                  && regex_match(i->leaf(),flags.regexes.source))
            {
              if(flags.source && flags.directories)
                {
                  result[source][arx_dir]
                    .push_back(file_attributes
                               (*i,get_implicit_directory_tag(*i,flags,info)));
                }
              inventory_directory(*i,result,flags,info);
            }
          else
            {
              if(flags.unrecognized && flags.directories)
                {
                  result[unrecognized][arx_dir].push_back(*i);
                  /* If we're inventorying everything, then all files in an
                     unrecognized directory are unrecognized. */
                }
              if(flags.inventory_non_source)
                inventory_directory_as(*i,result,unrecognized,flags,info);
            }
        }
      /* It must be a file or symbolic link */
      else if(flags.files)
        {
          if(regex_match(i->leaf(),flags.regexes.control))
            {
              if(flags.control)
                result[control][arx_file].push_back(*i);
            }
          else if(push_if_tagged_file(*i,result,flags,info))
            {
            }
          else if(regex_match(i->leaf(),flags.regexes.junk))
            {
              if(flags.junk)
                result[junk][arx_file].push_back(*i);
            }
          else if(regex_match(i->leaf(),flags.regexes.precious))
            {
              if(flags.precious)
                result[precious][arx_file].push_back(*i);
            }
          else if(regex_match(i->leaf(),flags.regexes.backup))
            {
              if(flags.backups)
                result[backup][arx_file].push_back(*i);
            }
          else if(regex_match(i->leaf(),flags.regexes.unrecognized))
            {
              if(flags.unrecognized)
                result[unrecognized][arx_file].push_back(*i);
            }
          else if(push_if_implicit_or_names(*i,result,flags,info))
            {
            }
          else
            {
              if(flags.unrecognized)
                result[unrecognized][arx_file].push_back(*i);
            }
        }
    }
}
