/* Checks whether a particular file is categorized as source for implicit
   and names inventory methods.

   Copyright (C) 2001, 2002 Tom Lord
   Copyright (C) 2003 Walter Landry

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/regex.hpp"
#include "Inventory_Flags.hpp"
#include "Inventory_Method.hpp"
#include "file_attributes.hpp"
#include "inventory_types.hpp"
#include "Command_Info.hpp"
#include "Inventory.hpp"
#include "escape_string.hpp"

#include "names_tag.hpp"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

bool push_if_implicit_or_names(const path &file_path,
                               Inventory &result,
                               const Inventory_Flags &flags,
                               const Command_Info &info)
{
  bool done(false);

  /* First, make sure we are not using explicit tagging and the file
     name passes the naming conventions. */
  if(flags.inv_method!=Inventory_Method::explicit_inventory
     && regex_match(file_path.leaf(),flags.regexes.source))
    {
      if(flags.source)
        {
          /* We have to check a bunch of things for implicit inventories */
          if(flags.inv_method==Inventory_Method::implicit_inventory)
            {
              /* First, check for strong defaults */
              const path strong_default_path(file_path.branch_path()
                                             / ".arch-ids/=all");
              if(exists(strong_default_path))
                {
                  result[source][arx_file]
                    .push_back(file_attributes
                               (file_path,
                                "a_" + get_option_from_file(strong_default_path,
                                                            info)));
                  done=true;
                }
              /* If it isn't a symlink, check for implicit tags */
              else if(!is_link(file_path))
                {
                  try
                    {
                      fs::ifstream file(file_path);
                      int total_bytes(0);
                      string file_line;
                      
                      /* More broken stuff.  A basename will never
                         match if the file name starts with
                         punctuation.  This is probably a bug, but
                         this is the backwards compatible way. */
                      match_results<string::const_iterator> m;
                      regex basename_regex;

                      char bar[]="b";
                      bar[0]=file_path.leaf()[0];

                      if(!regex_match(bar,regex("[[:punct:]]")))
                        {
                      /* Remember to escape all of the special regex
                         characters, so that we can use the file name
                         as a regular expression. */

                          basename_regex=
                            "^[[:punct:][:space:]]+("
                            + escape_string(file_path.leaf())
                            + "[[:space:]]*-)[[:space:]]*";
                        }
                      
                      const regex tag_regex
                        ("^[[:punct:][:space:]]+(tag:[[:space:]]+)[[:space:]]*");
                      
                      /* Implicit tags are within 1024 bytes of the
                         beginning or end of the file.  We check the
                         end of the file first, and use the first tag
                         we find. */

                      const int max_bytes=1024;
                      bool top(false);
                      total_bytes=-max_bytes;
                      file.seekg(-max_bytes,std::ios::end);
                      /* If the file is less than max_bytes characters
                         long, just start from the top. */
                      if(file.fail())
                        {
                          file.clear();
                          file.seekg(std::ios::beg);
                          total_bytes=0;
                          top=true;
                        }
                      while(total_bytes<max_bytes && !done)
                        {
                          if(getline(file,file_line))
                            {
                              total_bytes+=file_line.size();
                              if(regex_search(file_line,m,tag_regex)
                                 && m.length()<file_line.size())
                                {
                                  result[source][arx_file].
                                    push_back(file_attributes
                                              (file_path,"i_"
                                               + file_line.substr(m.length())));
                                  
                                  done=true;
                                }
                              else if(regex_search(file_line,m,basename_regex)
                                      && m.length()<file_line.size())
                                {
                                  result[source][arx_file].
                                    push_back(file_attributes
                                              (file_path,"i_"
                                               + file_line.substr(m.length())));
                                  done=true;
                                }
                            }
                          else
                            {
                              if(!top)
                                {
                                  top=true;
                                  file.clear();
                                  file.seekg(std::ios::beg);
                                  total_bytes=0;
                                }
                              else
                                {
                                  total_bytes=max_bytes;
                                }
                            }
                        }
                    }
                  catch (fs::filesystem_error &fs_error)
                    {
                      throw arx_error("Can't read the tags from "
                                      + file_path.native_file_string(),info);
                    }
                }

              if(!done)
                {
                  /* Check for a weak default. */
                  const path weak_default_path(file_path.branch_path()
                                               / ".arch-ids/=default");
                  if(exists(weak_default_path))
                    {
                      result[source][arx_file]
                        .push_back(file_attributes
                                   (file_path,"w_"
                                    + get_option_from_file(weak_default_path,info)
                                    + file_path.leaf()));
                      done=true;
                    }
                  else
                    {
                      /* Finally, it is either a don't care or default naming
                         conventions. */
                      const path dont_care_path(file_path.branch_path()
                                                / ".arch-ids/=dont-care");

                      string prefix;
                      if(exists(dont_care_path))
                        {
                          prefix="k_";
                        }
                      else
                        {
                          prefix="?_";
                        }
                      result[source][arx_file]
                        .push_back(file_attributes
                                   (file_path,prefix + names_tag(file_path,info)));
                      done=true;
                    }
                }
            }
          else
            {
              /* Otherwise, we're just using naming conventions. */
              
              result[source][arx_file]
                .push_back(file_attributes
                           (file_path,"?" + names_tag(file_path,info)));
              done=true;
            }
        }
      return true;
    }
  return false;
}

