/* Create and register an archive

   Copyright (C) 2001, 2002 Tom Lord
   Copyright (C) 2002, 2003 Walter Landry and the Regents of the
                                  University of California

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include "parse_common_options.hpp"
#include "parse_unknown_options.hpp"
#include "check_extra_args.hpp"
#include "set_option_in_file.hpp"
#include "command_initializer.hpp"
#include "create_archive_registration.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/exception.hpp"
#include "arx_error.hpp"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

int make_archive(list<string> &argument_list, const command &cmd);

static command_initializer make_archive_init(command("make-archive",
"Create and register an archive",
"usage: make-archive [options] name directory",
"NAME is the global name for the archive.  For now, it is best\n\
to make it be an email address with a fully qualified domain name,\n\
optionally followed by \"--\" and a string of letters, digits, periods\n\
and dashes (but not two dashes in a row).  This restriction will be lifted\n\
in future releases.\n\
\n\
The archive must not already exist.  It will create the directory if it\n\
does not already exist.",
make_archive,"Archive",true));

int make_archive(list<string> &argument_list, const command &cmd)
{
  Command_Info info(cmd);
  const string ArX_archive_version("Hackerlab arch archive directory, format version 1");

  while(parse_common_options(argument_list,info))
    {}
  parse_unknown_options(argument_list,info);

  if(argument_list.size()<2)
    {
      throw arx_error ("Not enough arguments",info);
    }
  string name(*(argument_list.begin()));
  argument_list.pop_front();
  string directory(*(argument_list.begin()));
  argument_list.pop_front();
  check_extra_args(argument_list,info);
  
  /* Make sure the archive is not already registered. */
  {
    path home(getenv("HOME"));
    path config_path(home / ".arch-params/=locations" / name);
    
    if(exists(config_path))
      {
        throw arx_error("Archive is already registered: " + name
                        + "\nEither choose a different name, or unregister\nthe name you chose, but be careful not to create\ntwo archives with the same name.",
                        info);
      }
  }

  /* Create directories if needed */
  path dir_path(directory);
  if(!exists(dir_path))
    {
      try
        {
          create_directory(dir_path);
        }
      catch(fs::filesystem_error &fs_error)
        {
          throw arx_error("Can't create archive directory: "
                          + dir_path.native_file_string(), info);
        }
    }
  else if(!is_directory(dir_path))
    {
      throw arx_error("The argument: " + directory
                      + "\n is not a directory",info);
    }

  path meta_path(dir_path / "=meta-info");
  if(!exists(meta_path))
    {
      try
        {
          create_directory(meta_path);
        }
      catch(fs::filesystem_error &fs_error)
        {
          throw arx_error("Can't create meta-info directory: "
                          + dir_path.native_file_string(), info);
        }
    }
  else if(!is_directory(meta_path))
    {
      throw arx_error("Corrupt archive\n" + meta_path.native_file_string()
                      + "\n is not a directory", info);
    }

  /* Enter the characteristics of the archive. */
  set_option_in_file(meta_path / name, name, info);
  set_option_in_file(dir_path / ".archive-version", ArX_archive_version, info);

  /* Register the new archive. */
  create_archive_registration(name,system_complete(dir_path)
                              .native_file_string(),info);
  return 0;
}
