/* Compare the files in two inventories, writing patches as needed.

  Copyright (C) 2001, 2002 Tom Lord
  Copyright (C) 2002, 2003 Walter Landry and the Regents
                           of the University of California
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 dated June, 1991.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

#include "config.h"
#include "Inventory.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include <list>
#include <utility>
#include "../link_target.hpp"
#include "../create_directory_and_parents.hpp"
#include "Command_Info.hpp"
#include "arx_error.hpp"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

extern bool same_contents(const path &f1, const path &f2);
extern void remove_empty_directories(const path &directory);

extern int invoke_diff(const string &orig, const string &mod,
                       const string &dest,
                       const Command_Info &info);

void compare_files(const list<pair<file_attributes,file_attributes> >
                   in_both[num_inventory_types][2],
                   const path &original,
                   const path &modified,
                   list<file_attributes> changed_orig[2],
                   list<file_attributes> changed_mod[2],
                   const path &destination,
                   const Command_Info &info)
{

  for(int l=0;l<num_inventory_types;++l)
    for(list<pair<file_attributes,file_attributes> >::const_iterator
          i=in_both[l][arx_file].begin(); i!=in_both[l][arx_file].end(); ++i)
      {
        const file_attributes &orig(i->first), &mod(i->second);

        create_directory_and_parents(destination/mod.file_path.branch_path());

        if(orig.file_path.string()!=mod.file_path.string())
          {
            /* Check for renames */

            changed_orig[arx_file].push_back(orig);
            changed_mod[arx_file].push_back(mod);
          }

        if(orig.link)
          {
            /* If they are both symlinks */
            if(mod.link)
              {
                const path orig_link(link_target(original/orig.file_path));
                const path mod_link(link_target(modified/mod.file_path));
                if(orig_link.string()!=mod_link.string())
                  {
                    fs::ofstream orig_link_file(destination/
                                                (mod.file_path.string()
                                                 + ".link-orig"));
                    orig_link_file << orig_link.native_file_string() << "\n";
                    fs::ofstream mod_link_file(destination/
                                               (mod.file_path.string()
                                                + ".link-mod"));
                    mod_link_file << mod_link.native_file_string() << "\n";

                    changed_orig[arx_file].push_back(orig);
                    changed_mod[arx_file].push_back(mod);
                  }
              }
            /* Orig is a link, mod is a file */
            else
              {
                const path orig_link(link_target(original/orig.file_path));
                fs::ofstream orig_link_file(destination/
                                            (mod.file_path.string()
                                             + ".link-orig"));
                orig_link_file << orig_link.native_file_string() << "\n";
                copy_file(modified/mod.file_path,
                          destination/(mod.file_path.string() + ".modified"));
                changed_orig[arx_file].push_back(orig);
                changed_mod[arx_file].push_back(mod);
              }
          }
        else
          {
            /* Orig is a file, mod is a link */
            if(mod.link)
              {
                fs::ofstream mod_link_file(destination/(mod.file_path.string()
                                                        + ".link-mod"));
                mod_link_file << link_target(mod.file_path)
                  .native_file_string() << "\n";
                copy_file(original/orig.file_path,
                          destination/(mod.file_path.string() + ".original"));
                changed_orig[arx_file].push_back(orig);
                changed_mod[arx_file].push_back(mod);
              }
            /* Both are files */
            else
              {
                bool files_different(false);

                /* Have permissions changed? */
                if(orig.permissions!=mod.permissions)
                  {
                    fs::ofstream orig_file(destination/
                                           (mod.file_path.string()
                                           + ".meta-orig"));
                    orig_file << "--permissions "
                              << oct << orig.permissions.to_ulong() << endl;
                    fs::ofstream mod_file(destination/
                                          (mod.file_path.string()
                                           + ".meta-mod"));
                    mod_file << "--permissions "
                             << oct << mod.permissions.to_ulong() << endl;

                    files_different=true;
                  }

                /* Try to diff */

                if(!same_contents(original/orig.file_path,
                                  modified/mod.file_path))
                  {
                    files_different=true;
                    const int diffstat
                      (system((ARXDIFF
                               + (" -u \""
                                  + (original/orig.file_path)
                                  .native_file_string() + "\" \""
                                  + (modified/mod.file_path)
                                  .native_file_string() + "\" > \"" 
                                  + (destination/mod.file_path)
                                  .native_file_string()
                                  + ".patch\" 2> /dev/null")).c_str()) >> 8);
                    
                    /* If there is no difference, there is something
                       wrong. */

                    if(diffstat==0)
                      {
                        throw arx_error("same_contents says there is a difference, but diff says there isn't\n\t"
                                        + (original/orig.file_path)
                                        .native_file_string() + "\n\t"
                                        + (modified/mod.file_path)
                                        .native_file_string(),info);
                      }
                    /* If the diff failed, remove the patch and save
                       copies of the original and modified. */
                    if(diffstat==2)
                      {
                        remove(destination/(mod.file_path.string() +".patch"));
                        copy_file(original/orig.file_path,
                                  destination/(mod.file_path.string()
                                               + ".original"));
                        copy_file(modified/mod.file_path,
                                  destination/(mod.file_path.string()
                                               + ".modified"));
                      }
                  }
                
                /* If there are any differences, add these files to
                   the list of changed files. */
                if(files_different)
                  {
                    changed_orig[arx_file].push_back(orig);
                    changed_mod[arx_file].push_back(mod);
                  }
              }
          }
      }
  remove_empty_directories(destination);
}
