/* Create a patch between two source trees.

  Copyright (C) 2001, 2002 Tom Lord
  Copyright (C) 2002, 2003 Walter Landry and the Regents
                           of the University of California
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; version 2 dated June, 1991.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA */

#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/exception.hpp"
#include "file_attributes.hpp"
#include "arx_error.hpp"
#include "Command_Info.hpp"
#include "Inventory_Flags.hpp"
#include "Inventory.hpp"
#include "inventory_directory.hpp"
#include "url_encode.hpp"
#include "Tom_encode.hpp"
#include <list>
#include <utility>
#include "Current_Path.hpp"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

extern void check_duplicate_inventory_tags(const Inventory &inv,
                                           const Command_Info &info);
extern void cull_retagged_directories(Inventory &orig_only,
                                      Inventory &mod_only);

extern void copy_inventory_files_dirs(const path &source,
                                      const path &destination,
                                      Inventory &inv);
extern void save_directory_permissions(Inventory &inv,
                                       list<file_attributes> &changed_list,
                                       const path &prefix,
                                       const path &destination);
extern void compare_files(const list<pair<file_attributes,file_attributes> >
                          in_both[num_inventory_types][2],
                          const path &original,
                          const path &modified,
                          list<file_attributes> changed_orig[2],
                          list<file_attributes> changed_mod[2],
                          const path &destination,
                          const Command_Info &info);
extern void compare_directories(list<pair<file_attributes,file_attributes> >
                                in_both[num_inventory_types][2],
                                list<file_attributes> changed_orig[2],
                                list<file_attributes> changed_mod[2],
                                const path &destination);

void make_patch(const path &original, const path &modified,
                const path &destination, const Command_Info &info)
{
  try
    {
      if(!exists(original))
        {
          throw arx_error("Original does not exist: "
                          + original.native_file_string(),info);
        }
      if(!exists(modified))
        {
          throw arx_error("Modified does not exist: "
                          + modified.native_file_string(),info);
        }
      if(exists(destination))
        {
          throw arx_error("Destination already exists: "
                          + destination.native_file_string(),info);
        }
      else
        {
          create_directory(destination);
        }

      /* Inventory the two directories, setting the appropriate
         flags.  We set and reset the directories.  This is a
         performance hack. */

      Inventory_Flags flags;
      flags.source=flags.control=true;
      flags.files=flags.directories=true;
      flags.inv_method=Inventory_Method::get(original,flags.regexes,info); 
     
      Inventory original_inventory, modified_inventory;
      
      {
        Current_Path curr_path(original);
        inventory_directory(".",original_inventory,flags,info);
      }
      {
        flags.inv_method=Inventory_Method::get(modified,flags.regexes,info);
        
        Current_Path curr_path(modified);
        inventory_directory(".",modified_inventory,flags,info);
      }

      /* Check for duplicate tags */
      
      check_duplicate_inventory_tags(original_inventory,info);
      check_duplicate_inventory_tags(modified_inventory,info);
      
      /* Figure out which files are only in original or modified. */
      
      /* It is probably more efficient to splice elements from one
         list to the other, but this is easier for now. */
      
      list<pair<file_attributes,file_attributes> > 
        in_both[num_inventory_types][2];
      list<file_attributes> changed_orig[2], changed_mod[2];
      Inventory orig_only, mod_only;
      
      for(int l=0;l<num_inventory_types;++l)
        for(int m=0;m<2;++m)
          {
            original_inventory[l][m].sort(inventory_tag_cmp);
            modified_inventory[l][m].sort(inventory_tag_cmp);
            
            Inventory_iterator i(original_inventory[l][m].begin()),
              j(modified_inventory[l][m].begin());
            while(i!=original_inventory[l][m].end()
                  && j!=modified_inventory[l][m].end())
              {
                if(i->inventory_tag==j->inventory_tag)
                  {
                    in_both[l][m].push_back(make_pair(*i,*j));
                    ++i;
                    ++j;
                  }
                else if(i->inventory_tag < j->inventory_tag)
                  {
                    orig_only[l][m].push_back(*i);
                    ++i;
                  }
                else
                  {
                    mod_only[l][m].push_back(*j);
                    ++j;
                  }
              }
            for(;i!=original_inventory[l][m].end();++i)
              {
                orig_only[l][m].push_back(*i);
              }
            for(;j!=modified_inventory[l][m].end();++j)
              {
                mod_only[l][m].push_back(*j);
              }
          }

      /* Remove directories that appear to be deleted and created with
         the same name.  This is just a directory that has been
         retagged. */
      cull_retagged_directories(orig_only,mod_only);

      /* Save copies of deleted and new files. */
      copy_inventory_files_dirs(original,destination / "removed-files-archive",
                                orig_only);
      copy_inventory_files_dirs(modified,destination / "new-files-archive",
                                mod_only);
      
      /* Save permissions of deleted and new directories. */
      save_directory_permissions(orig_only,changed_orig[arx_dir],original,
                                 destination/"original-only-dir-metadata");
      save_directory_permissions(mod_only,changed_mod[arx_dir],modified,
                                 destination/"modified-only-dir-metadata");
      
      for(int l=0;l<num_inventory_types;++l)
        for(int m=0;m<2;++m)
          {
            changed_orig[m].splice(changed_orig[m].begin(),orig_only[l][m]);
            changed_mod[m].splice(changed_mod[m].begin(),mod_only[l][m]);
          }

      /* Compare files and directories, writing out patches. */
      
      compare_files(in_both, original, modified, changed_orig, changed_mod,
                    destination / "patches",info);
      compare_directories(in_both, changed_orig, changed_mod,
                          destination / "patches");

      /* Sort and uniquify the changed lists */
      
      for(int m=0;m<2;++m)
        {
          /* For now, we sort using Tom's encoding, since that is
             backwards compatible.  In the future, we will be using
             url encoding, so we will just sort on the real tag. */
          changed_orig[m].sort(Tom_encoded_tag_cmp);
          changed_orig[m].unique();
          
          changed_mod[m].sort(Tom_encoded_tag_cmp);
          changed_mod[m].unique();
        }
      
      /* Write out the indices.  These contain files and directories
         that have changed in some way.  */
      
      fs::ofstream orig_files_index(destination / "orig-files-index");
      for(Inventory_iterator i=changed_orig[arx_file].begin();
          i!=changed_orig[arx_file].end(); ++i)
        orig_files_index << url_encode(i->file_path.string(),info) << " "
                         << Tom_encode(i->inventory_tag) << endl;
      
      fs::ofstream mod_files_index(destination / "mod-files-index");
      for(Inventory_iterator i=changed_mod[arx_file].begin();
          i!=changed_mod[arx_file].end(); ++i)
        mod_files_index << url_encode(i->file_path.string(),info) << " "
                        << Tom_encode(i->inventory_tag) << endl;
      
      fs::ofstream orig_dirs_index(destination / "orig-dirs-index");
      for(Inventory_iterator i=changed_orig[arx_dir].begin();
          i!=changed_orig[arx_dir].end(); ++i)
        orig_dirs_index << url_encode(i->file_path.string(),info) << " "
                        << Tom_encode(i->inventory_tag) << endl;
      
      fs::ofstream mod_dirs_index(destination / "mod-dirs-index");
      for(Inventory_iterator i=changed_mod[arx_dir].begin();
          i!=changed_mod[arx_dir].end(); ++i)
        mod_dirs_index << url_encode(i->file_path.string(),info) << " "
                       << Tom_encode(i->inventory_tag) << endl;
    }
  catch (fs::filesystem_error &fs_error)
    {
      throw arx_error("Problem opening or writing a file:\n"
                      + fs_error.who() + "\n"
                      + fs_error.path1().native_file_string() + "\n"
                      + fs_error.path2().native_file_string() + "\n",
                      info);
    }
}

