/* Atomically move an individual file

   Copyright 2003 Walter Landry

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include "arx_error.hpp"
#include "parse_package_name.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/exception.hpp"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

void atomic_move(const path &source, const path &destination,
                 const Command_Info &info)
{
  path target;

  if(exists(destination))
    {
      if(!is_directory(destination) || exists(destination / source.leaf()))
        {
          throw arx_error("can't move " + source.native_file_string() + " to "
                          + destination.native_file_string()
                          + "\n.  Destination already exists",info);
        }
      target=destination / source.leaf();
    }
  else
    {
      target=destination;
    }

  path source_id(source.branch_path() / ".arch-ids" / (source.leaf() + ".id")),
    target_id(target.branch_path() / ".arch-ids" / (target.leaf() + ".id"));

  /* If the source_id exists, then go through a lot of singing and
     dancing to ensure that both the source and id are either both
     moved or both not moved.  It isn't perfectly transactional, but
     it should at least complain if the tree is broken and give
     directions on how to fix it. */

  if(exists(source_id))
    {
      if(exists(target_id))
        {
          throw arx_error("can't move " + source_id.native_file_string() + " to "
                          + target_id.native_file_string()
                          + "\n.  Destination already exists",info);
        }
      try
        {
          if(!exists(target_id.branch_path()))
            create_directory(target_id.branch_path());
          copy_file(source_id,target_id);
        }
      catch(fs::filesystem_error &fs_error)
        {
          if(exists(target_id) && !remove(target_id))
            {
              throw arx_error("Move failed:  Can not remove the new arch id at "
                              + target_id.native_file_string()
                              + "\nPlease remove it.",info);
            }
          throw arx_error("Can't copy " + source_id.native_file_string() + " to "
                          + target_id.native_file_string(),info);
        }
      try
        {
          rename(source,target);
        }
      catch(fs::filesystem_error &fs_error)
        {
          if(exists(target_id) && !remove(target_id))
            {
              throw arx_error("Move failed:  Can not remove the new arch id at "
                              + target_id.native_file_string()
                              + "\nPlease remove it.",info);
            }
          throw arx_error("Can't move " + source.native_file_string() + " to "
                          + target.native_file_string(),info);
        }
      if(!remove(source_id))
        {
          throw arx_error ("Can't remove the old id file at "
                           + source_id.native_file_string()
                           + "\nPlease remove it.",info);
        }
    }
  else
    /* Otherwise, it is a simple rename. */
    {
      try
        {
          rename(source,target);
        }
      catch(fs::filesystem_error &fs_error)
        {
          throw arx_error("Can't rename " + source.native_file_string()
                          + " to " + target.native_file_string(),info);
        }
    }
}
