/* Move files and, if they have them, their tags.  If there are no
   tags, it still moves them.  The intention is to be able to
   transparently replace "mv" with "arx move".

   Copyright (C) 2001, 2002 Tom Lord
   Copyright (C) 2002, 2003 Walter Landry and the Regents of the
                                  University of California

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include "parse_common_options.hpp"
#include "parse_unknown_options.hpp"
#include "check_extra_args.hpp"
#include "command_initializer.hpp"
#include "arx_error.hpp"
#include "parse_package_name.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/exception.hpp"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

extern void atomic_move(const path &source, const path &destination,
                        const Command_Info &info);

int move(list<string> &argument_list, const command &cmd);

static command_initializer move_init(command("move",
"Rename a file or directory and its explicit tag.",
"usage: move [options] source target\n   or: move [options] source... directory",
"Rename a file or directory and its explicit inventory tag.\n\
The semantics should be equivalent to mv(1).",
move,"Inventory",true));

int move(list<string> &argument_list, const command &cmd)
{
  Command_Info info(cmd);

  while(parse_common_options(argument_list,info))
    {}
  parse_unknown_options(argument_list,info);

  if(argument_list.size()<2)
    {
      throw arx_error("Need at least two arguments",info);
    }
  else if(argument_list.size()==2)
    /* Moving a file to a file, a file to a directory, or a directory
       to a directory. */
    {
      list<string>::iterator arg(argument_list.begin());
      path source(*arg);
      ++arg;
      path target(*arg);
      atomic_move(source,target,info);
    }
  else
    /* Moving a bunch of files and/or directories into another directory. */
    {
      list<string>::iterator destination=argument_list.end();
      --destination;
      if(!(exists(path(*destination)) && is_directory(path(*destination))))
        {
          throw arx_error("when moving multiple files, last argument must\nbe an existing directory",info);
        }
      else
        {
          for(list<string>::iterator i=argument_list.begin();
              i!=destination;++i)
            {
              atomic_move(path(*i),path(*destination),info);
            }
        }
    }
  return 0;
}
