/* Get or set the naming convention regular expressions for a project tree.

   Copyright (C) 2003 Walter Landry

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include "parse_common_options.hpp"
#include "parse_unknown_options.hpp"
#include "check_extra_args.hpp"
#include "command_initializer.hpp"
#include "arx_error.hpp"
#include "tree_root.hpp"
#include "parse_package_name.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/exception.hpp"
#include "boost/regex.hpp"
#include "Inventory_Regexes.hpp"
#include "Inventory_Method.hpp"
#include <iostream>

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

int naming_convention(list<string> &argument_list, const command &cmd);

static command_initializer naming_convention_init(command("naming-convention",
"print or set the naming conventions for a source tree",
"usage: naming-convention [options] [type [RE]]",
" --dir DIR             look for the project tree in DIR\n\
With no arguments, print all of the regular expresions for the various\n\
inventory types for a project tree.\n\
With one argument, print the regular expression for inventory type TYPE.\n\
With two arguments, record RE as the regular expression to use for inventory\n\
type TYPE.\n\
The inventory types can be one of:\n\
  source, precious, backup, junk, unrecognized",
naming_convention,"Project Tree",true));

int naming_convention(list<string> &argument_list, const command &cmd)
{
  bool set_RE(false);
  Command_Info info(cmd);

  path tree_path(fs::current_path());

  while(!argument_list.empty())
    if(!parse_common_options(argument_list,info))
      {
        list<string>::iterator arg=argument_list.begin();
        if(*arg=="--dir")
          {
            argument_list.pop_front();
            if(argument_list.empty())
              throw arx_error("Need an argument for the --dir option", info);
            tree_path=path(*(argument_list.begin()));
            argument_list.pop_front();
          }
        parse_unknown_options(argument_list,info);
        break;
      }

  string type, new_version;
  if(!argument_list.empty())
    {
      type=*(argument_list.begin());
      argument_list.pop_front();
      if(type!="source" && type!="precious" && type!="backup" && type!="junk"
         && type!="unrecognized")
        throw arx_error("Bad inventory type: " + type,info);
    }

  regex new_RE;
  if(!argument_list.empty())
    {
      try {
        new_RE=*(argument_list.begin());
      }
      catch (bad_expression &bad_RE) {
        throw arx_error ("Bad regular expression " + *(argument_list.begin()),
                         info);
      }
      set_RE=true;
      argument_list.pop_front();
    }
  check_extra_args(argument_list,info);

  path root=tree_root(tree_path,info) / "{arch}";
  if(set_RE)
    {
      Inventory_Regexes inv_regexes;
      Inventory_Method::get(root,inv_regexes,info);

      if(type=="source")
        {
          inv_regexes.source=new_RE;
        }
      else if(type=="precious")
        {
          inv_regexes.precious=new_RE;
        }
      else if(type=="backup")
        {
          inv_regexes.backup=new_RE;
        }
      else if(type=="junk")
        {
          inv_regexes.junk=new_RE;
        }
      else if(type=="unrecognized")
        {
          inv_regexes.unrecognized=new_RE;
        }
      else
        {
          throw arx_error("Bad inventory type: " + type,info);
        }
      Inventory_Method::set(Inventory_Method::get(root,info),inv_regexes,
                            root,info);
    }
  else
    {
      Inventory_Regexes inv_regexes;
      Inventory_Method::get(root,inv_regexes,info);

      if(type.empty())
        {
          cout << "source " << inv_regexes.source.str() << endl;
          cout << "precious " << inv_regexes.precious.str() << endl;
          cout << "backup " << inv_regexes.backup.str() << endl;
          cout << "junk " << inv_regexes.junk.str() << endl;
          cout << "unrecognized " << inv_regexes.unrecognized.str() << endl;
        }
      else
        {
          if(type=="source")
            {
              cout << inv_regexes.source.str();
            }
          else if(type=="precious")
            {
              cout << inv_regexes.precious.str();
            }
          else if(type=="backup")
            {
              cout << inv_regexes.backup.str();
            }
          else if(type=="junk")
            {
              cout << inv_regexes.junk.str();
            }
          else if(type=="unrecognized")
            {
              cout << inv_regexes.unrecognized.str();
            }
          else
            {
              throw arx_error("Bad inventory type: " + type,info);
            }
          cout << endl;
        }
    }
  return 0;
}

