/* Get or set the default version of a project tree.

   Copyright (C) 2001, 2002 Tom Lord
   Copyright (C) 2002, 2003 Walter Landry and the Regents of the
                                  University of California

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include <iostream>
#include "parse_common_options.hpp"
#include "parse_unknown_options.hpp"
#include "check_extra_args.hpp"
#include "command_initializer.hpp"
#include "arx_error.hpp"
#include "tree_root.hpp"
#include "parse_package_name.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/exception.hpp"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

int tree_version(list<string> &argument_list, const command &cmd);

static command_initializer tree_version_init(command("tree-version",
"print or set the default version for a source tree",
"usage: tree-version [options] [[archive]/version]",
" --dir DIR             look for the project tree in DIR\n\
With no argument print the default version for a project tree.\n\
\n\
With an argument, record VERSION as the default version for a project tree",
tree_version,"Project Tree",true));

int tree_version(list<string> &argument_list, const command &cmd)
{
  bool set_version(false);
  Command_Info info(cmd);

  path tree_path(fs::current_path());

  while(!argument_list.empty())
    if(!parse_common_options(argument_list,info))
      {
        list<string>::iterator arg=argument_list.begin();
        if(*arg=="--dir")
          {
            argument_list.pop_front();
            if(argument_list.empty())
              throw arx_error("Need an argument for the --dir option", info);
            tree_path=path(*(argument_list.begin()));
            argument_list.pop_front();
          }
        parse_unknown_options(argument_list,info);
        break;
      }

  string new_version;
  if(!argument_list.empty())
    {
      new_version=*(argument_list.begin());
      set_version=true;
      argument_list.pop_front();
    }
  check_extra_args(argument_list,info);

  path root=tree_root(tree_path,info) / "{arch}";
  if(set_version)
    {
      string archive, category, branch, version, revision;
      parse_package_name(new_version,archive,category,branch,version,revision,
                         info);
      if(version.empty())
        {
          throw arx_error("Version is required: " + new_version,info);
        }
      else
        {
          try
            {
              fs::ofstream version_file(root / "++default-version");
              version_file << new_version << endl;
            }
          catch (fs::filesystem_error &fs_error)
            {
              throw arx_error("Can't open the default version file: "
                              + (root / "++default-version").native_file_string(),info);
            }
        }
    }
  else
    {
      try
        {
          fs::ifstream version_file(root / "++default-version");
          string version;
          if(!getline(version_file,version))
            {
              throw arx_error("Can't read the default version file: "
                              + (root / "++default-version").native_file_string(),info);
            }
          else
            {
              cout << version << endl;
            }
        }
      catch (fs::filesystem_error &fs_error)
        {
          throw arx_error("No default version set for "
                          + fs::current_path().native_file_string(),info);
        }
    }
  return 0;
}

