/* Given a list of archives, categories, branches, and/or versions,
   this program runs the notifier for each category, branch, version,
   and revision in those arguments.  This is useful for automated
   emailing of new versions, automated testing, automated updating,
   etc.

   Copyright 2002, 2003 Walter Landry and the Regents of the
   University of California.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */


#include <string>
#include <list>
#include <fstream>
#include <iostream>

#include "boost/regex.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/exception.hpp"

#include "get_archives.h"
#include "notify_categories.h"
#include "notify_branches.h"
#include "notify_versions.h"
#include "notify_revisions.h"
#include "get_notify_dir.h"
#include "parse_package_name.h"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

int main(int argv, char *argc[])
{

  try {

    /* First, change to the notify directory */
    path notify_directory;
    string notify_parser;
    get_notify_dir(notify_directory, notify_parser);

    /* Open the notify file and grep for the targets.  The file
       "rules" has makefile syntax, so legitimate targets can be found
       with the regular expression
       (^[^:\n]+--(category|branch|version|revision)).  Other targets
       are probably ignored. */

    fs::ifstream rules(notify_directory / "rules");

    /* Remove any output from previous notify runs */
    
    {
      const path rules_out=notify_directory / "rules.out";
      const path rules_err=notify_directory / "rules.err";
      if(fs::exists(rules_out))
        fs::remove(rules_out);
      if(fs::exists(rules_err))
        fs::remove(rules_err);
    }

    string targets, temp_targets;
    match_results<string::const_iterator> m;
    while(getline(rules,temp_targets))
      if(regex_search(temp_targets,m,
		      regex("^[^:\n]+--(category|branch|version|revision):")))
	targets+=temp_targets + "\n";


    /* If no arguments, do all of the notifications for all of the
       archives. */
    if(argv==1)
      {
	list<string> archive_list(get_archives());
	for(list<string>::const_iterator archive=archive_list.begin();
	    archive!=archive_list.end(); ++archive)
	  {
	    bool categories=
	      regex_search(targets,m,regex("^" + (*archive) + "--category:"));
	    bool branches=regex_search(targets,m,regex("^" + (*archive)
						       + "[^:\n]*--branch:"));
	    bool versions=regex_search(targets,m,regex("^" + (*archive)
						       + "[^:\n]*--version:"));
	    bool revisions=
	      regex_search(targets,m,regex("^" + (*archive)
					   + "[^:\n]*--revision:"));

	    if(categories || branches || versions || revisions)
	      notify_categories(notify_parser,notify_directory,targets,
                                *archive,categories,branches,versions,
                                revisions);
	  }
      }
    else
      {
	/* Try to figure out what we have */
	for(int i=1;i<argv;++i)
	  {
	    string archive,category,branch,version,revision;
 	    parse_package_name(argc[i],archive,category,branch,
			       version,revision);
	    if(!revision.empty())
	      throw (string) "Invalid input:" + string(argc[i])
		+ "\nYou may not specify a revision when calling notify";
	    if(!version.empty())
	      {
		notify_revisions(notify_parser,notify_directory,targets,
                                 archive,category,branch,version);
	      }
	    else if(!branch.empty())
	      {
		notify_versions(notify_parser,notify_directory,targets,
                                archive,category,branch,true,true);
	      }
	    else if(!category.empty())
	      {
		notify_branches(notify_parser,notify_directory,targets,
                                archive,category,true,true,true);
	      }
	    else if(!archive.empty())
	      {
		notify_categories(notify_parser,notify_directory,targets,
                                  archive,true,true,true,true);
	      }
	    else
	      {
		throw (string) "Parsing error with the input: "
		  + string(argc[i]) + "\nPlease report this error";
	      }
	  }
      }
  }
  catch(string &message)
    {
      cerr << "notify: " << message << endl;
      exit(1);
    }
  catch(fs::filesystem_error &fs_error)
    {
      cerr << "notify: Boost fs error" << endl;
//       cerr << "notify: Error in " << fs_error.who();
//       if(!fs_error.path1().empty())
//         cerr << "\nwith arguments: " << fs_error.path1().native_file_string()
//              << " " << fs_error.path2().native_file_string();
//       cerr << endl;
      exit(1);
    }

  exit(0);
}
