/* Recursively notifies all of the branches for a particular archive
   and category.

   Copyright 2002, 2003 Walter Landry and the Regents of the
   University of California.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include <string>
#include <list>
#include <algorithm>

#include "boost/regex.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/convenience.hpp"

#include "atomic_append.h"
#include "get_branches.h"
#include "notify_versions.h"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

void notify_branches(const string &notify_parser, const path &notify_directory,
                     const string &targets,
                     const string &archive,
		     const string &category,
		     bool branches, bool versions=false, bool revisions=false)
{
  /* Get a list of the branches that have already been notified */
  /* For the archive */
  list<string> archive_branch_list;
  const path archive_path=notify_directory / archive;
  if(branches)
    {
      fs::ifstream archive_file(archive_path / "branch_list");
      string archive_branch;
      while(getline(archive_file,archive_branch))
	archive_branch_list.push_back(archive_branch);
    }

  /* And for the category */
  list<string> category_branch_list;
  const path category_path=notify_directory / category;
  if(branches)
    {
      fs::ifstream category_file(category_path / "branch_list");
      string category_branch;
      while(getline(category_file,category_branch))
	category_branch_list.push_back(category_branch);
    }

  list<string> branch_list(get_branches(category));
  for(list<string>::const_iterator branch=branch_list.begin();
      branch!=branch_list.end();++branch)
    {
      match_results<string::const_iterator> m;

      string rules=(notify_directory / "rules").string();
      
      string arch_vars=" ARCHARCHIVES=" + archive
        + " ARCHCATEGORIES=" + category
        + " ARCHBRANCHES=" + *branch;

      string notify_begin=notify_parser + " " + rules + arch_vars + " ";
      string notify_end= + "--revision >> " + rules + ".out 2> "
        + rules + ".err";

      /* If we're notifying branches */
      if(branches)
	{
	  /* and there is a suitable archive target */
	  if(regex_search(targets,m,regex("^" + archive + "--branch:"))
	     /* and it hasn't been notified before */
	     && find(archive_branch_list.begin(),archive_branch_list.end(),
		     *branch)==archive_branch_list.end()
	     /* and the notify succeeds */
             && !system((notify_begin + archive + notify_end).c_str()))
	    {
	      /* Then update the notified list. */
              if(!fs::exists(archive_path))
                fs::create_directory(archive_path);
	      atomic_append(archive_path / "branch_list",
			    archive_path / ",,branch_list",*branch);
	    }
	  
	  /* if there is a suitable category target */
	  if(regex_search(targets,m,regex("^" + category + "--branch:"))
	     /* and it hasn't been notified before */
	     && find(category_branch_list.begin(),category_branch_list.end(),
		     *branch)==category_branch_list.end()
	     /* and the notify succeeds */
             && !system((notify_begin + category + notify_end).c_str()))
	    {
	      /* Then update the notified list. */
              if(!fs::exists(category_path))
                fs::create_directories(category_path);
	      atomic_append(category_path / "branch_list",
			    category_path / ",,branch_list", *branch);
	    }
	}

      /* If we are doing versions, or revisions */
      if((versions || revisions)
	 /* And we're doing either this particular branch */
	 && (regex_search(targets,m,regex("^" + (*branch)
					  + "[^:\n]*--(version|revision):"))
	     /* Or everything for this category */
	     || regex_search(targets,m,regex("^" + category
					     + "--(version|revision):"))
	     /* Or everything for this archive */
	     || regex_search(targets,m,regex("^" + archive
					     + "--(version|revision):"))))
	{
	  /* Then notify the versions */
	  notify_versions(notify_parser,notify_directory,targets,archive,
                          category,*branch,versions,revisions);
	}
    }
}
