/* Recursively notifies all of the revisions for a particular archive,
   category, branch, and version.

   Copyright 2002, 2003 Walter Landry and the Regents of the
   University of California.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include <string>
#include <list>
#include <algorithm>

#include "boost/regex.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/convenience.hpp"

#include "atomic_append.h"
#include "get_revisions.h"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;

void notify_revisions(const string &notify_parser,
                      const path &notify_directory, const string &targets,
                      const string &archive,
		      const string &category, const string &branch,
		      const string &version)
{
  /* Get a list of the revisions that have already been notified */
  /* For the archive */
  list<string> archive_revision_list;
  const path archive_path=notify_directory / archive;
  {
    fs::ifstream archive_file(archive_path / "revision_list");
    string archive_revision;
    while(getline(archive_file,archive_revision))
      archive_revision_list.push_back(archive_revision);
  }

  /* and the category */
  list<string> category_revision_list;
  const path category_path=notify_directory / category;
  {
    fs::ifstream category_file(category_path / "revision_list");
    string category_revision;
    while(getline(category_file,category_revision))
      category_revision_list.push_back(category_revision);
  }

  /* and the branch */
  list<string> branch_revision_list;
  const path branch_path=notify_directory / branch;
  {
    fs::ifstream branch_file(branch_path / "revision_list");
    string branch_revision;
    while(getline(branch_file,branch_revision))
      branch_revision_list.push_back(branch_revision);
  }

  /* and the version */
  list<string> version_revision_list;
  const path version_path=notify_directory / version;
  {
    fs::ifstream version_file(version_path / "revision_list");
    string version_revision;
    while(getline(version_file,version_revision))
      version_revision_list.push_back(version_revision);
  }

  list<string> revision_list(get_revisions(version));

  for(list<string>::const_iterator revision=revision_list.begin();
      revision!=revision_list.end();++revision)
    {
      match_results<string::const_iterator> m;

      string rules=(notify_directory / "rules").string();
      
      string arch_vars=" ARCHARCHIVES=" + archive
        + " ARCHCATEGORIES=" + category
        + " ARCHBRANCHES=" + branch
        + " ARCHVERSIONS=" + version
        + " ARCHREVISIONS=" + *revision;

      string notify_begin=notify_parser + " " + rules + arch_vars + " ";
      string notify_end= + "--revision >> " + rules + ".out 2> "
        + rules + ".err";

      /* If there is a suitable archive target */
      if(regex_search(targets,m,regex("^" + archive + "--revision:"))
	 /* and it hasn't been notified before */
	 && find(archive_revision_list.begin(),archive_revision_list.end(),
		 *revision)==archive_revision_list.end()
	 /* and the notify succeeds */
	 && !system((notify_begin + archive + notify_end).c_str()))
	{
	  /* Then update the notified list. */
          if(!fs::exists(archive_path))
            fs::create_directory(archive_path);
	  atomic_append(archive_path / "revision_list",
			archive_path / ",,revision_list", *revision);
	}
      
      /* If there is a suitable category target */
      if(regex_search(targets,m,regex("^" + category + "--revision:"))
	 /* and it hasn't been notified before */
	 && find(category_revision_list.begin(),category_revision_list.end(),
		 *revision)==category_revision_list.end()
	 /* and the notify succeeds */
	 && !system((notify_begin + category + notify_end).c_str()))
	{
	  /* Then update the notified list. */
          if(!fs::exists(category_path))
            fs::create_directories(category_path);
	  atomic_append(category_path / "revision_list",
			category_path / ",,revision_list", *revision);
	}

      /* If there is a suitable branch target */
      if(regex_search(targets,m,regex("^" + branch + "--revision:"))
	 /* and it hasn't been notified before */
	 && find(branch_revision_list.begin(),branch_revision_list.end(),
		 *revision)==branch_revision_list.end()
	 /* and the notify succeeds */
	 && !system((notify_begin + branch + notify_end).c_str()))
	{
	  /* Then update the notified list. */
          if(!fs::exists(branch_path))
            fs::create_directories(branch_path);
	  atomic_append(branch_path / "revision_list",
			branch_path / ",,revision_list", *revision);
	}
      
      /* If there is a suitable version target */
      if(regex_search(targets,m,regex("^" + version + "--revision:"))
	 /* and it hasn't been notified before */
	 && find(version_revision_list.begin(),version_revision_list.end(),
		 *revision)==version_revision_list.end()
	 /* and the notify succeeds */
	 && !system((notify_begin + version + notify_end).c_str()))
	{
	  /* Then update the notified list. */
          if(!fs::exists(version_path))
            fs::create_directories(version_path);
	  atomic_append(version_path / "revision_list",
			version_path / ",,revision_list", *revision);
	}
    }
}
