/* Recursively notifies all of the versions for a particular archive,
   category, and branch.

   Copyright 2002, 2003 Walter Landry and the Regents of the
   University of California.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include <string>
#include <list>
#include <algorithm>

#include "boost/regex.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#include "boost/filesystem/convenience.hpp"

#include "get_versions.h"
#include "atomic_append.h"
#include "notify_revisions.h"

using namespace std;
using namespace boost;
namespace fs=boost::filesystem;
using fs::path;
  
void notify_versions(const string &notify_parser, const path &notify_directory,
                     const string& targets,
                     const string &archive,
		     const string &category, const string &branch,
		     bool versions, bool revisions=false)
{
  /* Get a list of the versions that have already been notified */
  /* For the archive */
  list<string> archive_version_list;
  const path archive_path=notify_directory / archive;
  if(versions)
    {
      fs::ifstream archive_file(archive_path / "version_list");
      string archive_version;
      while(getline(archive_file,archive_version))
	archive_version_list.push_back(archive_version);
    }

  /* and the category */
  list<string> category_version_list;
  const path category_path=notify_directory / category;
  if(versions)
    {
      fs::ifstream category_file(category_path / "version_list");
      string category_version;
      while(getline(category_file,category_version))
	category_version_list.push_back(category_version);
    }

  /* and the branch */
  list<string> branch_version_list;
  const path branch_path=notify_directory / branch;
  if(versions)
    {
      fs::ifstream branch_file(branch_path / "version_list");
      string branch_version;
      while(getline(branch_file,branch_version))
	branch_version_list.push_back(branch_version);
    }

  list<string> version_list(get_versions(branch));
  for(list<string>::const_iterator version=version_list.begin();
      version!=version_list.end();++version)
    {
      match_results<string::const_iterator> m;

      string rules=(notify_directory / "rules").string();
      
      string arch_vars=" ARCHARCHIVES=" + archive
        + " ARCHCATEGORIES=" + category
        + " ARCHBRANCHES=" + branch
        + " ARCHVERSIONS=" + *version;

      string notify_begin=notify_parser + " " + rules + arch_vars + " ";
      string notify_end= + "--revision >> " + rules + ".out 2> "
        + rules + ".err";
      
      /* If we're notifying versions */
      if(versions)
	{
	  /* and there is a suitable archive target */
	  if(regex_search(targets,m,regex("^" + archive + "--version:"))
	     /* and it hasn't been notified before */
	     && find(archive_version_list.begin(),archive_version_list.end(),
		     *version)==archive_version_list.end()
	     /* and the notify succeeds */
             && !system((notify_begin + archive + notify_end).c_str()))
	    {
              if(!fs::exists(archive_path))
                fs::create_directory(archive_path);
	      /* Then update the notified list. */
	      atomic_append(archive_path / "version_list",
			    archive_path / ",,version_list",*version);
	    }

	  
	  /* If there is a suitable category target */
	  if(regex_search(targets,m,regex("^" + category + "--version:"))
	     /* and it hasn't been notified before */
	     && find(category_version_list.begin(),category_version_list.end(),
		     *version)==category_version_list.end()
	     /* and the notify succeeds */
             && !system((notify_begin + category + notify_end).c_str()))
	    {
	      /* Then update the notified list. */
              if(!fs::exists(category_path))
                fs::create_directories(category_path);
	      atomic_append(category_path / "version_list",
			    category_path / ",,version_list", *version);
	    }


	  /* If there is a suitable branch target */
	  if(regex_search(targets,m,regex("^" + branch + "--version:"))
	     /* and it hasn't been notified before */
	     && find(branch_version_list.begin(),branch_version_list.end(),
		     *version)==branch_version_list.end()
	     /* and the notify succeeds */
             && !system((notify_begin + branch + notify_end).c_str()))
	    {
	      /* Then update the notified list. */
              if(!fs::exists(branch_path))
                fs::create_directories(branch_path);
	      atomic_append(branch_path / "version_list",
			    branch_path / ",,version_list", *version);
	    }

	}

      /* If we're doing revisions */
      if(revisions
	 /* And we're doing this version */
	 && (regex_search(targets,m,regex("^" + (*version) + "--revision:"))
	     /* Or everything for this branch */
	     || regex_search(targets,m,regex("^" + branch + "--revision:"))
	     /* Or everything for this category */
	     || regex_search(targets,m,regex("^" + category + "--revision:"))
	     /* Or everything for this archive */
	     || regex_search(targets,m,regex("^" + archive + "--revision:"))))
	{
	  /* Then notify the revisions */
	  notify_revisions(notify_parser,notify_directory,targets,archive,
                           category,branch,*version);
	}
    }
}
