/* Splits up a full_name into it's component parts: archive, category,
   etc.  The category, branch, version, and revision are returned as
   fully qualified names including the archive.

   Copyright 2002, 2003 Walter Landry and the Regents of the
   University of California.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include <string>
#include "boost/regex.hpp"

#include "get_config_option.hpp"
#include "Component_Regexes.hpp"
#include "arx_error.hpp"

using namespace std;
using namespace boost;

void parse_package_name(const string &full_name, string &archive,
			string &category, string &branch, string &version,
			string &revision, const Command_Info &info,
                        const bool &require_archive=false)
{
  Component_Regexes component_regex;
  /* If passed the null string, just return the default archive. */
  if(full_name.empty())
    {
      throw arx_error("Empty name passed to parse_package_name",info);
    }

  /* Get the archive name */
  match_results<string::const_iterator> m;
  string package_name;
  if(regex_search(full_name,m,regex("/")))
    {
      archive=full_name.substr(0,m[-1].length());
      package_name=full_name.substr(m[-1].length()+1);
      if(!regex_match(archive,component_regex.archive))
	throw arx_error("Invalid archive name: " + archive,info);
    }
  else
    {
      if(require_archive)
        {
          throw arx_error("Archive name required: " + full_name,info);
        }
      else
        {
          archive=get_config_option("default-archive",info);
          package_name=full_name;
        }
    }

  /* Get all of the components */
  vector<string> component_list;
  int num_components=regex_split(back_inserter(component_list),package_name,
				 regex("--"));

  if(num_components<1)
    {
      throw arx_error("Bad package name: " + package_name
                      + "\nIt is entirely composed of separators",info);
    }
  else if(regex_match(component_list[0],component_regex.category))
    {
      category=archive + "/" + component_list[0];
    }
  else
    {
      throw arx_error("Invalid category name: " + component_list[0],info);
    }
  int placeholder=0;
  if(num_components>1)
    {
      if(regex_match(component_list[1],component_regex.branch))
	{
	  branch=category + "--" + component_list[1];
	  ++placeholder;
	}
      else
	{
	  branch=category;
	}
    }
  if(num_components>1+placeholder)
    {
      if(regex_match(component_list[1+placeholder],component_regex.version))
	{
	  version=branch + "--" + component_list[1+placeholder];
	}
      else
	{
	  throw arx_error("Bad branch or version name: "
                          + component_list[1+placeholder],info);
	}
    }
  if(num_components>2+placeholder)
    {
      if(regex_match(component_list[2+placeholder],component_regex.revision))
	{
	  revision=version + "--" + component_list[2+placeholder];
	}
      else
	{
	  throw arx_error("Bad revision name: "
                          + component_list[2+placeholder],info);
	}
    }
}
