/* tmp-files.c: 
 *
 ****************************************************************
 * Copyright (C) 2002 Tom Lord
 * Copyright (C) 2003 Environment Canada - Meteorological Service of Canada
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 dated June, 1991.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "hackerlab/os/errno.h"
#include "hackerlab/os/errno-to-string.h"
#include "hackerlab/os/stdlib.h"
#include "hackerlab/os/sys/types.h"
#include "hackerlab/os/unistd.h"
#include "hackerlab/bugs/panic.h"
#include "hackerlab/char/str.h"
#include "hackerlab/fmt/cvt.h"
#include "hackerlab/vu/safe.h"
#include "hackerlab/fs/file-names.h"
#include "hackerlab/fs/tmp-files.h"



char *
tmp_dir (struct alloc_limits * limits)
{
  char *a;

  if ((a = getenv ("TMPDIR")) == NULL)
    a = "/tmp";

  return (char *)str_save (limits, (const t_uchar *)a);
}


int
tmp_open (int * errn,
	  char ** name,
	  struct alloc_limits * limits,
	  char * basename,
	  int flags, int mode)
{
  char * dir;
  char * file;
  int len;

  dir = 0;
  file = 0;
  flags |= (O_CREAT | O_EXCL);

  dir = tmp_dir (limits);
  if (!dir)
    {
    enomem_error:
      *errn = ENOMEM;
    other_error:
      if (dir)
	lim_free (limits, dir);
      if (file)
	lim_free (limits, file);
      return -1;
    }

  file = (char *)file_name_in_vicinity (limits, (t_uchar *)dir, (t_uchar *)basename);
  if (!file)
    goto enomem_error;

  len = str_length ((const t_uchar *)dir) + str_length ((const t_uchar *)basename);

  {
    char * t;
    t = lim_realloc (limits, file, len + 64);
    if (!t)
      goto enomem_error;
    file = t;
  }

  file[len] = '.';
  ++len;
  
  {
    t_ulong seq;

    seq = (t_ulong)getpid ();

    while (1)
      {
	int fd;

	cvt_ulong_to_decimal ((t_uchar *)(file + len), seq);
	fd = vu_open (errn, file, flags, mode);
	if (fd >= 0)
	  {
	    *name = file;
	    lim_free (limits, dir);
	    return fd;
	  }
	if (*errn != EEXIST)
	  goto other_error;
	++seq;
      }
  }
  panic ("tmp-files: not reached!");
  return -1;
}


int
tmp_open_anonymous (int * errn, int flags, int mode)
{
  char * file;
  int fd;
  int ign;

  fd = tmp_open (errn, &file, lim_use_malloc, "anon", flags, mode);
  if (fd < 0)
    return -1;

  if (0 > vu_unlink (&ign, file))
    {
      safe_printfmt (2, "\n");
      safe_printfmt (2, "tmp_open_anonymous: unable to unlink anonymous tmp file\n");
      safe_printfmt (2, "    file: %s\n", file);
      safe_printfmt (2, "    error: %s (%d)\n", errno_to_string (ign), ign);
      safe_printfmt (2, "\n");
      panic ("unable to unlink anonymous tmp file\n");
    }
  lim_free (lim_use_malloc, file);
  
  return fd;
}




/* tag: Tom Lord Fri Feb 22 13:46:41 2002 (tmp-files.c)
 */
