#!/usr/bin/python
#  Small routine to add implicit tags to a project
#
# Copyright (C) 2003 Kristian G. Kvilekval
# Minor Modifications 2003 Walter Landry
#                     and the Regents of the University of California
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import os.path
import re
import getopt
import xreadlines, sys
import string
import stat
from time import ctime
import magic


dryrun=0
excludes  = []
verbose=0

TAGSTRING=r'^.*tag:[ \t].*$'


usage="""usage: arch-tag [-nhvV] [-X rex] [-t tag] [-r][files..dirs]
   -n                  dryrun .. do not change files
   -V                  verbose
   -h                  print this message
   -v                  print version
   -X [regexp]         Exclude any file/dirs matching regexp
   -t tag              Specify tag (default: tag: %n %m)
                       use %n=filename, %p=fullpath,
                           %a=access time ,%m=modtime
   -r                  Restore files from previous rewrite
   """
   

def normalize_path(name):
    from os.path import abspath
    from os.path import expanduser
    from os.path import expandvars

    return abspath(expanduser(expandvars (name)))


class EditableFile:
    def __init__(self, name):
        self.lines = []
        self.oplog = []
        if name:
            self.filename = name
            self.fullpath = normalize_path(name)
            self.filestat = os.stat (self.filename)
            self.open (self.filename)

    def open(self, name):
        try:
            self.filename = name
            self.fullpath = normalize_path(name)
            self.filestat = os.stat (self.filename)
            tfile = open (self.filename)
            self.lines = tfile.readlines()
            tfile.close();
        except IOError:
            print "Unable to open file" + name
    def save(self, path = ""):
        if len(self.lines) > 0:
            if not path: path = self.fullpath
            path = normalize_path(path)

            #readname,readext = os.path.splitext(path)
            os.rename(path, path + ".bak")
            
            tfile = open (path, "w")
            tfile.writelines(self.lines)
            tfile.close();
        return path
    def log(self, str):
        self.oplog.append (str)

    # Edit operation

    def findline(self, key):
        return self.lines.index(key)
    def insertline(self, pos, line):
        return self.lines.insert(pos, line)
    def appendline(self, line):
        return self.lines.append(line)
    def deleteline(self, pos):
        return self.lines.pop(pos)
    def countlines(self):
        return len(self.lines)
    def findre (self, pattern, begin =0, end = -1):
        regexp = re.compile (pattern)
        if end == -1: end = len(self.lines)
        for l in range(begin,end):
            if regexp.match(self.lines[l]):
                return l
        return None



class TaggableFile(EditableFile):
    def tag_file(self, tagline):
        line = self.findre (TAGSTRING)
        if line:
            if verbose:
                print "skipping tagged file "
            return 1
        else:
            tagline = string.replace(tagline, "%p", self.fullpath)
            tagline = string.replace(tagline, "%n", self.filename)
            tagline = string.replace(tagline, "%a", ctime(self.filestat[stat.ST_ATIME]))
            tagline = string.replace(tagline, "%m", ctime(self.filestat[stat.ST_MTIME]))
            self.appendline (tagline)
            if verbose:
                print "adding tag", tagline

            
            

class JavaFile(TaggableFile):
    def __init__(self, filename):
        EditableFile.__init__(self, filename)
#        self.log("opened " + filename)

    def replace_tag (self, tag):
        self.tag_file("/* " + tag + " */")

class XMLFile(TaggableFile):
    def __init__(self, filename):
        EditableFile.__init__(self, filename)

    def replace_tag (self, tag):
        self.tag_file("<!-- " + tag + " -->")


class ShFile(TaggableFile):
    def __init__(self, filename):
        EditableFile.__init__(self, filename)

    def replace_tag (self, tag):
        self.tag_file("### " + tag)
    

class TextFile(TaggableFile):
    def __init__(self, filename):
        EditableFile.__init__(self, filename)

    def replace_tag (self, tag):
        self.tag_file( tag )


class TeXFile(TaggableFile):
    def __init__(self, filename):
        EditableFile.__init__(self, filename)

    def replace_tag (self, tag):
        self.tag_file("%%% "+ tag )



commented_text=[ "application/x-sh",
                 "application/x-csh",
                 "application/x-zsh",
                 "application/x-python",
                 "application/x-perl",
                 "application/x-awk",
                 "text/script",
                 ]


extension_map = { '.c': "C", ".cpp" : "C", ".cxx" : "C",
                  ".java":"C", ".kan": "C", ".css":"C",
                  ".sh" : "SH", ".py" : "SH", ".pl" : "SH",
                  ".xml": "XML", ".html": "XML", ".htm": "XML" }




class Retagger:
    def __init__(self, excludelist, tagline):
        self.excludes = []
        self.tagline = tagline
        self.tagged_files = []
        self.current_dir = None
        for rex in excludelist:
             if verbose: print "excluding: ", rex
             self.excludes.append (re.compile (rex))
            
    def match_excludes(self,name):
        for rex in self.excludes:
            if rex.match (name):
                return 1
        return 0

    def match_tagged(self,name):
        return name in self.tagged_files


    def retag(self, filename):
        if verbose: print filename
        tf = None
        name,ext = os.path.splitext (filename)
        file_type = extension_map.get (ext, None)
        if file_type == "C":
            tf = JavaFile(filename)
        elif file_type == "XML":
            tf = XMLFile(filename)
        elif file_type == "TEX":
            tf = TeXFile(filename)
        elif file_type == "SH":
            tf = ShFile(filename)
        else:
            filetype = magic.file (filename)
            if verbose: print "File %s has type %s" % (filename,filetype)
            if filetype in commented_text:
                tf = ShFile (filename)
            elif filetype.find('text')>=0:
                tf = TextFile (filename)
        if tf:
            tf.replace_tag (self.tagline)
            if not dryrun:
                tf.save()
        else:
            print "Skipping unknown file type", filename

    def dispatch(self, dir, filenames):
        if self.current_dir != dir:
            self.current_dir = dir
            self.tagged_files = []
            tag_dir = dir+'/'+".arch-ids"
            if os.access(tag_dir,os.F_OK):
                for file in os.listdir(tag_dir):
                    tagged_file = os.path.splitext(file)[0]
                    if verbose: print "noting tag for ", tagged_file
                    self.tagged_files.append (tagged_file)
        for filename in filenames:
            if self.match_excludes(filename): continue
            if self.match_tagged (filename): continue
            fullname = dir + '/'+filename
            if self.match_excludes(fullname): continue
            if not os.path.isdir(fullname):
                self.retag (fullname)

    def recurse (self, dirname):
        os.path.walk(dirname, Retagger.dispatch, self)


def restore(dirname):
    """Recurse through the directories replacing
    the new file with the backup files"""
    for f in os.listdir(dirname):
        fname = dirname +'/'+f
        if os.path.isdir(fname):
            print "recursing in ",f
            restore (fname)
        else:
            parts=os.path.splitext(fname)
            if parts[1]==".bak":
                print "renaming %s to %s" % (fname, parts[0])
                os.rename(fname, parts[0])


default_tag = "tag: %n %m"

if __name__ == '__main__':
    excludes.append(".*\.bak$")
    excludes.append(".*~$")
    excludes.append("^=.*$")
    excludes.append(".*\.arch-ids.*")
    options, args =getopt.getopt(sys.argv[1:], 'nhvVX:rt:')
    arg_restore = 0
    for option in options:
        if option[0] == "-n": dryrun = 1
        elif option[0] == "-v": print >>sys.stderr, "arch-tag version 0.9"
        elif option[0] == "-h": print >> sys.stderr, usage
        elif option[0] == "-X": excludes.append(option[1])
        elif option[0] == "-r": arg_restore=1
        elif option[0] == "-V": verbose=1
        elif option[0] == "-t": default_tag = option[1]

    if arg_restore:
        for filename in args:
            restore(filename)
        sys.exit(0)

    if not args:
        print >>sys.stderr, "Please specify at least one filename/dir"
        print >>sys.stderr, usage
        sys.exit(0)

    rp = Retagger(excludes, default_tag)
    for filename in args:
        if os.path.isdir (filename) :
                rp.recurse(filename)
        else:
            rp.retag(filename)
        

    
